package io.netty.buffer;

import io.netty.util.internal.MathUtil;
import io.netty.util.internal.StringUtil;

final class ByteBufUtil$HexUtil
{
  private static final char[] BYTE2CHAR = new char['?'];
  private static final char[] HEXDUMP_TABLE = new char['?'];
  private static final String[] HEXPADDING = new String[16];
  private static final String[] HEXDUMP_ROWPREFIXES = new String['?'];
  private static final String[] BYTE2HEX = new String['?'];
  private static final String[] BYTEPADDING = new String[16];
  
  static
  {
    char[] DIGITS = "0123456789abcdef".toCharArray();
    for (int i = 0; i < 256; i++)
    {
      HEXDUMP_TABLE[(i << 1)] = DIGITS[(i >>> 4 & 0xF)];
      HEXDUMP_TABLE[((i << 1) + 1)] = DIGITS[(i & 0xF)];
    }
    for (int i = 0; i < HEXPADDING.length; i++)
    {
      int padding = HEXPADDING.length - i;
      StringBuilder buf = new StringBuilder(padding * 3);
      for (int j = 0; j < padding; j++) {
        buf.append("   ");
      }
      HEXPADDING[i] = buf.toString();
    }
    for (i = 0; i < HEXDUMP_ROWPREFIXES.length; i++)
    {
      StringBuilder buf = new StringBuilder(12);
      buf.append(StringUtil.NEWLINE);
      buf.append(Long.toHexString(i << 4 & 0xFFFFFFFF | 0x100000000));
      buf.setCharAt(buf.length() - 9, '|');
      buf.append('|');
      HEXDUMP_ROWPREFIXES[i] = buf.toString();
    }
    for (i = 0; i < BYTE2HEX.length; i++) {
      BYTE2HEX[i] = (' ' + StringUtil.byteToHexStringPadded(i));
    }
    for (i = 0; i < BYTEPADDING.length; i++)
    {
      int padding = BYTEPADDING.length - i;
      StringBuilder buf = new StringBuilder(padding);
      for (int j = 0; j < padding; j++) {
        buf.append(' ');
      }
      BYTEPADDING[i] = buf.toString();
    }
    for (i = 0; i < BYTE2CHAR.length; i++) {
      if ((i <= 31) || (i >= 127)) {
        BYTE2CHAR[i] = '.';
      } else {
        BYTE2CHAR[i] = ((char)i);
      }
    }
  }
  
  private static String hexDump(ByteBuf buffer, int fromIndex, int length)
  {
    if (length < 0) {
      throw new IllegalArgumentException("length: " + length);
    }
    if (length == 0) {
      return "";
    }
    int endIndex = fromIndex + length;
    char[] buf = new char[length << 1];
    
    int srcIdx = fromIndex;
    for (int dstIdx = 0; srcIdx < endIndex; dstIdx += 2)
    {
      System.arraycopy(HEXDUMP_TABLE, buffer.getUnsignedByte(srcIdx) << 1, buf, dstIdx, 2);srcIdx++;
    }
    return new String(buf);
  }
  
  private static String hexDump(byte[] array, int fromIndex, int length)
  {
    if (length < 0) {
      throw new IllegalArgumentException("length: " + length);
    }
    if (length == 0) {
      return "";
    }
    int endIndex = fromIndex + length;
    char[] buf = new char[length << 1];
    
    int srcIdx = fromIndex;
    for (int dstIdx = 0; srcIdx < endIndex; dstIdx += 2)
    {
      System.arraycopy(HEXDUMP_TABLE, (array[srcIdx] & 0xFF) << 1, buf, dstIdx, 2);srcIdx++;
    }
    return new String(buf);
  }
  
  private static String prettyHexDump(ByteBuf buffer, int offset, int length)
  {
    if (length == 0) {
      return "";
    }
    int rows = length / 16 + (length % 15 == 0 ? 0 : 1) + 4;
    StringBuilder buf = new StringBuilder(rows * 80);
    appendPrettyHexDump(buf, buffer, offset, length);
    return buf.toString();
  }
  
  private static void appendPrettyHexDump(StringBuilder dump, ByteBuf buf, int offset, int length)
  {
    if (MathUtil.isOutOfBounds(offset, length, buf.capacity())) {
      throw new IndexOutOfBoundsException("expected: 0 <= offset(" + offset + ") <= offset + length(" + length + ") <= " + "buf.capacity(" + buf.capacity() + ')');
    }
    if (length == 0) {
      return;
    }
    dump.append("         +-------------------------------------------------+" + StringUtil.NEWLINE + "         |  0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f |" + StringUtil.NEWLINE + "+--------+-------------------------------------------------+----------------+");
    
    int startIndex = offset;
    int fullRows = length >>> 4;
    int remainder = length & 0xF;
    for (int row = 0; row < fullRows; row++)
    {
      int rowStartIndex = (row << 4) + startIndex;
      
      appendHexDumpRowPrefix(dump, row, rowStartIndex);
      
      int rowEndIndex = rowStartIndex + 16;
      for (int j = rowStartIndex; j < rowEndIndex; j++) {
        dump.append(BYTE2HEX[buf.getUnsignedByte(j)]);
      }
      dump.append(" |");
      for (int j = rowStartIndex; j < rowEndIndex; j++) {
        dump.append(BYTE2CHAR[buf.getUnsignedByte(j)]);
      }
      dump.append('|');
    }
    if (remainder != 0)
    {
      int rowStartIndex = (fullRows << 4) + startIndex;
      appendHexDumpRowPrefix(dump, fullRows, rowStartIndex);
      
      int rowEndIndex = rowStartIndex + remainder;
      for (int j = rowStartIndex; j < rowEndIndex; j++) {
        dump.append(BYTE2HEX[buf.getUnsignedByte(j)]);
      }
      dump.append(HEXPADDING[remainder]);
      dump.append(" |");
      for (int j = rowStartIndex; j < rowEndIndex; j++) {
        dump.append(BYTE2CHAR[buf.getUnsignedByte(j)]);
      }
      dump.append(BYTEPADDING[remainder]);
      dump.append('|');
    }
    dump.append(StringUtil.NEWLINE + "+--------+-------------------------------------------------+----------------+");
  }
  
  private static void appendHexDumpRowPrefix(StringBuilder dump, int row, int rowStartIndex)
  {
    if (row < HEXDUMP_ROWPREFIXES.length)
    {
      dump.append(HEXDUMP_ROWPREFIXES[row]);
    }
    else
    {
      dump.append(StringUtil.NEWLINE);
      dump.append(Long.toHexString(rowStartIndex & 0xFFFFFFFF | 0x100000000));
      dump.setCharAt(dump.length() - 9, '|');
      dump.append('|');
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.buffer.ByteBufUtil.HexUtil
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */