package io.netty.channel;

import io.netty.util.ReferenceCountUtil;
import io.netty.util.internal.PlatformDependent;
import io.netty.util.internal.logging.InternalLogger;
import java.io.IOException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NoRouteToHostException;
import java.net.SocketAddress;
import java.net.SocketException;
import java.nio.channels.ClosedChannelException;
import java.util.concurrent.Executor;
import java.util.concurrent.RejectedExecutionException;

public abstract class AbstractChannel$AbstractUnsafe
  implements Channel.Unsafe
{
  private volatile ChannelOutboundBuffer outboundBuffer = new ChannelOutboundBuffer(this$0);
  private RecvByteBufAllocator.Handle recvHandle;
  private boolean inFlush0;
  private boolean neverRegistered = true;
  
  protected AbstractChannel$AbstractUnsafe(AbstractChannel paramAbstractChannel) {}
  
  private void assertEventLoop()
  {
    assert ((!AbstractChannel.access$000(this$0)) || (AbstractChannel.access$100(this$0).inEventLoop()));
  }
  
  public RecvByteBufAllocator.Handle recvBufAllocHandle()
  {
    if (recvHandle == null) {
      recvHandle = this$0.config().getRecvByteBufAllocator().newHandle();
    }
    return recvHandle;
  }
  
  public final ChannelOutboundBuffer outboundBuffer()
  {
    return outboundBuffer;
  }
  
  public final SocketAddress localAddress()
  {
    return this$0.localAddress0();
  }
  
  public final SocketAddress remoteAddress()
  {
    return this$0.remoteAddress0();
  }
  
  public final void register(EventLoop eventLoop, final ChannelPromise promise)
  {
    if (eventLoop == null) {
      throw new NullPointerException("eventLoop");
    }
    if (this$0.isRegistered())
    {
      promise.setFailure(new IllegalStateException("registered to an event loop already"));
      return;
    }
    if (!this$0.isCompatible(eventLoop))
    {
      promise.setFailure(new IllegalStateException("incompatible event loop type: " + eventLoop.getClass().getName()));
      
      return;
    }
    AbstractChannel.access$102(this$0, eventLoop);
    if (eventLoop.inEventLoop()) {
      register0(promise);
    } else {
      try
      {
        eventLoop.execute(new Runnable()
        {
          public void run()
          {
            AbstractChannel.AbstractUnsafe.this.register0(promise);
          }
        });
      }
      catch (Throwable t)
      {
        AbstractChannel.access$300().warn("Force-closing a channel whose registration task was not accepted by an event loop: {}", this$0, t);
        
        closeForcibly();
        AbstractChannel.access$400(this$0).setClosed();
        safeSetFailure(promise, t);
      }
    }
  }
  
  private void register0(ChannelPromise promise)
  {
    try
    {
      if ((!promise.setUncancellable()) || (!ensureOpen(promise))) {
        return;
      }
      boolean firstRegistration = neverRegistered;
      this$0.doRegister();
      neverRegistered = false;
      AbstractChannel.access$002(this$0, true);
      
      AbstractChannel.access$500(this$0).invokeHandlerAddedIfNeeded();
      
      safeSetSuccess(promise);
      AbstractChannel.access$500(this$0).fireChannelRegistered();
      if (this$0.isActive()) {
        if (firstRegistration) {
          AbstractChannel.access$500(this$0).fireChannelActive();
        } else if (this$0.config().isAutoRead()) {
          beginRead();
        }
      }
    }
    catch (Throwable t)
    {
      closeForcibly();
      AbstractChannel.access$400(this$0).setClosed();
      safeSetFailure(promise, t);
    }
  }
  
  public final void bind(SocketAddress localAddress, ChannelPromise promise)
  {
    assertEventLoop();
    if ((!promise.setUncancellable()) || (!ensureOpen(promise))) {
      return;
    }
    if ((Boolean.TRUE.equals(this$0.config().getOption(ChannelOption.SO_BROADCAST))) && ((localAddress instanceof InetSocketAddress)) && (!((InetSocketAddress)localAddress).getAddress().isAnyLocalAddress()) && (!PlatformDependent.isWindows()) && (!PlatformDependent.isRoot())) {
      AbstractChannel.access$300().warn("A non-root user can't receive a broadcast packet if the socket is not bound to a wildcard address; binding to a non-wildcard address (" + localAddress + ") anyway as requested.");
    }
    boolean wasActive = this$0.isActive();
    try
    {
      this$0.doBind(localAddress);
    }
    catch (Throwable t)
    {
      safeSetFailure(promise, t);
      closeIfClosed();
      return;
    }
    if ((!wasActive) && (this$0.isActive())) {
      invokeLater(new Runnable()
      {
        public void run()
        {
          AbstractChannel.access$500(this$0).fireChannelActive();
        }
      });
    }
    safeSetSuccess(promise);
  }
  
  public final void disconnect(ChannelPromise promise)
  {
    assertEventLoop();
    if (!promise.setUncancellable()) {
      return;
    }
    boolean wasActive = this$0.isActive();
    try
    {
      this$0.doDisconnect();
    }
    catch (Throwable t)
    {
      safeSetFailure(promise, t);
      closeIfClosed();
      return;
    }
    if ((wasActive) && (!this$0.isActive())) {
      invokeLater(new Runnable()
      {
        public void run()
        {
          AbstractChannel.access$500(this$0).fireChannelInactive();
        }
      });
    }
    safeSetSuccess(promise);
    closeIfClosed();
  }
  
  public final void close(ChannelPromise promise)
  {
    assertEventLoop();
    
    close(promise, AbstractChannel.access$600(), AbstractChannel.access$600(), false);
  }
  
  private void close(final ChannelPromise promise, final Throwable cause, final ClosedChannelException closeCause, final boolean notify)
  {
    if (!promise.setUncancellable()) {
      return;
    }
    final ChannelOutboundBuffer outboundBuffer = this.outboundBuffer;
    if (outboundBuffer == null)
    {
      if (!(promise instanceof VoidChannelPromise)) {
        AbstractChannel.access$400(this$0).addListener(new ChannelFutureListener()
        {
          public void operationComplete(ChannelFuture future)
            throws Exception
          {
            promise.setSuccess();
          }
        });
      }
      return;
    }
    if (AbstractChannel.access$400(this$0).isDone())
    {
      safeSetSuccess(promise);
      return;
    }
    final boolean wasActive = this$0.isActive();
    this.outboundBuffer = null;
    Executor closeExecutor = prepareToClose();
    if (closeExecutor != null)
    {
      closeExecutor.execute(new Runnable()
      {
        public void run()
        {
          try
          {
            AbstractChannel.AbstractUnsafe.this.doClose0(promise);
          }
          finally
          {
            AbstractChannel.AbstractUnsafe.this.invokeLater(new Runnable()
            {
              public void run()
              {
                val$outboundBuffer.failFlushed(val$cause, val$notify);
                val$outboundBuffer.close(val$closeCause);
                AbstractChannel.AbstractUnsafe.this.fireChannelInactiveAndDeregister(val$wasActive);
              }
            });
          }
        }
      });
    }
    else
    {
      try
      {
        doClose0(promise);
      }
      finally
      {
        outboundBuffer.failFlushed(cause, notify);
        outboundBuffer.close(closeCause);
      }
      if (inFlush0) {
        invokeLater(new Runnable()
        {
          public void run()
          {
            AbstractChannel.AbstractUnsafe.this.fireChannelInactiveAndDeregister(wasActive);
          }
        });
      } else {
        fireChannelInactiveAndDeregister(wasActive);
      }
    }
  }
  
  private void doClose0(ChannelPromise promise)
  {
    try
    {
      this$0.doClose();
      AbstractChannel.access$400(this$0).setClosed();
      safeSetSuccess(promise);
    }
    catch (Throwable t)
    {
      AbstractChannel.access$400(this$0).setClosed();
      safeSetFailure(promise, t);
    }
  }
  
  private void fireChannelInactiveAndDeregister(boolean wasActive)
  {
    deregister(voidPromise(), (wasActive) && (!this$0.isActive()));
  }
  
  public final void closeForcibly()
  {
    assertEventLoop();
    try
    {
      this$0.doClose();
    }
    catch (Exception e)
    {
      AbstractChannel.access$300().warn("Failed to close a channel.", e);
    }
  }
  
  public final void deregister(ChannelPromise promise)
  {
    assertEventLoop();
    
    deregister(promise, false);
  }
  
  private void deregister(final ChannelPromise promise, final boolean fireChannelInactive)
  {
    if (!promise.setUncancellable()) {
      return;
    }
    if (!AbstractChannel.access$000(this$0))
    {
      safeSetSuccess(promise);
      return;
    }
    invokeLater(new Runnable()
    {
      public void run()
      {
        try
        {
          this$0.doDeregister();
        }
        catch (Throwable t)
        {
          AbstractChannel.access$300().warn("Unexpected exception occurred while deregistering a channel.", t);
        }
        finally
        {
          if (fireChannelInactive) {
            AbstractChannel.access$500(this$0).fireChannelInactive();
          }
          if (AbstractChannel.access$000(this$0))
          {
            AbstractChannel.access$002(this$0, false);
            AbstractChannel.access$500(this$0).fireChannelUnregistered();
          }
          safeSetSuccess(promise);
        }
      }
    });
  }
  
  public final void beginRead()
  {
    assertEventLoop();
    if (!this$0.isActive()) {
      return;
    }
    try
    {
      this$0.doBeginRead();
    }
    catch (Exception e)
    {
      invokeLater(new Runnable()
      {
        public void run()
        {
          AbstractChannel.access$500(this$0).fireExceptionCaught(e);
        }
      });
      close(voidPromise());
    }
  }
  
  public final void write(Object msg, ChannelPromise promise)
  {
    assertEventLoop();
    
    ChannelOutboundBuffer outboundBuffer = this.outboundBuffer;
    if (outboundBuffer == null)
    {
      safeSetFailure(promise, AbstractChannel.access$1000());
      
      ReferenceCountUtil.release(msg);
      return;
    }
    try
    {
      msg = this$0.filterOutboundMessage(msg);
      int size = AbstractChannel.access$500(this$0).estimatorHandle().size(msg);
      if (size < 0) {
        size = 0;
      }
    }
    catch (Throwable t)
    {
      safeSetFailure(promise, t);
      ReferenceCountUtil.release(msg); return;
    }
    int size;
    outboundBuffer.addMessage(msg, size, promise);
  }
  
  public final void flush()
  {
    assertEventLoop();
    
    ChannelOutboundBuffer outboundBuffer = this.outboundBuffer;
    if (outboundBuffer == null) {
      return;
    }
    outboundBuffer.addFlush();
    flush0();
  }
  
  protected void flush0()
  {
    if (inFlush0) {
      return;
    }
    ChannelOutboundBuffer outboundBuffer = this.outboundBuffer;
    if ((outboundBuffer == null) || (outboundBuffer.isEmpty())) {
      return;
    }
    inFlush0 = true;
    if (!this$0.isActive())
    {
      try
      {
        if (this$0.isOpen()) {
          outboundBuffer.failFlushed(AbstractChannel.access$1100(), true);
        } else {
          outboundBuffer.failFlushed(AbstractChannel.access$1200(), false);
        }
      }
      finally
      {
        inFlush0 = false;
      }
      return;
    }
    try
    {
      this$0.doWrite(outboundBuffer);
    }
    catch (Throwable t)
    {
      if (((t instanceof IOException)) && (this$0.config().isAutoClose())) {
        close(voidPromise(), t, AbstractChannel.access$1200(), false);
      } else {
        outboundBuffer.failFlushed(t, true);
      }
    }
    finally
    {
      inFlush0 = false;
    }
  }
  
  public final ChannelPromise voidPromise()
  {
    assertEventLoop();
    
    return AbstractChannel.access$1300(this$0);
  }
  
  @Deprecated
  protected final boolean ensureOpen(ChannelPromise promise)
  {
    if (this$0.isOpen()) {
      return true;
    }
    safeSetFailure(promise, AbstractChannel.access$1400());
    return false;
  }
  
  protected final void safeSetSuccess(ChannelPromise promise)
  {
    if ((!(promise instanceof VoidChannelPromise)) && (!promise.trySuccess())) {
      AbstractChannel.access$300().warn("Failed to mark a promise as success because it is done already: {}", promise);
    }
  }
  
  protected final void safeSetFailure(ChannelPromise promise, Throwable cause)
  {
    if ((!(promise instanceof VoidChannelPromise)) && (!promise.tryFailure(cause))) {
      AbstractChannel.access$300().warn("Failed to mark a promise as failure because it's done already: {}", promise, cause);
    }
  }
  
  protected final void closeIfClosed()
  {
    if (this$0.isOpen()) {
      return;
    }
    close(voidPromise());
  }
  
  private void invokeLater(Runnable task)
  {
    try
    {
      this$0.eventLoop().execute(task);
    }
    catch (RejectedExecutionException e)
    {
      AbstractChannel.access$300().warn("Can't invoke task later as EventLoop rejected it", e);
    }
  }
  
  protected final Throwable annotateConnectException(Throwable cause, SocketAddress remoteAddress)
  {
    if ((cause instanceof ConnectException)) {
      return new AbstractChannel.AnnotatedConnectException((ConnectException)cause, remoteAddress);
    }
    if ((cause instanceof NoRouteToHostException)) {
      return new AbstractChannel.AnnotatedNoRouteToHostException((NoRouteToHostException)cause, remoteAddress);
    }
    if ((cause instanceof SocketException)) {
      return new AbstractChannel.AnnotatedSocketException((SocketException)cause, remoteAddress);
    }
    return cause;
  }
  
  protected Executor prepareToClose()
  {
    return null;
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.AbstractChannel.AbstractUnsafe
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */