package io.netty.channel;

import io.netty.buffer.ByteBufAllocator;
import io.netty.util.Attribute;
import io.netty.util.AttributeKey;
import io.netty.util.concurrent.EventExecutor;
import java.net.SocketAddress;

class CombinedChannelDuplexHandler$DelegatingChannelHandlerContext
  implements ChannelHandlerContext
{
  private final ChannelHandlerContext ctx;
  private final ChannelHandler handler;
  boolean removed;
  
  CombinedChannelDuplexHandler$DelegatingChannelHandlerContext(ChannelHandlerContext ctx, ChannelHandler handler)
  {
    this.ctx = ctx;
    this.handler = handler;
  }
  
  public Channel channel()
  {
    return ctx.channel();
  }
  
  public EventExecutor executor()
  {
    return ctx.executor();
  }
  
  public String name()
  {
    return ctx.name();
  }
  
  public ChannelHandler handler()
  {
    return ctx.handler();
  }
  
  public boolean isRemoved()
  {
    return (removed) || (ctx.isRemoved());
  }
  
  public ChannelHandlerContext fireChannelRegistered()
  {
    ctx.fireChannelRegistered();
    return this;
  }
  
  public ChannelHandlerContext fireChannelUnregistered()
  {
    ctx.fireChannelUnregistered();
    return this;
  }
  
  public ChannelHandlerContext fireChannelActive()
  {
    ctx.fireChannelActive();
    return this;
  }
  
  public ChannelHandlerContext fireChannelInactive()
  {
    ctx.fireChannelInactive();
    return this;
  }
  
  public ChannelHandlerContext fireExceptionCaught(Throwable cause)
  {
    ctx.fireExceptionCaught(cause);
    return this;
  }
  
  public ChannelHandlerContext fireUserEventTriggered(Object event)
  {
    ctx.fireUserEventTriggered(event);
    return this;
  }
  
  public ChannelHandlerContext fireChannelRead(Object msg)
  {
    ctx.fireChannelRead(msg);
    return this;
  }
  
  public ChannelHandlerContext fireChannelReadComplete()
  {
    ctx.fireChannelReadComplete();
    return this;
  }
  
  public ChannelHandlerContext fireChannelWritabilityChanged()
  {
    ctx.fireChannelWritabilityChanged();
    return this;
  }
  
  public ChannelFuture bind(SocketAddress localAddress)
  {
    return ctx.bind(localAddress);
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress)
  {
    return ctx.connect(remoteAddress);
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress, SocketAddress localAddress)
  {
    return ctx.connect(remoteAddress, localAddress);
  }
  
  public ChannelFuture disconnect()
  {
    return ctx.disconnect();
  }
  
  public ChannelFuture close()
  {
    return ctx.close();
  }
  
  public ChannelFuture deregister()
  {
    return ctx.deregister();
  }
  
  public ChannelFuture bind(SocketAddress localAddress, ChannelPromise promise)
  {
    return ctx.bind(localAddress, promise);
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress, ChannelPromise promise)
  {
    return ctx.connect(remoteAddress, promise);
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress, SocketAddress localAddress, ChannelPromise promise)
  {
    return ctx.connect(remoteAddress, localAddress, promise);
  }
  
  public ChannelFuture disconnect(ChannelPromise promise)
  {
    return ctx.disconnect(promise);
  }
  
  public ChannelFuture close(ChannelPromise promise)
  {
    return ctx.close(promise);
  }
  
  public ChannelFuture deregister(ChannelPromise promise)
  {
    return ctx.deregister(promise);
  }
  
  public ChannelHandlerContext read()
  {
    ctx.read();
    return this;
  }
  
  public ChannelFuture write(Object msg)
  {
    return ctx.write(msg);
  }
  
  public ChannelFuture write(Object msg, ChannelPromise promise)
  {
    return ctx.write(msg, promise);
  }
  
  public ChannelHandlerContext flush()
  {
    ctx.flush();
    return this;
  }
  
  public ChannelFuture writeAndFlush(Object msg, ChannelPromise promise)
  {
    return ctx.writeAndFlush(msg, promise);
  }
  
  public ChannelFuture writeAndFlush(Object msg)
  {
    return ctx.writeAndFlush(msg);
  }
  
  public ChannelPipeline pipeline()
  {
    return ctx.pipeline();
  }
  
  public ByteBufAllocator alloc()
  {
    return ctx.alloc();
  }
  
  public ChannelPromise newPromise()
  {
    return ctx.newPromise();
  }
  
  public ChannelProgressivePromise newProgressivePromise()
  {
    return ctx.newProgressivePromise();
  }
  
  public ChannelFuture newSucceededFuture()
  {
    return ctx.newSucceededFuture();
  }
  
  public ChannelFuture newFailedFuture(Throwable cause)
  {
    return ctx.newFailedFuture(cause);
  }
  
  public ChannelPromise voidPromise()
  {
    return ctx.voidPromise();
  }
  
  public <T> Attribute<T> attr(AttributeKey<T> key)
  {
    return ctx.attr(key);
  }
  
  public <T> boolean hasAttr(AttributeKey<T> key)
  {
    return ctx.hasAttr(key);
  }
  
  final void remove()
  {
    EventExecutor executor = executor();
    if (executor.inEventLoop()) {
      remove0();
    } else {
      executor.execute(new Runnable()
      {
        public void run()
        {
          CombinedChannelDuplexHandler.DelegatingChannelHandlerContext.this.remove0();
        }
      });
    }
  }
  
  private void remove0()
  {
    if (!removed)
    {
      removed = true;
      try
      {
        handler.handlerRemoved(this);
      }
      catch (Throwable cause)
      {
        fireExceptionCaught(new ChannelPipelineException(handler.getClass().getName() + ".handlerRemoved() has thrown an exception.", cause));
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.CombinedChannelDuplexHandler.DelegatingChannelHandlerContext
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */