package io.netty.channel.epoll;

import io.netty.channel.AbstractChannel.AbstractUnsafe;
import io.netty.channel.Channel.Unsafe;
import io.netty.channel.ChannelConfig;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoop;
import io.netty.channel.RecvByteBufAllocator.Handle;
import io.netty.channel.socket.ChannelInputShutdownEvent;
import io.netty.channel.unix.Socket;
import java.io.IOException;
import java.nio.channels.NotYetConnectedException;

public abstract class AbstractEpollChannel$AbstractEpollUnsafe
  extends AbstractChannel.AbstractUnsafe
{
  boolean readPending;
  boolean maybeMoreDataToRead;
  boolean epollInReadyRunnablePending;
  private EpollRecvByteAllocatorHandle allocHandle;
  private Runnable epollInReadyRunnable;
  
  protected AbstractEpollChannel$AbstractEpollUnsafe(AbstractEpollChannel paramAbstractEpollChannel)
  {
    super(paramAbstractEpollChannel);
  }
  
  abstract void epollInReady();
  
  final void epollInBefore()
  {
    maybeMoreDataToRead = false;
  }
  
  final void epollInFinally(ChannelConfig config)
  {
    maybeMoreDataToRead = allocHandle.maybeMoreDataToRead();
    if ((!readPending) && (!config.isAutoRead())) {
      this$0.clearEpollIn();
    } else if ((readPending) && (maybeMoreDataToRead) && (!this$0.fd().isInputShutdown())) {
      executeEpollInReadyRunnable();
    }
  }
  
  final void executeEpollInReadyRunnable()
  {
    if (epollInReadyRunnablePending) {
      return;
    }
    epollInReadyRunnablePending = true;
    if (epollInReadyRunnable == null) {
      epollInReadyRunnable = new Runnable()
      {
        public void run()
        {
          epollInReadyRunnablePending = false;
          epollInReady();
        }
      };
    }
    this$0.eventLoop().execute(epollInReadyRunnable);
  }
  
  final void epollRdHupReady()
  {
    recvBufAllocHandle().receivedRdHup();
    if (this$0.isActive())
    {
      epollInReady();
      
      clearEpollRdHup();
    }
    shutdownInput();
  }
  
  private void clearEpollRdHup()
  {
    try
    {
      this$0.clearFlag(Native.EPOLLRDHUP);
    }
    catch (IOException e)
    {
      this$0.pipeline().fireExceptionCaught(e);
      close(voidPromise());
    }
  }
  
  void shutdownInput()
  {
    if (!this$0.fd().isInputShutdown()) {
      if (Boolean.TRUE.equals(this$0.config().getOption(ChannelOption.ALLOW_HALF_CLOSURE)))
      {
        try
        {
          this$0.fd().shutdown(true, false);
          clearEpollIn0();
        }
        catch (IOException ignored)
        {
          fireEventAndClose(ChannelInputShutdownEvent.INSTANCE);
          return;
        }
        catch (NotYetConnectedException ignore)
        {
          fireEventAndClose(ChannelInputShutdownEvent.INSTANCE);
          return;
        }
        this$0.pipeline().fireUserEventTriggered(ChannelInputShutdownEvent.INSTANCE);
      }
      else
      {
        close(voidPromise());
      }
    }
  }
  
  private void fireEventAndClose(Object evt)
  {
    this$0.pipeline().fireUserEventTriggered(evt);
    close(voidPromise());
  }
  
  public EpollRecvByteAllocatorHandle recvBufAllocHandle()
  {
    if (allocHandle == null) {
      allocHandle = newEpollHandle(super.recvBufAllocHandle());
    }
    return allocHandle;
  }
  
  EpollRecvByteAllocatorHandle newEpollHandle(RecvByteBufAllocator.Handle handle)
  {
    return new EpollRecvByteAllocatorHandle(handle, this$0.config());
  }
  
  protected void flush0()
  {
    if (this$0.isFlagSet(Native.EPOLLOUT)) {
      return;
    }
    super.flush0();
  }
  
  void epollOutReady()
  {
    if (this$0.fd().isOutputShutdown()) {
      return;
    }
    super.flush0();
  }
  
  protected final void clearEpollIn0()
  {
    assert (this$0.eventLoop().inEventLoop());
    try
    {
      readPending = false;
      this$0.clearFlag(AbstractEpollChannel.access$000(this$0));
    }
    catch (IOException e)
    {
      this$0.pipeline().fireExceptionCaught(e);
      this$0.unsafe().close(this$0.unsafe().voidPromise());
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.epoll.AbstractEpollChannel.AbstractEpollUnsafe
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */