package io.netty.channel.epoll;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import io.netty.channel.ChannelConfig;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.ChannelPromise;
import io.netty.channel.ConnectTimeoutException;
import io.netty.channel.EventLoop;
import io.netty.channel.RecvByteBufAllocator.Handle;
import io.netty.channel.unix.Socket;
import java.io.IOException;
import java.net.SocketAddress;
import java.nio.channels.ConnectionPendingException;
import java.util.Queue;
import java.util.concurrent.Executor;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

class AbstractEpollStreamChannel$EpollStreamUnsafe
  extends AbstractEpollChannel.AbstractEpollUnsafe
{
  AbstractEpollStreamChannel$EpollStreamUnsafe(AbstractEpollStreamChannel paramAbstractEpollStreamChannel)
  {
    super(paramAbstractEpollStreamChannel);
  }
  
  protected Executor prepareToClose()
  {
    return super.prepareToClose();
  }
  
  private void handleReadException(ChannelPipeline pipeline, ByteBuf byteBuf, Throwable cause, boolean close, EpollRecvByteAllocatorHandle allocHandle)
  {
    if (byteBuf != null) {
      if (byteBuf.isReadable())
      {
        readPending = false;
        pipeline.fireChannelRead(byteBuf);
      }
      else
      {
        byteBuf.release();
      }
    }
    allocHandle.readComplete();
    pipeline.fireChannelReadComplete();
    pipeline.fireExceptionCaught(cause);
    if ((close) || ((cause instanceof IOException))) {
      shutdownInput();
    }
  }
  
  public void connect(final SocketAddress remoteAddress, SocketAddress localAddress, ChannelPromise promise)
  {
    if ((!promise.setUncancellable()) || (!ensureOpen(promise))) {
      return;
    }
    try
    {
      if (AbstractEpollStreamChannel.access$400(this$0) != null) {
        throw new ConnectionPendingException();
      }
      boolean wasActive = this$0.isActive();
      if (this$0.doConnect(remoteAddress, localAddress))
      {
        fulfillConnectPromise(promise, wasActive);
      }
      else
      {
        AbstractEpollStreamChannel.access$402(this$0, promise);
        AbstractEpollStreamChannel.access$502(this$0, remoteAddress);
        
        int connectTimeoutMillis = this$0.config().getConnectTimeoutMillis();
        if (connectTimeoutMillis > 0) {
          AbstractEpollStreamChannel.access$602(this$0, this$0.eventLoop().schedule(new Runnable()
          {
            public void run()
            {
              ChannelPromise connectPromise = AbstractEpollStreamChannel.access$400(this$0);
              ConnectTimeoutException cause = new ConnectTimeoutException("connection timed out: " + remoteAddress);
              if ((connectPromise != null) && (connectPromise.tryFailure(cause))) {
                close(voidPromise());
              }
            }
          }, connectTimeoutMillis, TimeUnit.MILLISECONDS));
        }
        promise.addListener(new ChannelFutureListener()
        {
          public void operationComplete(ChannelFuture future)
            throws Exception
          {
            if (future.isCancelled())
            {
              if (AbstractEpollStreamChannel.access$600(this$0) != null) {
                AbstractEpollStreamChannel.access$600(this$0).cancel(false);
              }
              AbstractEpollStreamChannel.access$402(this$0, null);
              close(voidPromise());
            }
          }
        });
      }
    }
    catch (Throwable t)
    {
      closeIfClosed();
      promise.tryFailure(annotateConnectException(t, remoteAddress));
    }
  }
  
  private void fulfillConnectPromise(ChannelPromise promise, boolean wasActive)
  {
    if (promise == null) {
      return;
    }
    this$0.active = true;
    
    boolean active = this$0.isActive();
    
    boolean promiseSet = promise.trySuccess();
    if ((!wasActive) && (active)) {
      this$0.pipeline().fireChannelActive();
    }
    if (!promiseSet) {
      close(voidPromise());
    }
  }
  
  private void fulfillConnectPromise(ChannelPromise promise, Throwable cause)
  {
    if (promise == null) {
      return;
    }
    promise.tryFailure(cause);
    closeIfClosed();
  }
  
  private void finishConnect()
  {
    assert (this$0.eventLoop().inEventLoop());
    
    boolean connectStillInProgress = false;
    try
    {
      boolean wasActive = this$0.isActive();
      if (!doFinishConnect()) {
        connectStillInProgress = true;
      } else {
        fulfillConnectPromise(AbstractEpollStreamChannel.access$400(this$0), wasActive);
      }
    }
    catch (Throwable t)
    {
      fulfillConnectPromise(AbstractEpollStreamChannel.access$400(this$0), annotateConnectException(t, AbstractEpollStreamChannel.access$500(this$0)));
    }
    finally
    {
      if (!connectStillInProgress)
      {
        if (AbstractEpollStreamChannel.access$600(this$0) != null) {
          AbstractEpollStreamChannel.access$600(this$0).cancel(false);
        }
        AbstractEpollStreamChannel.access$402(this$0, null);
      }
    }
  }
  
  void epollOutReady()
  {
    if (AbstractEpollStreamChannel.access$400(this$0) != null) {
      finishConnect();
    } else {
      super.epollOutReady();
    }
  }
  
  boolean doFinishConnect()
    throws Exception
  {
    if (this$0.fd().finishConnect())
    {
      this$0.clearFlag(Native.EPOLLOUT);
      return true;
    }
    this$0.setFlag(Native.EPOLLOUT);
    return false;
  }
  
  EpollRecvByteAllocatorHandle newEpollHandle(RecvByteBufAllocator.Handle handle)
  {
    return new EpollRecvByteAllocatorStreamingHandle(handle, this$0.config());
  }
  
  void epollInReady()
  {
    if (this$0.fd().isInputShutdown())
    {
      clearEpollIn0();
      return;
    }
    ChannelConfig config = this$0.config();
    EpollRecvByteAllocatorHandle allocHandle = recvBufAllocHandle();
    allocHandle.edgeTriggered(this$0.isFlagSet(Native.EPOLLET));
    
    ChannelPipeline pipeline = this$0.pipeline();
    ByteBufAllocator allocator = config.getAllocator();
    allocHandle.reset(config);
    epollInBefore();
    
    ByteBuf byteBuf = null;
    boolean close = false;
    try
    {
      do
      {
        if (AbstractEpollStreamChannel.access$700(this$0) != null)
        {
          AbstractEpollStreamChannel.SpliceInTask spliceTask = (AbstractEpollStreamChannel.SpliceInTask)AbstractEpollStreamChannel.access$700(this$0).peek();
          if (spliceTask != null)
          {
            if (!spliceTask.spliceIn(allocHandle)) {
              break;
            }
            if (!this$0.isActive()) {
              continue;
            }
            AbstractEpollStreamChannel.access$700(this$0).remove(); continue;
          }
        }
        byteBuf = allocHandle.allocate(allocator);
        allocHandle.lastBytesRead(this$0.doReadBytes(byteBuf));
        if (allocHandle.lastBytesRead() <= 0)
        {
          byteBuf.release();
          byteBuf = null;
          close = allocHandle.lastBytesRead() < 0;
        }
        else
        {
          allocHandle.incMessagesRead(1);
          readPending = false;
          pipeline.fireChannelRead(byteBuf);
          byteBuf = null;
          if (this$0.fd().isInputShutdown()) {
            break;
          }
        }
      } while (allocHandle.continueReading());
      allocHandle.readComplete();
      pipeline.fireChannelReadComplete();
      if (close) {
        shutdownInput();
      }
    }
    catch (Throwable t)
    {
      handleReadException(pipeline, byteBuf, t, close, allocHandle);
    }
    finally
    {
      epollInFinally(config);
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.epoll.AbstractEpollStreamChannel.EpollStreamUnsafe
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */