package io.netty.channel.nio;

import io.netty.channel.AbstractChannel.AbstractUnsafe;
import io.netty.channel.ChannelConfig;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.ChannelPromise;
import io.netty.channel.ConnectTimeoutException;
import java.net.SocketAddress;
import java.nio.channels.ConnectionPendingException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

public abstract class AbstractNioChannel$AbstractNioUnsafe
  extends AbstractChannel.AbstractUnsafe
  implements AbstractNioChannel.NioUnsafe
{
  protected AbstractNioChannel$AbstractNioUnsafe(AbstractNioChannel paramAbstractNioChannel)
  {
    super(paramAbstractNioChannel);
  }
  
  protected final void removeReadOp()
  {
    SelectionKey key = this$0.selectionKey();
    if (!key.isValid()) {
      return;
    }
    int interestOps = key.interestOps();
    if ((interestOps & this$0.readInterestOp) != 0) {
      key.interestOps(interestOps & (this$0.readInterestOp ^ 0xFFFFFFFF));
    }
  }
  
  public final SelectableChannel ch()
  {
    return this$0.javaChannel();
  }
  
  public final void connect(final SocketAddress remoteAddress, SocketAddress localAddress, ChannelPromise promise)
  {
    if ((!promise.setUncancellable()) || (!ensureOpen(promise))) {
      return;
    }
    try
    {
      if (AbstractNioChannel.access$200(this$0) != null) {
        throw new ConnectionPendingException();
      }
      boolean wasActive = this$0.isActive();
      if (this$0.doConnect(remoteAddress, localAddress))
      {
        fulfillConnectPromise(promise, wasActive);
      }
      else
      {
        AbstractNioChannel.access$202(this$0, promise);
        AbstractNioChannel.access$302(this$0, remoteAddress);
        
        int connectTimeoutMillis = this$0.config().getConnectTimeoutMillis();
        if (connectTimeoutMillis > 0) {
          AbstractNioChannel.access$402(this$0, this$0.eventLoop().schedule(new Runnable()
          {
            public void run()
            {
              ChannelPromise connectPromise = AbstractNioChannel.access$200(this$0);
              ConnectTimeoutException cause = new ConnectTimeoutException("connection timed out: " + remoteAddress);
              if ((connectPromise != null) && (connectPromise.tryFailure(cause))) {
                close(voidPromise());
              }
            }
          }, connectTimeoutMillis, TimeUnit.MILLISECONDS));
        }
        promise.addListener(new ChannelFutureListener()
        {
          public void operationComplete(ChannelFuture future)
            throws Exception
          {
            if (future.isCancelled())
            {
              if (AbstractNioChannel.access$400(this$0) != null) {
                AbstractNioChannel.access$400(this$0).cancel(false);
              }
              AbstractNioChannel.access$202(this$0, null);
              close(voidPromise());
            }
          }
        });
      }
    }
    catch (Throwable t)
    {
      promise.tryFailure(annotateConnectException(t, remoteAddress));
      closeIfClosed();
    }
  }
  
  private void fulfillConnectPromise(ChannelPromise promise, boolean wasActive)
  {
    if (promise == null) {
      return;
    }
    boolean active = this$0.isActive();
    
    boolean promiseSet = promise.trySuccess();
    if ((!wasActive) && (active)) {
      this$0.pipeline().fireChannelActive();
    }
    if (!promiseSet) {
      close(voidPromise());
    }
  }
  
  private void fulfillConnectPromise(ChannelPromise promise, Throwable cause)
  {
    if (promise == null) {
      return;
    }
    promise.tryFailure(cause);
    closeIfClosed();
  }
  
  public final void finishConnect()
  {
    assert (this$0.eventLoop().inEventLoop());
    try
    {
      boolean wasActive = this$0.isActive();
      this$0.doFinishConnect();
      fulfillConnectPromise(AbstractNioChannel.access$200(this$0), wasActive);
    }
    catch (Throwable t)
    {
      fulfillConnectPromise(AbstractNioChannel.access$200(this$0), annotateConnectException(t, AbstractNioChannel.access$300(this$0)));
    }
    finally
    {
      if (AbstractNioChannel.access$400(this$0) != null) {
        AbstractNioChannel.access$400(this$0).cancel(false);
      }
      AbstractNioChannel.access$202(this$0, null);
    }
  }
  
  protected final void flush0()
  {
    if (isFlushPending()) {
      return;
    }
    super.flush0();
  }
  
  public final void forceFlush()
  {
    super.flush0();
  }
  
  private boolean isFlushPending()
  {
    SelectionKey selectionKey = this$0.selectionKey();
    return (selectionKey.isValid()) && ((selectionKey.interestOps() & 0x4) != 0);
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.nio.AbstractNioChannel.AbstractNioUnsafe
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */