package io.netty.handler.codec;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import io.netty.buffer.CompositeByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelConfig;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.socket.ChannelInputShutdownEvent;
import io.netty.util.internal.StringUtil;
import java.util.List;

public abstract class ByteToMessageDecoder
  extends ChannelInboundHandlerAdapter
{
  public static final Cumulator MERGE_CUMULATOR = new Cumulator()
  {
    public ByteBuf cumulate(ByteBufAllocator alloc, ByteBuf cumulation, ByteBuf in)
    {
      ByteBuf buffer;
      ByteBuf buffer;
      if ((cumulation.writerIndex() > cumulation.maxCapacity() - in.readableBytes()) || (cumulation.refCnt() > 1)) {
        buffer = ByteToMessageDecoder.expandCumulation(alloc, cumulation, in.readableBytes());
      } else {
        buffer = cumulation;
      }
      buffer.writeBytes(in);
      in.release();
      return buffer;
    }
  };
  public static final Cumulator COMPOSITE_CUMULATOR = new Cumulator()
  {
    public ByteBuf cumulate(ByteBufAllocator alloc, ByteBuf cumulation, ByteBuf in)
    {
      ByteBuf buffer;
      if (cumulation.refCnt() > 1)
      {
        ByteBuf buffer = ByteToMessageDecoder.expandCumulation(alloc, cumulation, in.readableBytes());
        buffer.writeBytes(in);
        in.release();
      }
      else
      {
        CompositeByteBuf composite;
        CompositeByteBuf composite;
        if ((cumulation instanceof CompositeByteBuf))
        {
          composite = (CompositeByteBuf)cumulation;
        }
        else
        {
          composite = alloc.compositeBuffer(Integer.MAX_VALUE);
          composite.addComponent(true, cumulation);
        }
        composite.addComponent(true, in);
        buffer = composite;
      }
      return buffer;
    }
  };
  ByteBuf cumulation;
  private Cumulator cumulator = MERGE_CUMULATOR;
  private boolean singleDecode;
  private boolean decodeWasNull;
  private boolean first;
  private int discardAfterReads = 16;
  private int numReads;
  
  protected ByteToMessageDecoder()
  {
    CodecUtil.ensureNotSharable(this);
  }
  
  public void setSingleDecode(boolean singleDecode)
  {
    this.singleDecode = singleDecode;
  }
  
  public boolean isSingleDecode()
  {
    return singleDecode;
  }
  
  public void setCumulator(Cumulator cumulator)
  {
    if (cumulator == null) {
      throw new NullPointerException("cumulator");
    }
    this.cumulator = cumulator;
  }
  
  public void setDiscardAfterReads(int discardAfterReads)
  {
    if (discardAfterReads <= 0) {
      throw new IllegalArgumentException("discardAfterReads must be > 0");
    }
    this.discardAfterReads = discardAfterReads;
  }
  
  protected int actualReadableBytes()
  {
    return internalBuffer().readableBytes();
  }
  
  protected ByteBuf internalBuffer()
  {
    if (cumulation != null) {
      return cumulation;
    }
    return Unpooled.EMPTY_BUFFER;
  }
  
  public final void handlerRemoved(ChannelHandlerContext ctx)
    throws Exception
  {
    ByteBuf buf = cumulation;
    if (buf != null)
    {
      cumulation = null;
      
      int readable = buf.readableBytes();
      if (readable > 0)
      {
        ByteBuf bytes = buf.readBytes(readable);
        buf.release();
        ctx.fireChannelRead(bytes);
      }
      else
      {
        buf.release();
      }
      numReads = 0;
      ctx.fireChannelReadComplete();
    }
    handlerRemoved0(ctx);
  }
  
  protected void handlerRemoved0(ChannelHandlerContext ctx)
    throws Exception
  {}
  
  public void channelRead(ChannelHandlerContext ctx, Object msg)
    throws Exception
  {
    if ((msg instanceof ByteBuf))
    {
      CodecOutputList out = CodecOutputList.newInstance();
      try
      {
        ByteBuf data = (ByteBuf)msg;
        first = (cumulation == null);
        if (first) {
          cumulation = data;
        } else {
          cumulation = cumulator.cumulate(ctx.alloc(), cumulation, data);
        }
        callDecode(ctx, cumulation, out);
      }
      catch (DecoderException e)
      {
        int size;
        throw e;
      }
      catch (Throwable t)
      {
        throw new DecoderException(t);
      }
      finally
      {
        if ((cumulation != null) && (!cumulation.isReadable()))
        {
          numReads = 0;
          cumulation.release();
          cumulation = null;
        }
        else if (++numReads >= discardAfterReads)
        {
          numReads = 0;
          discardSomeReadBytes();
        }
        int size = out.size();
        decodeWasNull = (!out.insertSinceRecycled());
        fireChannelRead(ctx, out, size);
        out.recycle();
      }
    }
    else
    {
      ctx.fireChannelRead(msg);
    }
  }
  
  static void fireChannelRead(ChannelHandlerContext ctx, List<Object> msgs, int numElements)
  {
    if ((msgs instanceof CodecOutputList)) {
      fireChannelRead(ctx, (CodecOutputList)msgs, numElements);
    } else {
      for (int i = 0; i < numElements; i++) {
        ctx.fireChannelRead(msgs.get(i));
      }
    }
  }
  
  static void fireChannelRead(ChannelHandlerContext ctx, CodecOutputList msgs, int numElements)
  {
    for (int i = 0; i < numElements; i++) {
      ctx.fireChannelRead(msgs.getUnsafe(i));
    }
  }
  
  public void channelReadComplete(ChannelHandlerContext ctx)
    throws Exception
  {
    numReads = 0;
    discardSomeReadBytes();
    if (decodeWasNull)
    {
      decodeWasNull = false;
      if (!ctx.channel().config().isAutoRead()) {
        ctx.read();
      }
    }
    ctx.fireChannelReadComplete();
  }
  
  protected final void discardSomeReadBytes()
  {
    if ((cumulation != null) && (!first) && (cumulation.refCnt() == 1)) {
      cumulation.discardSomeReadBytes();
    }
  }
  
  public void channelInactive(ChannelHandlerContext ctx)
    throws Exception
  {
    channelInputClosed(ctx, true);
  }
  
  public void userEventTriggered(ChannelHandlerContext ctx, Object evt)
    throws Exception
  {
    if ((evt instanceof ChannelInputShutdownEvent)) {
      channelInputClosed(ctx, false);
    }
    super.userEventTriggered(ctx, evt);
  }
  
  private void channelInputClosed(ChannelHandlerContext ctx, boolean callChannelInactive)
    throws Exception
  {
    CodecOutputList out = CodecOutputList.newInstance();
    try
    {
      channelInputClosed(ctx, out);
    }
    catch (DecoderException e)
    {
      int size;
      throw e;
    }
    catch (Exception e)
    {
      throw new DecoderException(e);
    }
    finally
    {
      try
      {
        if (cumulation != null)
        {
          cumulation.release();
          cumulation = null;
        }
        int size = out.size();
        fireChannelRead(ctx, out, size);
        if (size > 0) {
          ctx.fireChannelReadComplete();
        }
        if (callChannelInactive) {
          ctx.fireChannelInactive();
        }
      }
      finally
      {
        out.recycle();
      }
    }
  }
  
  void channelInputClosed(ChannelHandlerContext ctx, List<Object> out)
    throws Exception
  {
    if (cumulation != null)
    {
      callDecode(ctx, cumulation, out);
      decodeLast(ctx, cumulation, out);
    }
    else
    {
      decodeLast(ctx, Unpooled.EMPTY_BUFFER, out);
    }
  }
  
  protected void callDecode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
  {
    try
    {
      while (in.isReadable())
      {
        int outSize = out.size();
        if (outSize > 0)
        {
          fireChannelRead(ctx, out, outSize);
          out.clear();
          if (!ctx.isRemoved()) {
            outSize = 0;
          }
        }
        else
        {
          int oldInputLength = in.readableBytes();
          decode(ctx, in, out);
          if (!ctx.isRemoved()) {
            if (outSize == out.size())
            {
              if (oldInputLength != in.readableBytes()) {
                continue;
              }
            }
            else
            {
              if (oldInputLength == in.readableBytes()) {
                throw new DecoderException(StringUtil.simpleClassName(getClass()) + ".decode() did not read anything but decoded a message.");
              }
              if (isSingleDecode()) {
                break;
              }
            }
          }
        }
      }
    }
    catch (DecoderException e)
    {
      throw e;
    }
    catch (Throwable cause)
    {
      throw new DecoderException(cause);
    }
  }
  
  protected abstract void decode(ChannelHandlerContext paramChannelHandlerContext, ByteBuf paramByteBuf, List<Object> paramList)
    throws Exception;
  
  protected void decodeLast(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    if (in.isReadable()) {
      decode(ctx, in, out);
    }
  }
  
  static ByteBuf expandCumulation(ByteBufAllocator alloc, ByteBuf cumulation, int readable)
  {
    ByteBuf oldCumulation = cumulation;
    cumulation = alloc.buffer(oldCumulation.readableBytes() + readable);
    cumulation.writeBytes(oldCumulation);
    oldCumulation.release();
    return cumulation;
  }
  
  public static abstract interface Cumulator
  {
    public abstract ByteBuf cumulate(ByteBufAllocator paramByteBufAllocator, ByteBuf paramByteBuf1, ByteBuf paramByteBuf2);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.ByteToMessageDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */