package io.netty.handler.codec.http2;

import io.netty.util.collection.IntObjectMap;
import io.netty.util.internal.ObjectUtil;
import io.netty.util.internal.logging.InternalLogger;
import java.util.ArrayList;
import java.util.List;

final class DefaultHttp2Connection$DefaultEndpoint<F extends Http2FlowController>
  implements Http2Connection.Endpoint<F>
{
  private final boolean server;
  private int nextStreamIdToCreate;
  private int nextReservationStreamId;
  private int lastStreamKnownByPeer = -1;
  private boolean pushToAllowed = true;
  private F flowController;
  private int maxActiveStreams;
  private int maxStreams;
  int numActiveStreams;
  int numStreams;
  
  DefaultHttp2Connection$DefaultEndpoint(DefaultHttp2Connection paramDefaultHttp2Connection, boolean server)
  {
    this.server = server;
    if (server)
    {
      nextStreamIdToCreate = 2;
      nextReservationStreamId = 0;
    }
    else
    {
      nextStreamIdToCreate = 1;
      
      nextReservationStreamId = 1;
    }
    pushToAllowed = (!server);
    maxStreams = (maxActiveStreams = Integer.MAX_VALUE);
  }
  
  public int incrementAndGetNextStreamId()
  {
    return nextReservationStreamId >= 0 ? nextReservationStreamId += 2 : nextReservationStreamId;
  }
  
  private void incrementExpectedStreamId(int streamId)
  {
    if ((streamId > nextReservationStreamId) && (nextReservationStreamId >= 0)) {
      nextReservationStreamId = streamId;
    }
    nextStreamIdToCreate = (streamId + 2);
    numStreams += 1;
  }
  
  public boolean isValidStreamId(int streamId)
  {
    if (streamId > 0) {}
    return server == ((streamId & 0x1) == 0);
  }
  
  public boolean mayHaveCreatedStream(int streamId)
  {
    return (isValidStreamId(streamId)) && (streamId <= lastStreamCreated());
  }
  
  public boolean canOpenStream()
  {
    return numActiveStreams < maxActiveStreams;
  }
  
  private DefaultHttp2Connection.DefaultStream createStream(int streamId, Http2Stream.State state)
    throws Http2Exception
  {
    checkNewStreamAllowed(streamId, state);
    
    DefaultHttp2Connection.DefaultStream stream = new DefaultHttp2Connection.DefaultStream(this$0, streamId, state);
    
    incrementExpectedStreamId(streamId);
    
    addStream(stream);
    return stream;
  }
  
  public DefaultHttp2Connection.DefaultStream createIdleStream(int streamId)
    throws Http2Exception
  {
    return createStream(streamId, Http2Stream.State.IDLE);
  }
  
  public DefaultHttp2Connection.DefaultStream createStream(int streamId, boolean halfClosed)
    throws Http2Exception
  {
    DefaultHttp2Connection.DefaultStream stream = createStream(streamId, DefaultHttp2Connection.activeState(streamId, Http2Stream.State.IDLE, isLocal(), halfClosed));
    stream.activate();
    return stream;
  }
  
  public boolean created(Http2Stream stream)
  {
    return ((stream instanceof DefaultHttp2Connection.DefaultStream)) && (((DefaultHttp2Connection.DefaultStream)stream).createdBy() == this);
  }
  
  public boolean isServer()
  {
    return server;
  }
  
  public DefaultHttp2Connection.DefaultStream reservePushStream(int streamId, Http2Stream parent)
    throws Http2Exception
  {
    if (parent == null) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Parent stream missing", new Object[0]);
    }
    if (isLocal() ? !parent.state().localSideOpen() : !parent.state().remoteSideOpen()) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Stream %d is not open for sending push promise", new Object[] { Integer.valueOf(parent.id()) });
    }
    if (!opposite().allowPushTo()) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Server push not allowed to opposite endpoint.", new Object[0]);
    }
    Http2Stream.State state = isLocal() ? Http2Stream.State.RESERVED_LOCAL : Http2Stream.State.RESERVED_REMOTE;
    checkNewStreamAllowed(streamId, state);
    
    DefaultHttp2Connection.DefaultStream stream = new DefaultHttp2Connection.DefaultStream(this$0, streamId, state);
    
    incrementExpectedStreamId(streamId);
    
    addStream(stream);
    return stream;
  }
  
  private void addStream(DefaultHttp2Connection.DefaultStream stream)
  {
    this$0.streamMap.put(stream.id(), stream);
    
    List<DefaultHttp2Connection.ParentChangedEvent> events = new ArrayList(1);
    this$0.connectionStream.takeChild(stream, false, events);
    for (int i = 0; i < this$0.listeners.size(); i++) {
      try
      {
        ((Http2Connection.Listener)this$0.listeners.get(i)).onStreamAdded(stream);
      }
      catch (Throwable cause)
      {
        DefaultHttp2Connection.access$600().error("Caught Throwable from listener onStreamAdded.", cause);
      }
    }
    DefaultHttp2Connection.access$400(this$0, events);
  }
  
  public void allowPushTo(boolean allow)
  {
    if ((allow) && (server)) {
      throw new IllegalArgumentException("Servers do not allow push");
    }
    pushToAllowed = allow;
  }
  
  public boolean allowPushTo()
  {
    return pushToAllowed;
  }
  
  public int numActiveStreams()
  {
    return numActiveStreams;
  }
  
  public int maxActiveStreams()
  {
    return maxActiveStreams;
  }
  
  public int maxStreams()
  {
    return maxStreams;
  }
  
  public void maxStreams(int maxActiveStreams, int maxStreams)
    throws Http2Exception
  {
    if (maxStreams < maxActiveStreams) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "maxStream[%d] streams must be >= maxActiveStreams[%d]", new Object[] { Integer.valueOf(maxStreams), Integer.valueOf(maxActiveStreams) });
    }
    this.maxStreams = maxStreams;
    this.maxActiveStreams = maxActiveStreams;
  }
  
  public int lastStreamCreated()
  {
    return nextStreamIdToCreate > 1 ? nextStreamIdToCreate - 2 : 0;
  }
  
  public int lastStreamKnownByPeer()
  {
    return lastStreamKnownByPeer;
  }
  
  private void lastStreamKnownByPeer(int lastKnownStream)
  {
    lastStreamKnownByPeer = lastKnownStream;
  }
  
  public F flowController()
  {
    return flowController;
  }
  
  public void flowController(F flowController)
  {
    this.flowController = ((Http2FlowController)ObjectUtil.checkNotNull(flowController, "flowController"));
  }
  
  public Http2Connection.Endpoint<? extends Http2FlowController> opposite()
  {
    return isLocal() ? this$0.remoteEndpoint : this$0.localEndpoint;
  }
  
  private void checkNewStreamAllowed(int streamId, Http2Stream.State state)
    throws Http2Exception
  {
    if ((this$0.goAwayReceived()) && (streamId > this$0.localEndpoint.lastStreamKnownByPeer())) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Cannot create stream %d since this endpoint has received a GOAWAY frame with last stream id %d.", new Object[] { Integer.valueOf(streamId), Integer.valueOf(this$0.localEndpoint.lastStreamKnownByPeer()) });
    }
    if (streamId < 0) {
      throw new Http2NoMoreStreamIdsException();
    }
    if (!isValidStreamId(streamId)) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Request stream %d is not correct for %s connection", new Object[] { Integer.valueOf(streamId), server ? "server" : "client" });
    }
    if (streamId < nextStreamIdToCreate) {
      throw Http2Exception.closedStreamError(Http2Error.PROTOCOL_ERROR, "Request stream %d is behind the next expected stream %d", new Object[] { Integer.valueOf(streamId), Integer.valueOf(nextStreamIdToCreate) });
    }
    if (nextStreamIdToCreate <= 0) {
      throw Http2Exception.connectionError(Http2Error.REFUSED_STREAM, "Stream IDs are exhausted for this endpoint.", new Object[0]);
    }
    if ((state.localSideOpen()) || (state.remoteSideOpen()))
    {
      if (!canOpenStream()) {
        throw Http2Exception.streamError(streamId, Http2Error.REFUSED_STREAM, "Maximum active streams violated for this endpoint.", new Object[0]);
      }
    }
    else if (numStreams == maxStreams) {
      throw Http2Exception.streamError(streamId, Http2Error.REFUSED_STREAM, "Maximum streams violated for this endpoint.", new Object[0]);
    }
    if (this$0.isClosed()) {
      throw Http2Exception.connectionError(Http2Error.INTERNAL_ERROR, "Attempted to create stream id %d after connection was closed", new Object[] { Integer.valueOf(streamId) });
    }
  }
  
  private boolean isLocal()
  {
    return this == this$0.localEndpoint;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2Connection.DefaultEndpoint
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */