package io.netty.handler.codec.http2;

import io.netty.util.collection.IntCollections;
import io.netty.util.collection.IntObjectHashMap;
import io.netty.util.collection.IntObjectMap;
import io.netty.util.internal.EmptyArrays;
import io.netty.util.internal.logging.InternalLogger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

class DefaultHttp2Connection$DefaultStream
  implements Http2Stream
{
  private final int id;
  private final PropertyMap properties = new PropertyMap(null);
  private Http2Stream.State state;
  private short weight = 16;
  private DefaultStream parent;
  private IntObjectMap<DefaultStream> children = IntCollections.emptyMap();
  private boolean resetSent;
  
  DefaultHttp2Connection$DefaultStream(DefaultHttp2Connection paramDefaultHttp2Connection, int id, Http2Stream.State state)
  {
    this.id = id;
    this.state = state;
  }
  
  public final int id()
  {
    return id;
  }
  
  public final Http2Stream.State state()
  {
    return state;
  }
  
  public boolean isResetSent()
  {
    return resetSent;
  }
  
  public Http2Stream resetSent()
  {
    resetSent = true;
    return this;
  }
  
  public final <V> V setProperty(Http2Connection.PropertyKey key, V value)
  {
    return (V)properties.add(this$0.verifyKey(key), value);
  }
  
  public final <V> V getProperty(Http2Connection.PropertyKey key)
  {
    return (V)properties.get(this$0.verifyKey(key));
  }
  
  public final <V> V removeProperty(Http2Connection.PropertyKey key)
  {
    return (V)properties.remove(this$0.verifyKey(key));
  }
  
  public final boolean isRoot()
  {
    return parent == null;
  }
  
  public final short weight()
  {
    return weight;
  }
  
  public final DefaultStream parent()
  {
    return parent;
  }
  
  public final boolean isDescendantOf(Http2Stream stream)
  {
    Http2Stream next = parent();
    while (next != null)
    {
      if (next == stream) {
        return true;
      }
      next = next.parent();
    }
    return false;
  }
  
  public final boolean isLeaf()
  {
    return numChildren() == 0;
  }
  
  public final int numChildren()
  {
    return children.size();
  }
  
  public Http2Stream forEachChild(Http2StreamVisitor visitor)
    throws Http2Exception
  {
    for (DefaultStream stream : children.values()) {
      if (!visitor.visit(stream)) {
        return stream;
      }
    }
    return null;
  }
  
  public Http2Stream setPriority(int parentStreamId, short weight, boolean exclusive)
    throws Http2Exception
  {
    if ((weight < 1) || (weight > 256)) {
      throw new IllegalArgumentException(String.format("Invalid weight: %d.  Must be between %d and %d (inclusive).", new Object[] { Short.valueOf(weight), Short.valueOf(1), Short.valueOf(256) }));
    }
    DefaultStream newParent = (DefaultStream)this$0.stream(parentStreamId);
    if (newParent == null) {
      newParent = createdBy().createIdleStream(parentStreamId);
    } else if (this == newParent) {
      throw new IllegalArgumentException("A stream cannot depend on itself");
    }
    weight(weight);
    if ((newParent != parent()) || ((exclusive) && (newParent.numChildren() != 1)))
    {
      List<DefaultHttp2Connection.ParentChangedEvent> events;
      if (newParent.isDescendantOf(this))
      {
        List<DefaultHttp2Connection.ParentChangedEvent> events = new ArrayList(2 + (exclusive ? newParent.numChildren() : 0));
        parent.takeChild(newParent, false, events);
      }
      else
      {
        events = new ArrayList(1 + (exclusive ? newParent.numChildren() : 0));
      }
      newParent.takeChild(this, exclusive, events);
      DefaultHttp2Connection.access$400(this$0, events);
    }
    return this;
  }
  
  public Http2Stream open(boolean halfClosed)
    throws Http2Exception
  {
    state = DefaultHttp2Connection.activeState(id, state, isLocal(), halfClosed);
    if (!createdBy().canOpenStream()) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Maximum active streams violated for this endpoint.", new Object[0]);
    }
    activate();
    return this;
  }
  
  void activate()
  {
    this$0.activeStreams.activate(this);
  }
  
  Http2Stream close(Iterator<?> itr)
  {
    if (state == Http2Stream.State.CLOSED) {
      return this;
    }
    state = Http2Stream.State.CLOSED;
    
    createdBynumStreams -= 1;
    this$0.activeStreams.deactivate(this, itr);
    return this;
  }
  
  public Http2Stream close()
  {
    return close(null);
  }
  
  public Http2Stream closeLocalSide()
  {
    switch (DefaultHttp2Connection.3.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[state.ordinal()])
    {
    case 4: 
      state = Http2Stream.State.HALF_CLOSED_LOCAL;
      this$0.notifyHalfClosed(this);
      break;
    case 5: 
      break;
    default: 
      close();
    }
    return this;
  }
  
  public Http2Stream closeRemoteSide()
  {
    switch (DefaultHttp2Connection.3.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[state.ordinal()])
    {
    case 4: 
      state = Http2Stream.State.HALF_CLOSED_REMOTE;
      this$0.notifyHalfClosed(this);
      break;
    case 6: 
      break;
    default: 
      close();
    }
    return this;
  }
  
  private void initChildrenIfEmpty()
  {
    if (children == IntCollections.emptyMap()) {
      initChildren();
    }
  }
  
  private void initChildren()
  {
    children = new IntObjectHashMap(DefaultHttp2Connection.access$500());
  }
  
  DefaultHttp2Connection.DefaultEndpoint<? extends Http2FlowController> createdBy()
  {
    return this$0.localEndpoint.isValidStreamId(id) ? this$0.localEndpoint : this$0.remoteEndpoint;
  }
  
  final boolean isLocal()
  {
    return this$0.localEndpoint.isValidStreamId(id);
  }
  
  final void weight(short weight)
  {
    if (weight != this.weight)
    {
      short oldWeight = this.weight;
      this.weight = weight;
      for (int i = 0; i < this$0.listeners.size(); i++) {
        try
        {
          ((Http2Connection.Listener)this$0.listeners.get(i)).onWeightChanged(this, oldWeight);
        }
        catch (Throwable cause)
        {
          DefaultHttp2Connection.access$600().error("Caught Throwable from listener onWeightChanged.", cause);
        }
      }
    }
  }
  
  private IntObjectMap<DefaultStream> retain(DefaultStream streamToRetain)
  {
    streamToRetain = (DefaultStream)children.remove(streamToRetain.id());
    IntObjectMap<DefaultStream> prevChildren = children;
    
    initChildren();
    if (streamToRetain != null) {
      children.put(streamToRetain.id(), streamToRetain);
    }
    return prevChildren;
  }
  
  final void takeChild(DefaultStream child, boolean exclusive, List<DefaultHttp2Connection.ParentChangedEvent> events)
  {
    DefaultStream oldParent = child.parent();
    if (oldParent != this)
    {
      events.add(new DefaultHttp2Connection.ParentChangedEvent(child, oldParent));
      DefaultHttp2Connection.access$700(this$0, child, this);
      parent = this;
      if (oldParent != null) {
        children.remove(child.id());
      }
      initChildrenIfEmpty();
      
      Http2Stream oldChild = (Http2Stream)children.put(child.id(), child);
      assert (oldChild == null) : "A stream with the same stream ID was already in the child map.";
    }
    if ((exclusive) && (!children.isEmpty())) {
      for (DefaultStream grandchild : retain(child).values()) {
        child.takeChild(grandchild, false, events);
      }
    }
  }
  
  final boolean removeChild(DefaultStream child)
  {
    if (children.remove(child.id()) != null)
    {
      List<DefaultHttp2Connection.ParentChangedEvent> events = new ArrayList(1 + child.numChildren());
      events.add(new DefaultHttp2Connection.ParentChangedEvent(child, child.parent()));
      DefaultHttp2Connection.access$700(this$0, child, null);
      parent = null;
      for (DefaultStream grandchild : children.values()) {
        takeChild(grandchild, false, events);
      }
      DefaultHttp2Connection.access$400(this$0, events);
      return true;
    }
    return false;
  }
  
  private class PropertyMap
  {
    Object[] values = EmptyArrays.EMPTY_OBJECTS;
    
    private PropertyMap() {}
    
    <V> V add(DefaultHttp2Connection.DefaultPropertyKey key, V value)
    {
      resizeIfNecessary(index);
      
      V prevValue = values[index];
      values[index] = value;
      return prevValue;
    }
    
    <V> V get(DefaultHttp2Connection.DefaultPropertyKey key)
    {
      if (index >= values.length) {
        return null;
      }
      return (V)values[index];
    }
    
    <V> V remove(DefaultHttp2Connection.DefaultPropertyKey key)
    {
      V prevValue = null;
      if (index < values.length)
      {
        prevValue = values[index];
        values[index] = null;
      }
      return prevValue;
    }
    
    void resizeIfNecessary(int index)
    {
      if (index >= values.length) {
        values = Arrays.copyOf(values, this$0.propertyKeyRegistry.size());
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2Connection.DefaultStream
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */