package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.util.internal.logging.InternalLogger;

final class DefaultHttp2ConnectionDecoder$FrameReadListener
  implements Http2FrameListener
{
  private DefaultHttp2ConnectionDecoder$FrameReadListener(DefaultHttp2ConnectionDecoder paramDefaultHttp2ConnectionDecoder) {}
  
  public int onDataRead(ChannelHandlerContext ctx, int streamId, ByteBuf data, int padding, boolean endOfStream)
    throws Http2Exception
  {
    Http2Stream stream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    Http2LocalFlowController flowController = this$0.flowController();
    int bytesToReturn = data.readableBytes() + padding;
    
    boolean shouldIgnore = true;
    try
    {
      shouldIgnore = shouldIgnoreHeadersOrDataFrame(ctx, streamId, stream, "DATA");
    }
    finally
    {
      if (shouldIgnore)
      {
        flowController.receiveFlowControlledFrame(stream, data, padding, endOfStream);
        flowController.consumeBytes(stream, bytesToReturn);
        
        verifyStreamMayHaveExisted(streamId);
        
        return bytesToReturn;
      }
    }
    Http2Exception error = null;
    switch (DefaultHttp2ConnectionDecoder.1.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[stream.state().ordinal()])
    {
    case 1: 
    case 2: 
      break;
    case 3: 
    case 4: 
      error = Http2Exception.streamError(stream.id(), Http2Error.STREAM_CLOSED, "Stream %d in unexpected state: %s", new Object[] { Integer.valueOf(stream.id()), stream.state() });
      
      break;
    default: 
      error = Http2Exception.streamError(stream.id(), Http2Error.PROTOCOL_ERROR, "Stream %d in unexpected state: %s", new Object[] { Integer.valueOf(stream.id()), stream.state() });
    }
    int unconsumedBytes = DefaultHttp2ConnectionDecoder.access$200(this$0, stream);
    try
    {
      flowController.receiveFlowControlledFrame(stream, data, padding, endOfStream);
      
      unconsumedBytes = DefaultHttp2ConnectionDecoder.access$200(this$0, stream);
      if (error != null) {
        throw error;
      }
      bytesToReturn = DefaultHttp2ConnectionDecoder.access$300(this$0).onDataRead(ctx, streamId, data, padding, endOfStream);
      return bytesToReturn;
    }
    catch (Http2Exception e)
    {
      int delta = unconsumedBytes - DefaultHttp2ConnectionDecoder.access$200(this$0, stream);
      bytesToReturn -= delta;
      throw e;
    }
    catch (RuntimeException e)
    {
      int delta = unconsumedBytes - DefaultHttp2ConnectionDecoder.access$200(this$0, stream);
      bytesToReturn -= delta;
      throw e;
    }
    finally
    {
      flowController.consumeBytes(stream, bytesToReturn);
      if (endOfStream) {
        DefaultHttp2ConnectionDecoder.access$400(this$0).closeStreamRemote(stream, ctx.newSucceededFuture());
      }
    }
  }
  
  public void onHeadersRead(ChannelHandlerContext ctx, int streamId, Http2Headers headers, int padding, boolean endOfStream)
    throws Http2Exception
  {
    onHeadersRead(ctx, streamId, headers, 0, (short)16, false, padding, endOfStream);
  }
  
  public void onHeadersRead(ChannelHandlerContext ctx, int streamId, Http2Headers headers, int streamDependency, short weight, boolean exclusive, int padding, boolean endOfStream)
    throws Http2Exception
  {
    Http2Stream stream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    boolean allowHalfClosedRemote = false;
    if ((stream == null) && (!DefaultHttp2ConnectionDecoder.access$100(this$0).streamMayHaveExisted(streamId)))
    {
      stream = DefaultHttp2ConnectionDecoder.access$100(this$0).remote().createStream(streamId, endOfStream);
      
      allowHalfClosedRemote = stream.state() == Http2Stream.State.HALF_CLOSED_REMOTE;
    }
    if (shouldIgnoreHeadersOrDataFrame(ctx, streamId, stream, "HEADERS")) {
      return;
    }
    switch (DefaultHttp2ConnectionDecoder.1.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[stream.state().ordinal()])
    {
    case 5: 
      stream.open(endOfStream);
      break;
    case 1: 
    case 2: 
      break;
    case 3: 
      if (!allowHalfClosedRemote) {
        throw Http2Exception.streamError(stream.id(), Http2Error.STREAM_CLOSED, "Stream %d in unexpected state: %s", new Object[] { Integer.valueOf(stream.id()), stream.state() });
      }
      break;
    case 4: 
      throw Http2Exception.streamError(stream.id(), Http2Error.STREAM_CLOSED, "Stream %d in unexpected state: %s", new Object[] { Integer.valueOf(stream.id()), stream.state() });
    default: 
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Stream %d in unexpected state: %s", new Object[] { Integer.valueOf(stream.id()), stream.state() });
    }
    try
    {
      stream.setPriority(streamDependency, weight, exclusive);
    }
    catch (Http2Exception.ClosedStreamCreationException localClosedStreamCreationException) {}
    DefaultHttp2ConnectionDecoder.access$300(this$0).onHeadersRead(ctx, streamId, headers, streamDependency, weight, exclusive, padding, endOfStream);
    if (endOfStream) {
      DefaultHttp2ConnectionDecoder.access$400(this$0).closeStreamRemote(stream, ctx.newSucceededFuture());
    }
  }
  
  public void onPriorityRead(ChannelHandlerContext ctx, int streamId, int streamDependency, short weight, boolean exclusive)
    throws Http2Exception
  {
    Http2Stream stream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    try
    {
      if (stream == null)
      {
        if (DefaultHttp2ConnectionDecoder.access$100(this$0).streamMayHaveExisted(streamId))
        {
          DefaultHttp2ConnectionDecoder.access$500().info("{} ignoring PRIORITY frame for stream {}. Stream doesn't exist but may  have existed", ctx.channel(), Integer.valueOf(streamId));
          
          return;
        }
        stream = DefaultHttp2ConnectionDecoder.access$100(this$0).remote().createIdleStream(streamId);
      }
      else if (streamCreatedAfterGoAwaySent(streamId))
      {
        DefaultHttp2ConnectionDecoder.access$500().info("{} ignoring PRIORITY frame for stream {}. Stream created after GOAWAY sent. Last known stream by peer {}", new Object[] { ctx.channel(), Integer.valueOf(streamId), Integer.valueOf(DefaultHttp2ConnectionDecoder.access$100(this$0).remote().lastStreamKnownByPeer()) });
        
        return;
      }
      stream.setPriority(streamDependency, weight, exclusive);
    }
    catch (Http2Exception.ClosedStreamCreationException localClosedStreamCreationException) {}
    DefaultHttp2ConnectionDecoder.access$300(this$0).onPriorityRead(ctx, streamId, streamDependency, weight, exclusive);
  }
  
  public void onRstStreamRead(ChannelHandlerContext ctx, int streamId, long errorCode)
    throws Http2Exception
  {
    Http2Stream stream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    if (stream == null)
    {
      verifyStreamMayHaveExisted(streamId);
      return;
    }
    switch (DefaultHttp2ConnectionDecoder.1.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[stream.state().ordinal()])
    {
    case 6: 
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "RST_STREAM received for IDLE stream %d", new Object[] { Integer.valueOf(streamId) });
    case 4: 
      return;
    }
    DefaultHttp2ConnectionDecoder.access$300(this$0).onRstStreamRead(ctx, streamId, errorCode);
    
    DefaultHttp2ConnectionDecoder.access$400(this$0).closeStream(stream, ctx.newSucceededFuture());
  }
  
  public void onSettingsAckRead(ChannelHandlerContext ctx)
    throws Http2Exception
  {
    Http2Settings settings = DefaultHttp2ConnectionDecoder.access$600(this$0).pollSentSettings();
    if (settings != null) {
      applyLocalSettings(settings);
    }
    DefaultHttp2ConnectionDecoder.access$300(this$0).onSettingsAckRead(ctx);
  }
  
  private void applyLocalSettings(Http2Settings settings)
    throws Http2Exception
  {
    Boolean pushEnabled = settings.pushEnabled();
    Http2FrameReader.Configuration config = DefaultHttp2ConnectionDecoder.access$700(this$0).configuration();
    Http2HeaderTable headerTable = config.headerTable();
    Http2FrameSizePolicy frameSizePolicy = config.frameSizePolicy();
    if (pushEnabled != null)
    {
      if (DefaultHttp2ConnectionDecoder.access$100(this$0).isServer()) {
        throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Server sending SETTINGS frame with ENABLE_PUSH specified", new Object[0]);
      }
      DefaultHttp2ConnectionDecoder.access$100(this$0).local().allowPushTo(pushEnabled.booleanValue());
    }
    Long maxConcurrentStreams = settings.maxConcurrentStreams();
    if (maxConcurrentStreams != null)
    {
      int value = (int)Math.min(maxConcurrentStreams.longValue(), 2147483647L);
      DefaultHttp2ConnectionDecoder.access$100(this$0).remote().maxStreams(value, calculateMaxStreams(value));
    }
    Long headerTableSize = settings.headerTableSize();
    if (headerTableSize != null) {
      headerTable.maxHeaderTableSize((int)Math.min(headerTableSize.longValue(), 2147483647L));
    }
    Integer maxHeaderListSize = settings.maxHeaderListSize();
    if (maxHeaderListSize != null) {
      headerTable.maxHeaderListSize(maxHeaderListSize.intValue());
    }
    Integer maxFrameSize = settings.maxFrameSize();
    if (maxFrameSize != null) {
      frameSizePolicy.maxFrameSize(maxFrameSize.intValue());
    }
    Integer initialWindowSize = settings.initialWindowSize();
    if (initialWindowSize != null) {
      this$0.flowController().initialWindowSize(initialWindowSize.intValue());
    }
  }
  
  protected int calculateMaxStreams(int maxConcurrentStreams)
  {
    int maxStreams = maxConcurrentStreams + 100;
    return maxStreams < 0 ? Integer.MAX_VALUE : maxStreams;
  }
  
  public void onSettingsRead(ChannelHandlerContext ctx, Http2Settings settings)
    throws Http2Exception
  {
    DefaultHttp2ConnectionDecoder.access$600(this$0).remoteSettings(settings);
    
    DefaultHttp2ConnectionDecoder.access$600(this$0).writeSettingsAck(ctx, ctx.newPromise());
    
    DefaultHttp2ConnectionDecoder.access$300(this$0).onSettingsRead(ctx, settings);
  }
  
  public void onPingRead(ChannelHandlerContext ctx, ByteBuf data)
    throws Http2Exception
  {
    DefaultHttp2ConnectionDecoder.access$600(this$0).writePing(ctx, true, data.retainedSlice(), ctx.newPromise());
    
    DefaultHttp2ConnectionDecoder.access$300(this$0).onPingRead(ctx, data);
  }
  
  public void onPingAckRead(ChannelHandlerContext ctx, ByteBuf data)
    throws Http2Exception
  {
    DefaultHttp2ConnectionDecoder.access$300(this$0).onPingAckRead(ctx, data);
  }
  
  public void onPushPromiseRead(ChannelHandlerContext ctx, int streamId, int promisedStreamId, Http2Headers headers, int padding)
    throws Http2Exception
  {
    Http2Stream parentStream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    if (shouldIgnoreHeadersOrDataFrame(ctx, streamId, parentStream, "PUSH_PROMISE")) {
      return;
    }
    if (parentStream == null) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Stream %d does not exist", new Object[] { Integer.valueOf(streamId) });
    }
    switch (DefaultHttp2ConnectionDecoder.1.$SwitchMap$io$netty$handler$codec$http2$Http2Stream$State[parentStream.state().ordinal()])
    {
    case 1: 
    case 2: 
      break;
    default: 
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Stream %d in unexpected state for receiving push promise: %s", new Object[] { Integer.valueOf(parentStream.id()), parentStream.state() });
    }
    if (!DefaultHttp2ConnectionDecoder.access$800(this$0).isAuthoritative(ctx, headers)) {
      throw Http2Exception.streamError(promisedStreamId, Http2Error.PROTOCOL_ERROR, "Promised request on stream %d for promised stream %d is not authoritative", new Object[] { Integer.valueOf(streamId), Integer.valueOf(promisedStreamId) });
    }
    if (!DefaultHttp2ConnectionDecoder.access$800(this$0).isCacheable(headers)) {
      throw Http2Exception.streamError(promisedStreamId, Http2Error.PROTOCOL_ERROR, "Promised request on stream %d for promised stream %d is not known to be cacheable", new Object[] { Integer.valueOf(streamId), Integer.valueOf(promisedStreamId) });
    }
    if (!DefaultHttp2ConnectionDecoder.access$800(this$0).isSafe(headers)) {
      throw Http2Exception.streamError(promisedStreamId, Http2Error.PROTOCOL_ERROR, "Promised request on stream %d for promised stream %d is not known to be safe", new Object[] { Integer.valueOf(streamId), Integer.valueOf(promisedStreamId) });
    }
    DefaultHttp2ConnectionDecoder.access$100(this$0).remote().reservePushStream(promisedStreamId, parentStream);
    
    DefaultHttp2ConnectionDecoder.access$300(this$0).onPushPromiseRead(ctx, streamId, promisedStreamId, headers, padding);
  }
  
  public void onGoAwayRead(ChannelHandlerContext ctx, int lastStreamId, long errorCode, ByteBuf debugData)
    throws Http2Exception
  {
    this$0.onGoAwayRead0(ctx, lastStreamId, errorCode, debugData);
  }
  
  public void onWindowUpdateRead(ChannelHandlerContext ctx, int streamId, int windowSizeIncrement)
    throws Http2Exception
  {
    Http2Stream stream = DefaultHttp2ConnectionDecoder.access$100(this$0).stream(streamId);
    if ((stream == null) || (stream.state() == Http2Stream.State.CLOSED) || (streamCreatedAfterGoAwaySent(streamId)))
    {
      verifyStreamMayHaveExisted(streamId);
      return;
    }
    DefaultHttp2ConnectionDecoder.access$600(this$0).flowController().incrementWindowSize(stream, windowSizeIncrement);
    
    DefaultHttp2ConnectionDecoder.access$300(this$0).onWindowUpdateRead(ctx, streamId, windowSizeIncrement);
  }
  
  public void onUnknownFrame(ChannelHandlerContext ctx, byte frameType, int streamId, Http2Flags flags, ByteBuf payload)
    throws Http2Exception
  {
    this$0.onUnknownFrame0(ctx, frameType, streamId, flags, payload);
  }
  
  private boolean shouldIgnoreHeadersOrDataFrame(ChannelHandlerContext ctx, int streamId, Http2Stream stream, String frameName)
    throws Http2Exception
  {
    if (stream == null)
    {
      if (streamCreatedAfterGoAwaySent(streamId))
      {
        DefaultHttp2ConnectionDecoder.access$500().info("{} ignoring {} frame for stream {}. Stream sent after GOAWAY sent", new Object[] { ctx.channel(), frameName, Integer.valueOf(streamId) });
        
        return true;
      }
      throw Http2Exception.streamError(streamId, Http2Error.STREAM_CLOSED, "Received %s frame for an unknown stream %d", new Object[] { frameName, Integer.valueOf(streamId) });
    }
    if ((stream.isResetSent()) || (streamCreatedAfterGoAwaySent(streamId)))
    {
      if (DefaultHttp2ConnectionDecoder.access$500().isInfoEnabled()) {
        DefaultHttp2ConnectionDecoder.access$500().info("{} ignoring {} frame for stream {} {}", new Object[] { ctx.channel(), frameName, "Stream created after GOAWAY sent. Last known stream by peer " + DefaultHttp2ConnectionDecoder.access$100(this$0).remote().lastStreamKnownByPeer() });
      }
      return true;
    }
    return false;
  }
  
  private boolean streamCreatedAfterGoAwaySent(int streamId)
  {
    Http2Connection.Endpoint<?> remote = DefaultHttp2ConnectionDecoder.access$100(this$0).remote();
    return (DefaultHttp2ConnectionDecoder.access$100(this$0).goAwaySent()) && (remote.isValidStreamId(streamId)) && (streamId > remote.lastStreamKnownByPeer());
  }
  
  private void verifyStreamMayHaveExisted(int streamId)
    throws Http2Exception
  {
    if (!DefaultHttp2ConnectionDecoder.access$100(this$0).streamMayHaveExisted(streamId)) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Stream %d does not exist", new Object[] { Integer.valueOf(streamId) });
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2ConnectionDecoder.FrameReadListener
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */