package io.netty.handler.codec.http2;

import io.netty.channel.ChannelHandlerContext;
import io.netty.util.concurrent.EventExecutor;

class DefaultHttp2LocalFlowController$DefaultState
  implements DefaultHttp2LocalFlowController.FlowState
{
  private final Http2Stream stream;
  private int window;
  private int processedWindow;
  private int initialStreamWindowSize;
  private float streamWindowUpdateRatio;
  private int lowerBound;
  private boolean endOfStream;
  
  public DefaultHttp2LocalFlowController$DefaultState(DefaultHttp2LocalFlowController paramDefaultHttp2LocalFlowController, Http2Stream stream, int initialWindowSize)
  {
    this.stream = stream;
    window(initialWindowSize);
    streamWindowUpdateRatio = DefaultHttp2LocalFlowController.access$600(paramDefaultHttp2LocalFlowController);
  }
  
  public void window(int initialWindowSize)
  {
    assert ((DefaultHttp2LocalFlowController.access$400(this$0) == null) || (DefaultHttp2LocalFlowController.access$400(this$0).executor().inEventLoop()));
    window = (processedWindow = initialStreamWindowSize = initialWindowSize);
  }
  
  public int windowSize()
  {
    return window;
  }
  
  public int initialWindowSize()
  {
    return initialStreamWindowSize;
  }
  
  public void endOfStream(boolean endOfStream)
  {
    this.endOfStream = endOfStream;
  }
  
  public float windowUpdateRatio()
  {
    return streamWindowUpdateRatio;
  }
  
  public void windowUpdateRatio(float ratio)
  {
    assert ((DefaultHttp2LocalFlowController.access$400(this$0) == null) || (DefaultHttp2LocalFlowController.access$400(this$0).executor().inEventLoop()));
    streamWindowUpdateRatio = ratio;
  }
  
  public void incrementInitialStreamWindow(int delta)
  {
    int newValue = (int)Math.min(2147483647L, Math.max(0L, initialStreamWindowSize + delta));
    
    delta = newValue - initialStreamWindowSize;
    
    initialStreamWindowSize += delta;
  }
  
  public void incrementFlowControlWindows(int delta)
    throws Http2Exception
  {
    if ((delta > 0) && (window > Integer.MAX_VALUE - delta)) {
      throw Http2Exception.streamError(stream.id(), Http2Error.FLOW_CONTROL_ERROR, "Flow control window overflowed for stream: %d", new Object[] { Integer.valueOf(stream.id()) });
    }
    window += delta;
    processedWindow += delta;
    lowerBound = (delta < 0 ? delta : 0);
  }
  
  public void receiveFlowControlledFrame(int dataLength)
    throws Http2Exception
  {
    assert (dataLength >= 0);
    
    window -= dataLength;
    if (window < lowerBound) {
      throw Http2Exception.streamError(stream.id(), Http2Error.FLOW_CONTROL_ERROR, "Flow control window exceeded for stream: %d", new Object[] { Integer.valueOf(stream.id()) });
    }
  }
  
  private void returnProcessedBytes(int delta)
    throws Http2Exception
  {
    if (processedWindow - delta < window) {
      throw Http2Exception.streamError(stream.id(), Http2Error.INTERNAL_ERROR, "Attempting to return too many bytes for stream %d", new Object[] { Integer.valueOf(stream.id()) });
    }
    processedWindow -= delta;
  }
  
  public boolean consumeBytes(int numBytes)
    throws Http2Exception
  {
    returnProcessedBytes(numBytes);
    return writeWindowUpdateIfNeeded();
  }
  
  public int unconsumedBytes()
  {
    return processedWindow - window;
  }
  
  public boolean writeWindowUpdateIfNeeded()
    throws Http2Exception
  {
    if ((endOfStream) || (initialStreamWindowSize <= 0)) {
      return false;
    }
    int threshold = (int)(initialStreamWindowSize * streamWindowUpdateRatio);
    if (processedWindow <= threshold)
    {
      writeWindowUpdate();
      return true;
    }
    return false;
  }
  
  private void writeWindowUpdate()
    throws Http2Exception
  {
    int deltaWindowSize = initialStreamWindowSize - processedWindow;
    try
    {
      incrementFlowControlWindows(deltaWindowSize);
    }
    catch (Throwable t)
    {
      throw Http2Exception.connectionError(Http2Error.INTERNAL_ERROR, t, "Attempting to return too many bytes for stream %d", new Object[] { Integer.valueOf(stream.id()) });
    }
    DefaultHttp2LocalFlowController.access$700(this$0).writeWindowUpdate(DefaultHttp2LocalFlowController.access$400(this$0), stream.id(), deltaWindowSize, DefaultHttp2LocalFlowController.access$400(this$0).newPromise());
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2LocalFlowController.DefaultState
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */