package io.netty.handler.codec.http2;

import io.netty.util.BooleanSupplier;
import java.util.ArrayDeque;
import java.util.Deque;

final class DefaultHttp2RemoteFlowController$FlowState
  implements StreamByteDistributor.StreamState
{
  private final Http2Stream stream;
  private final Deque<Http2RemoteFlowController.FlowControlled> pendingWriteQueue;
  private int window;
  private int pendingBytes;
  private boolean markedWritable;
  private boolean writing;
  private boolean cancelled;
  private BooleanSupplier isWritableSupplier = new BooleanSupplier()
  {
    public boolean get()
      throws Exception
    {
      return windowSize() > pendingBytes();
    }
  };
  
  DefaultHttp2RemoteFlowController$FlowState(DefaultHttp2RemoteFlowController paramDefaultHttp2RemoteFlowController, Http2Stream stream)
  {
    this.stream = stream;
    pendingWriteQueue = new ArrayDeque(2);
  }
  
  boolean isWritable()
  {
    try
    {
      return isWritableSupplier.get();
    }
    catch (Throwable cause)
    {
      throw new Error("isWritableSupplier should never throw!", cause);
    }
  }
  
  public Http2Stream stream()
  {
    return stream;
  }
  
  boolean markedWritability()
  {
    return markedWritable;
  }
  
  void markedWritability(boolean isWritable)
  {
    markedWritable = isWritable;
  }
  
  public int windowSize()
  {
    return window;
  }
  
  void windowSize(int initialWindowSize)
  {
    window = initialWindowSize;
  }
  
  int writeAllocatedBytes(int allocated)
  {
    int initialAllocated = allocated;
    
    Throwable cause = null;
    int writtenBytes;
    try
    {
      assert (!writing);
      writing = true;
      
      boolean writeOccurred = false;
      Http2RemoteFlowController.FlowControlled frame;
      int maxBytes;
      while ((!cancelled) && ((frame = peek()) != null))
      {
        maxBytes = Math.min(allocated, writableWindow());
        if ((maxBytes <= 0) && (frame.size() > 0)) {
          break;
        }
        writeOccurred = true;
        int initialFrameSize = frame.size();
        try
        {
          frame.write(DefaultHttp2RemoteFlowController.access$500(this$0), Math.max(0, maxBytes));
          if (frame.size() == 0)
          {
            pendingWriteQueue.remove();
            frame.writeComplete();
          }
        }
        finally
        {
          allocated -= initialFrameSize - frame.size();
        }
      }
      if (!writeOccurred)
      {
        int writtenBytes;
        return -1;
      }
    }
    catch (Throwable t)
    {
      int writtenBytes;
      cancelled = true;
      cause = t;
    }
    finally
    {
      int writtenBytes;
      writing = false;
      
      writtenBytes = initialAllocated - allocated;
      
      decrementPendingBytes(writtenBytes, false);
      decrementFlowControlWindow(writtenBytes);
      if (cancelled) {
        cancel(cause);
      }
    }
    return writtenBytes;
  }
  
  int incrementStreamWindow(int delta)
    throws Http2Exception
  {
    if ((delta > 0) && (Integer.MAX_VALUE - delta < window)) {
      throw Http2Exception.streamError(stream.id(), Http2Error.FLOW_CONTROL_ERROR, "Window size overflow for stream: %d", new Object[] { Integer.valueOf(stream.id()) });
    }
    window += delta;
    
    DefaultHttp2RemoteFlowController.access$600(this$0).updateStreamableBytes(this);
    return window;
  }
  
  private int writableWindow()
  {
    return Math.min(window, DefaultHttp2RemoteFlowController.access$700(this$0));
  }
  
  public int pendingBytes()
  {
    return pendingBytes;
  }
  
  void enqueueFrame(Http2RemoteFlowController.FlowControlled frame)
  {
    Http2RemoteFlowController.FlowControlled last = (Http2RemoteFlowController.FlowControlled)pendingWriteQueue.peekLast();
    if (last == null)
    {
      enqueueFrameWithoutMerge(frame);
      return;
    }
    int lastSize = last.size();
    if (last.merge(DefaultHttp2RemoteFlowController.access$500(this$0), frame))
    {
      incrementPendingBytes(last.size() - lastSize, true);
      return;
    }
    enqueueFrameWithoutMerge(frame);
  }
  
  private void enqueueFrameWithoutMerge(Http2RemoteFlowController.FlowControlled frame)
  {
    pendingWriteQueue.offer(frame);
    
    incrementPendingBytes(frame.size(), true);
  }
  
  public boolean hasFrame()
  {
    return !pendingWriteQueue.isEmpty();
  }
  
  private Http2RemoteFlowController.FlowControlled peek()
  {
    return (Http2RemoteFlowController.FlowControlled)pendingWriteQueue.peek();
  }
  
  void cancel()
  {
    cancel(null);
  }
  
  private void cancel(Throwable cause)
  {
    cancelled = true;
    if (writing) {
      return;
    }
    for (;;)
    {
      Http2RemoteFlowController.FlowControlled frame = (Http2RemoteFlowController.FlowControlled)pendingWriteQueue.poll();
      if (frame == null) {
        break;
      }
      writeError(frame, Http2Exception.streamError(stream.id(), Http2Error.INTERNAL_ERROR, cause, "Stream closed before write could take place", new Object[0]));
    }
    DefaultHttp2RemoteFlowController.access$600(this$0).updateStreamableBytes(this);
    
    isWritableSupplier = BooleanSupplier.FALSE_SUPPLIER;
    DefaultHttp2RemoteFlowController.access$300(this$0).stateCancelled(this);
  }
  
  private void incrementPendingBytes(int numBytes, boolean updateStreamableBytes)
  {
    pendingBytes += numBytes;
    DefaultHttp2RemoteFlowController.access$300(this$0).incrementPendingBytes(numBytes);
    if (updateStreamableBytes) {
      DefaultHttp2RemoteFlowController.access$600(this$0).updateStreamableBytes(this);
    }
  }
  
  private void decrementPendingBytes(int bytes, boolean updateStreamableBytes)
  {
    incrementPendingBytes(-bytes, updateStreamableBytes);
  }
  
  private void decrementFlowControlWindow(int bytes)
  {
    try
    {
      int negativeBytes = -bytes;
      DefaultHttp2RemoteFlowController.access$800(this$0).incrementStreamWindow(negativeBytes);
      incrementStreamWindow(negativeBytes);
    }
    catch (Http2Exception e)
    {
      throw new IllegalStateException("Invalid window state when writing frame: " + e.getMessage(), e);
    }
  }
  
  private void writeError(Http2RemoteFlowController.FlowControlled frame, Http2Exception cause)
  {
    assert (DefaultHttp2RemoteFlowController.access$500(this$0) != null);
    decrementPendingBytes(frame.size(), true);
    frame.error(DefaultHttp2RemoteFlowController.access$500(this$0), cause);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2RemoteFlowController.FlowState
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */