package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import java.util.List;

final class Http2ConnectionHandler$PrefaceDecoder
  extends Http2ConnectionHandler.BaseDecoder
{
  private ByteBuf clientPrefaceString;
  private boolean prefaceSent;
  
  public Http2ConnectionHandler$PrefaceDecoder(Http2ConnectionHandler paramHttp2ConnectionHandler, ChannelHandlerContext ctx)
  {
    super(paramHttp2ConnectionHandler, null);
    clientPrefaceString = Http2ConnectionHandler.access$200(Http2ConnectionHandler.access$100(paramHttp2ConnectionHandler).connection());
    
    sendPreface(ctx);
  }
  
  public boolean prefaceSent()
  {
    return prefaceSent;
  }
  
  public void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    try
    {
      if ((ctx.channel().isActive()) && (readClientPrefaceString(in)) && (verifyFirstFrameIsSettings(in)))
      {
        Http2ConnectionHandler.access$302(this$0, new Http2ConnectionHandler.FrameDecoder(this$0, null));
        Http2ConnectionHandler.access$300(this$0).decode(ctx, in, out);
      }
    }
    catch (Throwable e)
    {
      this$0.onError(ctx, e);
    }
  }
  
  public void channelActive(ChannelHandlerContext ctx)
    throws Exception
  {
    sendPreface(ctx);
  }
  
  public void channelInactive(ChannelHandlerContext ctx)
    throws Exception
  {
    cleanup();
    super.channelInactive(ctx);
  }
  
  public void handlerRemoved(ChannelHandlerContext ctx)
    throws Exception
  {
    cleanup();
  }
  
  private void cleanup()
  {
    if (clientPrefaceString != null)
    {
      clientPrefaceString.release();
      clientPrefaceString = null;
    }
  }
  
  private boolean readClientPrefaceString(ByteBuf in)
    throws Http2Exception
  {
    if (clientPrefaceString == null) {
      return true;
    }
    int prefaceRemaining = clientPrefaceString.readableBytes();
    int bytesRead = Math.min(in.readableBytes(), prefaceRemaining);
    if ((bytesRead == 0) || (!ByteBufUtil.equals(in, in.readerIndex(), clientPrefaceString, clientPrefaceString.readerIndex(), bytesRead)))
    {
      String receivedBytes = ByteBufUtil.hexDump(in, in.readerIndex(), Math.min(in.readableBytes(), clientPrefaceString.readableBytes()));
      
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "HTTP/2 client preface string missing or corrupt. Hex dump for received bytes: %s", new Object[] { receivedBytes });
    }
    in.skipBytes(bytesRead);
    clientPrefaceString.skipBytes(bytesRead);
    if (!clientPrefaceString.isReadable())
    {
      clientPrefaceString.release();
      clientPrefaceString = null;
      return true;
    }
    return false;
  }
  
  private boolean verifyFirstFrameIsSettings(ByteBuf in)
    throws Http2Exception
  {
    if (in.readableBytes() < 5) {
      return false;
    }
    short frameType = in.getUnsignedByte(in.readerIndex() + 3);
    short flags = in.getUnsignedByte(in.readerIndex() + 4);
    if ((frameType != 4) || ((flags & 0x1) != 0)) {
      throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "First received frame was not SETTINGS. Hex dump for first 5 bytes: %s", new Object[] { ByteBufUtil.hexDump(in, in.readerIndex(), 5) });
    }
    return true;
  }
  
  private void sendPreface(ChannelHandlerContext ctx)
  {
    if ((prefaceSent) || (!ctx.channel().isActive())) {
      return;
    }
    prefaceSent = true;
    if (!this$0.connection().isServer()) {
      ctx.write(Http2CodecUtil.connectionPrefaceBuf()).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);
    }
    Http2ConnectionHandler.access$100(this$0).writeSettings(ctx, Http2ConnectionHandler.access$500(this$0), ctx.newPromise()).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.Http2ConnectionHandler.PrefaceDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */