package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageCodec;
import io.netty.handler.codec.http.DefaultHttpContent;
import io.netty.handler.codec.http.DefaultLastHttpContent;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpContent;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.handler.codec.http.HttpObject;
import io.netty.handler.codec.http.HttpResponse;
import io.netty.handler.codec.http.HttpVersion;
import io.netty.handler.codec.http.LastHttpContent;
import io.netty.util.ReferenceCountUtil;
import java.util.List;

public class Http2ServerDowngrader
  extends MessageToMessageCodec<Http2StreamFrame, HttpObject>
{
  private final boolean validateHeaders;
  
  public Http2ServerDowngrader(boolean validateHeaders)
  {
    this.validateHeaders = validateHeaders;
  }
  
  public Http2ServerDowngrader()
  {
    this(true);
  }
  
  public boolean acceptInboundMessage(Object msg)
    throws Exception
  {
    return ((msg instanceof Http2HeadersFrame)) || ((msg instanceof Http2DataFrame));
  }
  
  protected void decode(ChannelHandlerContext ctx, Http2StreamFrame frame, List<Object> out)
    throws Exception
  {
    if ((frame instanceof Http2HeadersFrame))
    {
      int id = 0;
      Http2HeadersFrame headersFrame = (Http2HeadersFrame)frame;
      Http2Headers headers = headersFrame.headers();
      if (headersFrame.isEndStream())
      {
        if (headers.method() == null)
        {
          LastHttpContent last = new DefaultLastHttpContent(Unpooled.EMPTY_BUFFER, validateHeaders);
          HttpConversionUtil.addHttp2ToHttpHeaders(id, headers, last.trailingHeaders(), HttpVersion.HTTP_1_1, true, true);
          
          out.add(last);
        }
        else
        {
          FullHttpRequest full = HttpConversionUtil.toFullHttpRequest(id, headers, ctx.alloc(), validateHeaders);
          
          out.add(full);
        }
      }
      else {
        out.add(HttpConversionUtil.toHttpRequest(id, headersFrame.headers(), validateHeaders));
      }
    }
    else if ((frame instanceof Http2DataFrame))
    {
      Http2DataFrame dataFrame = (Http2DataFrame)frame;
      if (dataFrame.isEndStream()) {
        out.add(new DefaultLastHttpContent(dataFrame.content(), validateHeaders));
      } else {
        out.add(new DefaultHttpContent(dataFrame.content()));
      }
    }
    ReferenceCountUtil.retain(frame);
  }
  
  private void encodeLastContent(LastHttpContent last, List<Object> out)
  {
    boolean needFiller = (!(last instanceof FullHttpResponse)) && (last.trailingHeaders().isEmpty());
    if ((last.content().isReadable()) || (needFiller)) {
      out.add(new DefaultHttp2DataFrame(last.content(), last.trailingHeaders().isEmpty()));
    }
    if (!last.trailingHeaders().isEmpty())
    {
      Http2Headers headers = HttpConversionUtil.toHttp2Headers(last.trailingHeaders(), validateHeaders);
      out.add(new DefaultHttp2HeadersFrame(headers, true));
    }
  }
  
  protected void encode(ChannelHandlerContext ctx, HttpObject obj, List<Object> out)
    throws Exception
  {
    if ((obj instanceof HttpResponse))
    {
      Http2Headers headers = HttpConversionUtil.toHttp2Headers((HttpResponse)obj, validateHeaders);
      boolean noMoreFrames = false;
      if ((obj instanceof FullHttpResponse))
      {
        FullHttpResponse full = (FullHttpResponse)obj;
        noMoreFrames = (!full.content().isReadable()) && (full.trailingHeaders().isEmpty());
      }
      out.add(new DefaultHttp2HeadersFrame(headers, noMoreFrames));
    }
    if ((obj instanceof LastHttpContent))
    {
      LastHttpContent last = (LastHttpContent)obj;
      encodeLastContent(last, out);
    }
    else if ((obj instanceof HttpContent))
    {
      HttpContent cont = (HttpContent)obj;
      out.add(new DefaultHttp2DataFrame(cont.content(), false));
    }
    ReferenceCountUtil.retain(obj);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.Http2ServerDowngrader
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */