package io.netty.handler.ssl;

import io.netty.util.internal.EmptyArrays;
import java.security.Principal;
import java.security.cert.Certificate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSessionBindingEvent;
import javax.net.ssl.SSLSessionBindingListener;
import javax.net.ssl.SSLSessionContext;
import org.apache.tomcat.jni.SSL;

final class ReferenceCountedOpenSslEngine$OpenSslSession
  implements SSLSession, ApplicationProtocolAccessor
{
  private final OpenSslSessionContext sessionContext;
  private javax.security.cert.X509Certificate[] x509PeerCerts;
  private String protocol;
  private String applicationProtocol;
  private Certificate[] peerCerts;
  private String cipher;
  private byte[] id;
  private long creationTime;
  private Map<String, Object> values;
  
  ReferenceCountedOpenSslEngine$OpenSslSession(ReferenceCountedOpenSslEngine paramReferenceCountedOpenSslEngine, OpenSslSessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }
  
  public byte[] getId()
  {
    synchronized (this$0)
    {
      if (id == null) {
        return EmptyArrays.EMPTY_BYTES;
      }
      return (byte[])id.clone();
    }
  }
  
  public SSLSessionContext getSessionContext()
  {
    return sessionContext;
  }
  
  public long getCreationTime()
  {
    synchronized (this$0)
    {
      if ((creationTime == 0L) && (!ReferenceCountedOpenSslEngine.access$100(this$0))) {
        creationTime = (SSL.getTime(ReferenceCountedOpenSslEngine.access$200(this$0)) * 1000L);
      }
    }
    return creationTime;
  }
  
  public long getLastAccessedTime()
  {
    long lastAccessed = ReferenceCountedOpenSslEngine.access$300(this$0);
    
    return lastAccessed == -1L ? getCreationTime() : lastAccessed;
  }
  
  public void invalidate()
  {
    synchronized (this$0)
    {
      if (!ReferenceCountedOpenSslEngine.access$100(this$0)) {
        SSL.setTimeout(ReferenceCountedOpenSslEngine.access$200(this$0), 0L);
      }
    }
  }
  
  public boolean isValid()
  {
    synchronized (this$0)
    {
      if (!ReferenceCountedOpenSslEngine.access$100(this$0)) {
        return System.currentTimeMillis() - SSL.getTimeout(ReferenceCountedOpenSslEngine.access$200(this$0)) * 1000L < SSL.getTime(ReferenceCountedOpenSslEngine.access$200(this$0)) * 1000L;
      }
    }
    return false;
  }
  
  public void putValue(String name, Object value)
  {
    if (name == null) {
      throw new NullPointerException("name");
    }
    if (value == null) {
      throw new NullPointerException("value");
    }
    Map<String, Object> values = this.values;
    if (values == null) {
      values = this.values = new HashMap(2);
    }
    Object old = values.put(name, value);
    if ((value instanceof SSLSessionBindingListener)) {
      ((SSLSessionBindingListener)value).valueBound(new SSLSessionBindingEvent(this, name));
    }
    notifyUnbound(old, name);
  }
  
  public Object getValue(String name)
  {
    if (name == null) {
      throw new NullPointerException("name");
    }
    if (values == null) {
      return null;
    }
    return values.get(name);
  }
  
  public void removeValue(String name)
  {
    if (name == null) {
      throw new NullPointerException("name");
    }
    Map<String, Object> values = this.values;
    if (values == null) {
      return;
    }
    Object old = values.remove(name);
    notifyUnbound(old, name);
  }
  
  public String[] getValueNames()
  {
    Map<String, Object> values = this.values;
    if ((values == null) || (values.isEmpty())) {
      return EmptyArrays.EMPTY_STRINGS;
    }
    return (String[])values.keySet().toArray(new String[values.size()]);
  }
  
  private void notifyUnbound(Object value, String name)
  {
    if ((value instanceof SSLSessionBindingListener)) {
      ((SSLSessionBindingListener)value).valueUnbound(new SSLSessionBindingEvent(this, name));
    }
  }
  
  void handshakeFinished()
    throws SSLException
  {
    synchronized (this$0)
    {
      if (!ReferenceCountedOpenSslEngine.access$100(this$0))
      {
        id = SSL.getSessionId(ReferenceCountedOpenSslEngine.access$200(this$0));
        cipher = ReferenceCountedOpenSslEngine.access$400(this$0, SSL.getCipherForSSL(ReferenceCountedOpenSslEngine.access$200(this$0)));
        protocol = SSL.getVersion(ReferenceCountedOpenSslEngine.access$200(this$0));
        
        initPeerCerts();
        selectApplicationProtocol();
        
        ReferenceCountedOpenSslEngine.access$502(this$0, ReferenceCountedOpenSslEngine.HandshakeState.FINISHED);
      }
      else
      {
        throw new SSLException("Already closed");
      }
    }
  }
  
  private void initPeerCerts()
  {
    byte[][] chain = SSL.getPeerCertChain(ReferenceCountedOpenSslEngine.access$200(this$0));
    byte[] clientCert;
    byte[] clientCert;
    if (!ReferenceCountedOpenSslEngine.access$600(this$0)) {
      clientCert = SSL.getPeerCertificate(ReferenceCountedOpenSslEngine.access$200(this$0));
    } else {
      clientCert = null;
    }
    if ((chain == null) && (clientCert == null))
    {
      this.peerCerts = ReferenceCountedOpenSslEngine.access$700();
      x509PeerCerts = ReferenceCountedOpenSslEngine.access$800();
    }
    else
    {
      int len = chain != null ? chain.length : 0;
      
      int i = 0;
      Certificate[] peerCerts;
      if (clientCert != null)
      {
        len++;
        Certificate[] peerCerts = new Certificate[len];
        peerCerts[(i++)] = new OpenSslX509Certificate(clientCert);
      }
      else
      {
        peerCerts = new Certificate[len];
      }
      if (chain != null)
      {
        javax.security.cert.X509Certificate[] pCerts = new javax.security.cert.X509Certificate[chain.length];
        for (int a = 0; a < pCerts.length; a++)
        {
          byte[] bytes = chain[a];
          pCerts[a] = new OpenSslJavaxX509Certificate(bytes);
          peerCerts[i] = new OpenSslX509Certificate(bytes);i++;
        }
        x509PeerCerts = pCerts;
      }
      else
      {
        x509PeerCerts = ReferenceCountedOpenSslEngine.access$800();
      }
      this.peerCerts = peerCerts;
    }
  }
  
  private void selectApplicationProtocol()
    throws SSLException
  {
    ApplicationProtocolConfig.SelectedListenerFailureBehavior behavior = ReferenceCountedOpenSslEngine.access$900(this$0).selectedListenerFailureBehavior();
    List<String> protocols = ReferenceCountedOpenSslEngine.access$900(this$0).protocols();
    switch (ReferenceCountedOpenSslEngine.2.$SwitchMap$io$netty$handler$ssl$ApplicationProtocolConfig$Protocol[ReferenceCountedOpenSslEngine.access$900(this$0).protocol().ordinal()])
    {
    case 1: 
      break;
    case 2: 
      String applicationProtocol = SSL.getAlpnSelected(ReferenceCountedOpenSslEngine.access$200(this$0));
      if (applicationProtocol != null) {
        this.applicationProtocol = selectApplicationProtocol(protocols, behavior, applicationProtocol);
      }
      break;
    case 3: 
      String applicationProtocol = SSL.getNextProtoNegotiated(ReferenceCountedOpenSslEngine.access$200(this$0));
      if (applicationProtocol != null) {
        this.applicationProtocol = selectApplicationProtocol(protocols, behavior, applicationProtocol);
      }
      break;
    case 4: 
      String applicationProtocol = SSL.getAlpnSelected(ReferenceCountedOpenSslEngine.access$200(this$0));
      if (applicationProtocol == null) {
        applicationProtocol = SSL.getNextProtoNegotiated(ReferenceCountedOpenSslEngine.access$200(this$0));
      }
      if (applicationProtocol != null) {
        this.applicationProtocol = selectApplicationProtocol(protocols, behavior, applicationProtocol);
      }
      break;
    default: 
      throw new Error();
    }
  }
  
  private String selectApplicationProtocol(List<String> protocols, ApplicationProtocolConfig.SelectedListenerFailureBehavior behavior, String applicationProtocol)
    throws SSLException
  {
    if (behavior == ApplicationProtocolConfig.SelectedListenerFailureBehavior.ACCEPT) {
      return applicationProtocol;
    }
    int size = protocols.size();
    assert (size > 0);
    if (protocols.contains(applicationProtocol)) {
      return applicationProtocol;
    }
    if (behavior == ApplicationProtocolConfig.SelectedListenerFailureBehavior.CHOOSE_MY_LAST_PROTOCOL) {
      return (String)protocols.get(size - 1);
    }
    throw new SSLException("unknown protocol " + applicationProtocol);
  }
  
  public Certificate[] getPeerCertificates()
    throws SSLPeerUnverifiedException
  {
    synchronized (this$0)
    {
      if ((peerCerts == null) || (peerCerts.length == 0)) {
        throw new SSLPeerUnverifiedException("peer not verified");
      }
      return peerCerts;
    }
  }
  
  public Certificate[] getLocalCertificates()
  {
    if (ReferenceCountedOpenSslEngine.access$1000(this$0) == null) {
      return null;
    }
    return (Certificate[])ReferenceCountedOpenSslEngine.access$1000(this$0).clone();
  }
  
  public javax.security.cert.X509Certificate[] getPeerCertificateChain()
    throws SSLPeerUnverifiedException
  {
    synchronized (this$0)
    {
      if ((x509PeerCerts == null) || (x509PeerCerts.length == 0)) {
        throw new SSLPeerUnverifiedException("peer not verified");
      }
      return x509PeerCerts;
    }
  }
  
  public Principal getPeerPrincipal()
    throws SSLPeerUnverifiedException
  {
    Certificate[] peer = getPeerCertificates();
    
    return ((java.security.cert.X509Certificate)peer[0]).getSubjectX500Principal();
  }
  
  public Principal getLocalPrincipal()
  {
    Certificate[] local = ReferenceCountedOpenSslEngine.access$1000(this$0);
    if ((local == null) || (local.length == 0)) {
      return null;
    }
    return ((java.security.cert.X509Certificate)local[0]).getIssuerX500Principal();
  }
  
  public String getCipherSuite()
  {
    synchronized (this$0)
    {
      if (cipher == null) {
        return "SSL_NULL_WITH_NULL_NULL";
      }
      return cipher;
    }
  }
  
  public String getProtocol()
  {
    String protocol = this.protocol;
    if (protocol == null) {
      synchronized (this$0)
      {
        if (!ReferenceCountedOpenSslEngine.access$100(this$0)) {
          protocol = SSL.getVersion(ReferenceCountedOpenSslEngine.access$200(this$0));
        } else {
          protocol = "";
        }
      }
    }
    return protocol;
  }
  
  /* Error */
  public String getApplicationProtocol()
  {
    // Byte code:
    //   0: aload_0
    //   1: getfield 1	io/netty/handler/ssl/ReferenceCountedOpenSslEngine$OpenSslSession:this$0	Lio/netty/handler/ssl/ReferenceCountedOpenSslEngine;
    //   4: dup
    //   5: astore_1
    //   6: monitorenter
    //   7: aload_0
    //   8: getfield 78	io/netty/handler/ssl/ReferenceCountedOpenSslEngine$OpenSslSession:applicationProtocol	Ljava/lang/String;
    //   11: aload_1
    //   12: monitorexit
    //   13: areturn
    //   14: astore_2
    //   15: aload_1
    //   16: monitorexit
    //   17: aload_2
    //   18: athrow
    // Line number table:
    //   Java source line #1929	-> byte code offset #0
    //   Java source line #1930	-> byte code offset #7
    //   Java source line #1931	-> byte code offset #14
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	19	0	this	OpenSslSession
    //   5	11	1	Ljava/lang/Object;	Object
    //   14	4	2	localObject1	Object
    // Exception table:
    //   from	to	target	type
    //   7	13	14	finally
    //   14	17	14	finally
  }
  
  public String getPeerHost()
  {
    return this$0.getPeerHost();
  }
  
  public int getPeerPort()
  {
    return this$0.getPeerPort();
  }
  
  public int getPacketBufferSize()
  {
    return 18713;
  }
  
  public int getApplicationBufferSize()
  {
    return 16384;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.ssl.ReferenceCountedOpenSslEngine.OpenSslSession
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */