package io.netty.util;

import io.netty.util.concurrent.FastThreadLocal;
import java.lang.ref.WeakReference;
import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

final class Recycler$Stack<T>
{
  final Recycler<T> parent;
  final Thread thread;
  final AtomicInteger availableSharedCapacity;
  final int maxDelayedQueues;
  private final int maxCapacity;
  private final int ratioMask;
  private Recycler.DefaultHandle<?>[] elements;
  private int size;
  private int handleRecycleCount = -1;
  private Recycler.WeakOrderQueue cursor;
  private Recycler.WeakOrderQueue prev;
  private volatile Recycler.WeakOrderQueue head;
  
  Recycler$Stack(Recycler<T> parent, Thread thread, int maxCapacity, int maxSharedCapacityFactor, int ratioMask, int maxDelayedQueues)
  {
    this.parent = parent;
    this.thread = thread;
    this.maxCapacity = maxCapacity;
    availableSharedCapacity = new AtomicInteger(Math.max(maxCapacity / maxSharedCapacityFactor, Recycler.access$800()));
    elements = new Recycler.DefaultHandle[Math.min(Recycler.access$1700(), maxCapacity)];
    this.ratioMask = ratioMask;
    this.maxDelayedQueues = maxDelayedQueues;
  }
  
  int increaseCapacity(int expectedCapacity)
  {
    int newCapacity = elements.length;
    int maxCapacity = this.maxCapacity;
    do
    {
      newCapacity <<= 1;
    } while ((newCapacity < expectedCapacity) && (newCapacity < maxCapacity));
    newCapacity = Math.min(newCapacity, maxCapacity);
    if (newCapacity != elements.length) {
      elements = ((Recycler.DefaultHandle[])Arrays.copyOf(elements, newCapacity));
    }
    return newCapacity;
  }
  
  Recycler.DefaultHandle<T> pop()
  {
    int size = this.size;
    if (size == 0)
    {
      if (!scavenge()) {
        return null;
      }
      size = this.size;
    }
    size--;
    Recycler.DefaultHandle ret = elements[size];
    elements[size] = null;
    if (Recycler.DefaultHandle.access$1200(ret) != Recycler.DefaultHandle.access$1600(ret)) {
      throw new IllegalStateException("recycled multiple times");
    }
    Recycler.DefaultHandle.access$1602(ret, 0);
    Recycler.DefaultHandle.access$1202(ret, 0);
    this.size = size;
    return ret;
  }
  
  boolean scavenge()
  {
    if (scavengeSome()) {
      return true;
    }
    prev = null;
    cursor = head;
    return false;
  }
  
  boolean scavengeSome()
  {
    Recycler.WeakOrderQueue cursor = this.cursor;
    if (cursor == null)
    {
      cursor = head;
      if (cursor == null) {
        return false;
      }
    }
    boolean success = false;
    Recycler.WeakOrderQueue prev = this.prev;
    do
    {
      if (cursor.transfer(this))
      {
        success = true;
        break;
      }
      Recycler.WeakOrderQueue next = Recycler.WeakOrderQueue.access$1800(cursor);
      if (Recycler.WeakOrderQueue.access$1900(cursor).get() == null)
      {
        if (cursor.hasFinalData()) {
          while (cursor.transfer(this)) {
            success = true;
          }
        }
        if (prev != null) {
          Recycler.WeakOrderQueue.access$1802(prev, next);
        }
      }
      else
      {
        prev = cursor;
      }
      cursor = next;
    } while ((cursor != null) && (!success));
    this.prev = prev;
    this.cursor = cursor;
    return success;
  }
  
  void push(Recycler.DefaultHandle<?> item)
  {
    Thread currentThread = Thread.currentThread();
    if (thread == currentThread) {
      pushNow(item);
    } else {
      pushLater(item, currentThread);
    }
  }
  
  private void pushNow(Recycler.DefaultHandle<?> item)
  {
    if ((Recycler.DefaultHandle.access$1600(item) | Recycler.DefaultHandle.access$1200(item)) != 0) {
      throw new IllegalStateException("recycled already");
    }
    Recycler.DefaultHandle.access$1602(item, Recycler.DefaultHandle.access$1202(item, Recycler.access$2000()));
    
    int size = this.size;
    if ((size >= maxCapacity) || (dropHandle(item))) {
      return;
    }
    if (size == elements.length) {
      elements = ((Recycler.DefaultHandle[])Arrays.copyOf(elements, Math.min(size << 1, maxCapacity)));
    }
    elements[size] = item;
    this.size = (size + 1);
  }
  
  private void pushLater(Recycler.DefaultHandle<?> item, Thread thread)
  {
    Map<Stack<?>, Recycler.WeakOrderQueue> delayedRecycled = (Map)Recycler.access$2100().get();
    Recycler.WeakOrderQueue queue = (Recycler.WeakOrderQueue)delayedRecycled.get(this);
    if (queue == null)
    {
      if (delayedRecycled.size() >= maxDelayedQueues)
      {
        delayedRecycled.put(this, Recycler.WeakOrderQueue.DUMMY);
        return;
      }
      if ((queue = Recycler.WeakOrderQueue.allocate(this, thread)) == null) {
        return;
      }
      delayedRecycled.put(this, queue);
    }
    else if (queue == Recycler.WeakOrderQueue.DUMMY)
    {
      return;
    }
    queue.add(item);
  }
  
  boolean dropHandle(Recycler.DefaultHandle<?> handle)
  {
    if (!hasBeenRecycled)
    {
      if ((++handleRecycleCount & ratioMask) != 0) {
        return true;
      }
      hasBeenRecycled = true;
    }
    return false;
  }
  
  Recycler.DefaultHandle<T> newHandle()
  {
    return new Recycler.DefaultHandle(this);
  }
}

/* Location:
 * Qualified Name:     io.netty.util.Recycler.Stack
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */