package io.netty.util;

import java.lang.ref.WeakReference;
import java.util.concurrent.atomic.AtomicInteger;

final class Recycler$WeakOrderQueue
{
  static final WeakOrderQueue DUMMY = new WeakOrderQueue();
  private Link head;
  private Link tail;
  private WeakOrderQueue next;
  private final WeakReference<Thread> owner;
  
  private static final class Link
    extends AtomicInteger
  {
    private final Recycler.DefaultHandle<?>[] elements = new Recycler.DefaultHandle[Recycler.access$800()];
    private int readIndex;
    private Link next;
  }
  
  private final int id = Recycler.access$900().getAndIncrement();
  private final AtomicInteger availableSharedCapacity;
  
  private Recycler$WeakOrderQueue()
  {
    owner = null;
    availableSharedCapacity = null;
  }
  
  private Recycler$WeakOrderQueue(Recycler.Stack<?> stack, Thread thread)
  {
    head = (tail = new Link(null));
    owner = new WeakReference(thread);
    synchronized (stack)
    {
      next = Recycler.Stack.access$1100(stack);
      Recycler.Stack.access$1102(stack, this);
    }
    availableSharedCapacity = availableSharedCapacity;
  }
  
  static WeakOrderQueue allocate(Recycler.Stack<?> stack, Thread thread)
  {
    return reserveSpace(availableSharedCapacity, Recycler.access$800()) ? new WeakOrderQueue(stack, thread) : null;
  }
  
  private static boolean reserveSpace(AtomicInteger availableSharedCapacity, int space)
  {
    assert (space >= 0);
    for (;;)
    {
      int available = availableSharedCapacity.get();
      if (available < space) {
        return false;
      }
      if (availableSharedCapacity.compareAndSet(available, available - space)) {
        return true;
      }
    }
  }
  
  private void reclaimSpace(int space)
  {
    assert (space >= 0);
    availableSharedCapacity.addAndGet(space);
  }
  
  void add(Recycler.DefaultHandle<?> handle)
  {
    Recycler.DefaultHandle.access$1202(handle, id);
    
    Link tail = this.tail;
    int writeIndex;
    if ((writeIndex = tail.get()) == Recycler.access$800())
    {
      if (!reserveSpace(availableSharedCapacity, Recycler.access$800())) {
        return;
      }
      this.tail = (tail = next = new Link(null));
      
      writeIndex = tail.get();
    }
    elements[writeIndex] = handle;
    Recycler.DefaultHandle.access$502(handle, null);
    
    tail.lazySet(writeIndex + 1);
  }
  
  boolean hasFinalData()
  {
    return tail.readIndex != tail.get();
  }
  
  boolean transfer(Recycler.Stack<?> dst)
  {
    Link head = this.head;
    if (head == null) {
      return false;
    }
    if (readIndex == Recycler.access$800())
    {
      if (next == null) {
        return false;
      }
      this.head = (head = next);
    }
    int srcStart = readIndex;
    int srcEnd = head.get();
    int srcSize = srcEnd - srcStart;
    if (srcSize == 0) {
      return false;
    }
    int dstSize = Recycler.Stack.access$700(dst);
    int expectedCapacity = dstSize + srcSize;
    if (expectedCapacity > Recycler.Stack.access$600(dst).length)
    {
      int actualCapacity = dst.increaseCapacity(expectedCapacity);
      srcEnd = Math.min(srcStart + actualCapacity - dstSize, srcEnd);
    }
    if (srcStart != srcEnd)
    {
      Recycler.DefaultHandle[] srcElems = elements;
      Recycler.DefaultHandle[] dstElems = Recycler.Stack.access$600(dst);
      int newDstSize = dstSize;
      for (int i = srcStart; i < srcEnd; i++)
      {
        Recycler.DefaultHandle element = srcElems[i];
        if (Recycler.DefaultHandle.access$1600(element) == 0) {
          Recycler.DefaultHandle.access$1602(element, Recycler.DefaultHandle.access$1200(element));
        } else if (Recycler.DefaultHandle.access$1600(element) != Recycler.DefaultHandle.access$1200(element)) {
          throw new IllegalStateException("recycled already");
        }
        srcElems[i] = null;
        if (!dst.dropHandle(element))
        {
          Recycler.DefaultHandle.access$502(element, dst);
          dstElems[(newDstSize++)] = element;
        }
      }
      if ((srcEnd == Recycler.access$800()) && (next != null))
      {
        reclaimSpace(Recycler.access$800());
        
        this.head = next;
      }
      readIndex = srcEnd;
      if (Recycler.Stack.access$700(dst) == newDstSize) {
        return false;
      }
      Recycler.Stack.access$702(dst, newDstSize);
      return true;
    }
    return false;
  }
  
  protected void finalize()
    throws Throwable
  {
    try
    {
      super.finalize();
    }
    finally
    {
      Link link;
      Link link = head;
      while (link != null)
      {
        reclaimSpace(Recycler.access$800());
        link = next;
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.util.Recycler.WeakOrderQueue
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */