package io.netty.bootstrap;

import io.netty.channel.Channel;
import io.netty.channel.Channel.Unsafe;
import io.netty.channel.ChannelConfig;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoop;
import io.netty.channel.EventLoopGroup;
import io.netty.util.Attribute;
import io.netty.util.AttributeKey;
import io.netty.util.internal.logging.InternalLogger;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

class ServerBootstrap$ServerBootstrapAcceptor
  extends ChannelInboundHandlerAdapter
{
  private final EventLoopGroup childGroup;
  private final ChannelHandler childHandler;
  private final Map.Entry<ChannelOption<?>, Object>[] childOptions;
  private final Map.Entry<AttributeKey<?>, Object>[] childAttrs;
  
  ServerBootstrap$ServerBootstrapAcceptor(EventLoopGroup childGroup, ChannelHandler childHandler, Map.Entry<ChannelOption<?>, Object>[] childOptions, Map.Entry<AttributeKey<?>, Object>[] childAttrs)
  {
    this.childGroup = childGroup;
    this.childHandler = childHandler;
    this.childOptions = childOptions;
    this.childAttrs = childAttrs;
  }
  
  public void channelRead(ChannelHandlerContext ctx, Object msg)
  {
    final Channel child = (Channel)msg;
    
    child.pipeline().addLast(new ChannelHandler[] { childHandler });
    for (Map.Entry<ChannelOption<?>, Object> e : childOptions) {
      try
      {
        if (!child.config().setOption((ChannelOption)e.getKey(), e.getValue())) {
          ServerBootstrap.access$100().warn("Unknown channel option: " + e);
        }
      }
      catch (Throwable t)
      {
        ServerBootstrap.access$100().warn("Failed to set a channel option: " + child, t);
      }
    }
    for (Map.Entry<AttributeKey<?>, Object> e : childAttrs) {
      child.attr((AttributeKey)e.getKey()).set(e.getValue());
    }
    try
    {
      childGroup.register(child).addListener(new ChannelFutureListener()
      {
        public void operationComplete(ChannelFuture future)
          throws Exception
        {
          if (!future.isSuccess()) {
            ServerBootstrap.ServerBootstrapAcceptor.forceClose(child, future.cause());
          }
        }
      });
    }
    catch (Throwable t)
    {
      forceClose(child, t);
    }
  }
  
  private static void forceClose(Channel child, Throwable t)
  {
    child.unsafe().closeForcibly();
    ServerBootstrap.access$100().warn("Failed to register an accepted channel: " + child, t);
  }
  
  public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause)
    throws Exception
  {
    final ChannelConfig config = ctx.channel().config();
    if (config.isAutoRead())
    {
      config.setAutoRead(false);
      ctx.channel().eventLoop().schedule(new Runnable()
      {
        public void run()
        {
          config.setAutoRead(true);
        }
      }, 1L, TimeUnit.SECONDS);
    }
    ctx.fireExceptionCaught(cause);
  }
}

/* Location:
 * Qualified Name:     io.netty.bootstrap.ServerBootstrap.ServerBootstrapAcceptor
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */