package io.netty.buffer;

import io.netty.util.internal.PlatformDependent;
import java.nio.Buffer;
import java.nio.ByteBuffer;

final class PoolArena$DirectArena
  extends PoolArena<ByteBuffer>
{
  PoolArena$DirectArena(PooledByteBufAllocator parent, int pageSize, int maxOrder, int pageShifts, int chunkSize)
  {
    super(parent, pageSize, maxOrder, pageShifts, chunkSize);
  }
  
  boolean isDirect()
  {
    return true;
  }
  
  protected PoolChunk<ByteBuffer> newChunk(int pageSize, int maxOrder, int pageShifts, int chunkSize)
  {
    return new PoolChunk(this, allocateDirect(chunkSize), pageSize, maxOrder, pageShifts, chunkSize);
  }
  
  protected PoolChunk<ByteBuffer> newUnpooledChunk(int capacity)
  {
    return new PoolChunk(this, allocateDirect(capacity), capacity);
  }
  
  private static ByteBuffer allocateDirect(int capacity)
  {
    return PlatformDependent.useDirectBufferNoCleaner() ? PlatformDependent.allocateDirectNoCleaner(capacity) : ByteBuffer.allocateDirect(capacity);
  }
  
  protected void destroyChunk(PoolChunk<ByteBuffer> chunk)
  {
    if (PlatformDependent.useDirectBufferNoCleaner()) {
      PlatformDependent.freeDirectNoCleaner((ByteBuffer)memory);
    } else {
      PlatformDependent.freeDirectBuffer((ByteBuffer)memory);
    }
  }
  
  protected PooledByteBuf<ByteBuffer> newByteBuf(int maxCapacity)
  {
    if (HAS_UNSAFE) {
      return PooledUnsafeDirectByteBuf.newInstance(maxCapacity);
    }
    return PooledDirectByteBuf.newInstance(maxCapacity);
  }
  
  protected void memoryCopy(ByteBuffer src, int srcOffset, ByteBuffer dst, int dstOffset, int length)
  {
    if (length == 0) {
      return;
    }
    if (HAS_UNSAFE)
    {
      PlatformDependent.copyMemory(PlatformDependent.directBufferAddress(src) + srcOffset, PlatformDependent.directBufferAddress(dst) + dstOffset, length);
    }
    else
    {
      src = src.duplicate();
      dst = dst.duplicate();
      src.position(srcOffset).limit(srcOffset + length);
      dst.position(dstOffset);
      dst.put(src);
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.buffer.PoolArena.DirectArena
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */