package io.netty.buffer;

import io.netty.util.Recycler;
import io.netty.util.Recycler.Handle;
import io.netty.util.internal.MathUtil;
import io.netty.util.internal.PlatformDependent;
import java.util.Queue;

abstract class PoolThreadCache$MemoryRegionCache<T>
{
  private final int size;
  private final Queue<Entry<T>> queue;
  private final PoolArena.SizeClass sizeClass;
  private int allocations;
  
  PoolThreadCache$MemoryRegionCache(int size, PoolArena.SizeClass sizeClass)
  {
    this.size = MathUtil.safeFindNextPositivePowerOfTwo(size);
    queue = PlatformDependent.newFixedMpscQueue(this.size);
    this.sizeClass = sizeClass;
  }
  
  protected abstract void initBuf(PoolChunk<T> paramPoolChunk, long paramLong, PooledByteBuf<T> paramPooledByteBuf, int paramInt);
  
  public final boolean add(PoolChunk<T> chunk, long handle)
  {
    Entry<T> entry = newEntry(chunk, handle);
    boolean queued = queue.offer(entry);
    if (!queued) {
      entry.recycle();
    }
    return queued;
  }
  
  public final boolean allocate(PooledByteBuf<T> buf, int reqCapacity)
  {
    Entry<T> entry = (Entry)queue.poll();
    if (entry == null) {
      return false;
    }
    initBuf(chunk, handle, buf, reqCapacity);
    entry.recycle();
    
    allocations += 1;
    return true;
  }
  
  public final int free()
  {
    return free(Integer.MAX_VALUE);
  }
  
  private int free(int max)
  {
    for (int numFreed = 0; numFreed < max; numFreed++)
    {
      Entry<T> entry = (Entry)queue.poll();
      if (entry != null) {
        freeEntry(entry);
      } else {
        return numFreed;
      }
    }
    return numFreed;
  }
  
  public final void trim()
  {
    int free = size - allocations;
    allocations = 0;
    if (free > 0) {
      free(free);
    }
  }
  
  private void freeEntry(Entry entry)
  {
    PoolChunk chunk = chunk;
    long handle = handle;
    
    entry.recycle();
    
    arena.freeChunk(chunk, handle, sizeClass);
  }
  
  static final class Entry<T>
  {
    final Recycler.Handle<Entry<?>> recyclerHandle;
    PoolChunk<T> chunk;
    long handle = -1L;
    
    Entry(Recycler.Handle<Entry<?>> recyclerHandle)
    {
      this.recyclerHandle = recyclerHandle;
    }
    
    void recycle()
    {
      chunk = null;
      handle = -1L;
      recyclerHandle.recycle(this);
    }
  }
  
  private static Entry newEntry(PoolChunk<?> chunk, long handle)
  {
    Entry entry = (Entry)RECYCLER.get();
    chunk = chunk;
    handle = handle;
    return entry;
  }
  
  private static final Recycler<Entry> RECYCLER = new Recycler()
  {
    protected PoolThreadCache.MemoryRegionCache.Entry newObject(Recycler.Handle<PoolThreadCache.MemoryRegionCache.Entry> handle)
    {
      return new PoolThreadCache.MemoryRegionCache.Entry(handle);
    }
  };
}

/* Location:
 * Qualified Name:     io.netty.buffer.PoolThreadCache.MemoryRegionCache
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */