package io.netty.buffer;

import io.netty.util.internal.PlatformDependent;

public final class UnpooledByteBufAllocator
  extends AbstractByteBufAllocator
{
  private final boolean disableLeakDetector;
  public static final UnpooledByteBufAllocator DEFAULT = new UnpooledByteBufAllocator(PlatformDependent.directBufferPreferred());
  
  public UnpooledByteBufAllocator(boolean preferDirect)
  {
    this(preferDirect, false);
  }
  
  public UnpooledByteBufAllocator(boolean preferDirect, boolean disableLeakDetector)
  {
    super(preferDirect);
    this.disableLeakDetector = disableLeakDetector;
  }
  
  protected ByteBuf newHeapBuffer(int initialCapacity, int maxCapacity)
  {
    return PlatformDependent.hasUnsafe() ? new UnpooledUnsafeHeapByteBuf(this, initialCapacity, maxCapacity) : new UnpooledHeapByteBuf(this, initialCapacity, maxCapacity);
  }
  
  protected ByteBuf newDirectBuffer(int initialCapacity, int maxCapacity)
  {
    ByteBuf buf = PlatformDependent.hasUnsafe() ? UnsafeByteBufUtil.newUnsafeDirectByteBuf(this, initialCapacity, maxCapacity) : new UnpooledDirectByteBuf(this, initialCapacity, maxCapacity);
    
    return disableLeakDetector ? buf : toLeakAwareBuffer(buf);
  }
  
  public CompositeByteBuf compositeHeapBuffer(int maxNumComponents)
  {
    CompositeByteBuf buf = new CompositeByteBuf(this, false, maxNumComponents);
    return disableLeakDetector ? buf : toLeakAwareBuffer(buf);
  }
  
  public CompositeByteBuf compositeDirectBuffer(int maxNumComponents)
  {
    CompositeByteBuf buf = new CompositeByteBuf(this, true, maxNumComponents);
    return disableLeakDetector ? buf : toLeakAwareBuffer(buf);
  }
  
  public boolean isDirectBufferPooled()
  {
    return false;
  }
}

/* Location:
 * Qualified Name:     io.netty.buffer.UnpooledByteBufAllocator
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */