package io.netty.buffer;

import java.nio.ByteOrder;

final class UnreleasableByteBuf
  extends WrappedByteBuf
{
  private SwappedByteBuf swappedBuf;
  
  UnreleasableByteBuf(ByteBuf buf)
  {
    super(buf);
  }
  
  public ByteBuf order(ByteOrder endianness)
  {
    if (endianness == null) {
      throw new NullPointerException("endianness");
    }
    if (endianness == order()) {
      return this;
    }
    SwappedByteBuf swappedBuf = this.swappedBuf;
    if (swappedBuf == null) {
      this.swappedBuf = (swappedBuf = new SwappedByteBuf(this));
    }
    return swappedBuf;
  }
  
  public ByteBuf asReadOnly()
  {
    return new UnreleasableByteBuf(buf.asReadOnly());
  }
  
  public ByteBuf readSlice(int length)
  {
    return new UnreleasableByteBuf(buf.readSlice(length));
  }
  
  public ByteBuf readRetainedSlice(int length)
  {
    return new UnreleasableByteBuf(buf.readRetainedSlice(length));
  }
  
  public ByteBuf slice()
  {
    return new UnreleasableByteBuf(buf.slice());
  }
  
  public ByteBuf retainedSlice()
  {
    return new UnreleasableByteBuf(buf.retainedSlice());
  }
  
  public ByteBuf slice(int index, int length)
  {
    return new UnreleasableByteBuf(buf.slice(index, length));
  }
  
  public ByteBuf retainedSlice(int index, int length)
  {
    return new UnreleasableByteBuf(buf.retainedSlice(index, length));
  }
  
  public ByteBuf duplicate()
  {
    return new UnreleasableByteBuf(buf.duplicate());
  }
  
  public ByteBuf retainedDuplicate()
  {
    return new UnreleasableByteBuf(buf.retainedDuplicate());
  }
  
  public ByteBuf retain(int increment)
  {
    return this;
  }
  
  public ByteBuf retain()
  {
    return this;
  }
  
  public ByteBuf touch()
  {
    return this;
  }
  
  public ByteBuf touch(Object hint)
  {
    return this;
  }
  
  public boolean release()
  {
    return false;
  }
  
  public boolean release(int decrement)
  {
    return false;
  }
}

/* Location:
 * Qualified Name:     io.netty.buffer.UnreleasableByteBuf
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */