package io.netty.channel;

import io.netty.buffer.Unpooled;
import io.netty.util.Recycler;
import io.netty.util.Recycler.Handle;
import io.netty.util.ReferenceCountUtil;
import java.nio.ByteBuffer;

final class ChannelOutboundBuffer$Entry
{
  private static final Recycler<Entry> RECYCLER = new Recycler()
  {
    protected ChannelOutboundBuffer.Entry newObject(Recycler.Handle handle)
    {
      return new ChannelOutboundBuffer.Entry(handle, null);
    }
  };
  private final Recycler.Handle<Entry> handle;
  Entry next;
  Object msg;
  ByteBuffer[] bufs;
  ByteBuffer buf;
  ChannelPromise promise;
  long progress;
  long total;
  int pendingSize;
  int count = -1;
  boolean cancelled;
  
  private ChannelOutboundBuffer$Entry(Recycler.Handle<Entry> handle)
  {
    this.handle = handle;
  }
  
  static Entry newInstance(Object msg, int size, long total, ChannelPromise promise)
  {
    Entry entry = (Entry)RECYCLER.get();
    msg = msg;
    pendingSize = size;
    total = total;
    promise = promise;
    return entry;
  }
  
  int cancel()
  {
    if (!cancelled)
    {
      cancelled = true;
      int pSize = pendingSize;
      
      ReferenceCountUtil.safeRelease(msg);
      msg = Unpooled.EMPTY_BUFFER;
      
      pendingSize = 0;
      total = 0L;
      progress = 0L;
      bufs = null;
      buf = null;
      return pSize;
    }
    return 0;
  }
  
  void recycle()
  {
    next = null;
    bufs = null;
    buf = null;
    msg = null;
    promise = null;
    progress = 0L;
    total = 0L;
    pendingSize = 0;
    count = -1;
    cancelled = false;
    handle.recycle(this);
  }
  
  Entry recycleAndGetNext()
  {
    Entry next = this.next;
    recycle();
    return next;
  }
}

/* Location:
 * Qualified Name:     io.netty.channel.ChannelOutboundBuffer.Entry
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */