package io.netty.handler.codec;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelOutboundHandlerAdapter;
import io.netty.channel.ChannelPromise;
import io.netty.util.ReferenceCountUtil;
import io.netty.util.internal.StringUtil;
import io.netty.util.internal.TypeParameterMatcher;
import java.util.List;

public abstract class MessageToMessageEncoder<I>
  extends ChannelOutboundHandlerAdapter
{
  private final TypeParameterMatcher matcher;
  
  protected MessageToMessageEncoder()
  {
    matcher = TypeParameterMatcher.find(this, MessageToMessageEncoder.class, "I");
  }
  
  protected MessageToMessageEncoder(Class<? extends I> outboundMessageType)
  {
    matcher = TypeParameterMatcher.get(outboundMessageType);
  }
  
  public boolean acceptOutboundMessage(Object msg)
    throws Exception
  {
    return matcher.match(msg);
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise)
    throws Exception
  {
    CodecOutputList out = null;
    try
    {
      if (acceptOutboundMessage(msg))
      {
        out = CodecOutputList.newInstance();
        
        I cast = (I)msg;
        try
        {
          encode(ctx, cast, out);
        }
        finally
        {
          ReferenceCountUtil.release(cast);
        }
        if (out.isEmpty())
        {
          out.recycle();
          out = null;
          
          throw new EncoderException(StringUtil.simpleClassName(this) + " must produce at least one message.");
        }
      }
      else
      {
        ctx.write(msg, promise);
      }
    }
    catch (EncoderException e)
    {
      int sizeMinusOne;
      ChannelPromise voidPromise;
      boolean isVoidPromise;
      int i;
      ChannelPromise p;
      ChannelPromise p;
      throw e;
    }
    catch (Throwable t)
    {
      throw new EncoderException(t);
    }
    finally
    {
      if (out != null)
      {
        int sizeMinusOne = out.size() - 1;
        if (sizeMinusOne == 0)
        {
          ctx.write(out.get(0), promise);
        }
        else if (sizeMinusOne > 0)
        {
          ChannelPromise voidPromise = ctx.voidPromise();
          boolean isVoidPromise = promise == voidPromise;
          for (int i = 0; i < sizeMinusOne; i++)
          {
            ChannelPromise p;
            ChannelPromise p;
            if (isVoidPromise) {
              p = voidPromise;
            } else {
              p = ctx.newPromise();
            }
            ctx.write(out.getUnsafe(i), p);
          }
          ctx.write(out.getUnsafe(sizeMinusOne), promise);
        }
        out.recycle();
      }
    }
  }
  
  protected abstract void encode(ChannelHandlerContext paramChannelHandlerContext, I paramI, List<Object> paramList)
    throws Exception;
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.MessageToMessageEncoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */