package io.netty.handler.codec.dns;

import io.netty.buffer.ByteBuf;
import io.netty.util.internal.ObjectUtil;
import io.netty.util.internal.StringUtil;

public class DefaultDnsRawRecord
  extends AbstractDnsRecord
  implements DnsRawRecord
{
  private final ByteBuf content;
  
  public DefaultDnsRawRecord(String name, DnsRecordType type, long timeToLive, ByteBuf content)
  {
    this(name, type, 1, timeToLive, content);
  }
  
  public DefaultDnsRawRecord(String name, DnsRecordType type, int dnsClass, long timeToLive, ByteBuf content)
  {
    super(name, type, dnsClass, timeToLive);
    this.content = ((ByteBuf)ObjectUtil.checkNotNull(content, "content"));
  }
  
  public ByteBuf content()
  {
    return content;
  }
  
  public DnsRawRecord copy()
  {
    return replace(content().copy());
  }
  
  public DnsRawRecord duplicate()
  {
    return replace(content().duplicate());
  }
  
  public DnsRawRecord retainedDuplicate()
  {
    return replace(content().retainedDuplicate());
  }
  
  public DnsRawRecord replace(ByteBuf content)
  {
    return new DefaultDnsRawRecord(name(), type(), dnsClass(), timeToLive(), content);
  }
  
  public int refCnt()
  {
    return content().refCnt();
  }
  
  public DnsRawRecord retain()
  {
    content().retain();
    return this;
  }
  
  public DnsRawRecord retain(int increment)
  {
    content().retain(increment);
    return this;
  }
  
  public boolean release()
  {
    return content().release();
  }
  
  public boolean release(int decrement)
  {
    return content().release(decrement);
  }
  
  public DnsRawRecord touch()
  {
    content().touch();
    return this;
  }
  
  public DnsRawRecord touch(Object hint)
  {
    content().touch(hint);
    return this;
  }
  
  public String toString()
  {
    StringBuilder buf = new StringBuilder(64).append(StringUtil.simpleClassName(this)).append('(');
    DnsRecordType type = type();
    if (type != DnsRecordType.OPT)
    {
      buf.append(name().isEmpty() ? "<root>" : name()).append(' ').append(timeToLive()).append(' ');
      
      DnsMessageUtil.appendRecordClass(buf, dnsClass()).append(' ').append(type.name());
    }
    else
    {
      buf.append("OPT flags:").append(timeToLive()).append(" udp:").append(dnsClass());
    }
    buf.append(' ').append(content().readableBytes()).append("B)");
    
    return buf.toString();
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.dns.DefaultDnsRawRecord
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */