package io.netty.handler.codec.http;

import io.netty.buffer.ByteBuf;
import io.netty.handler.codec.TooLongFrameException;
import io.netty.util.ByteProcessor;
import io.netty.util.internal.AppendableCharSequence;

class HttpObjectDecoder$HeaderParser
  implements ByteProcessor
{
  private final AppendableCharSequence seq;
  private final int maxLength;
  private int size;
  
  HttpObjectDecoder$HeaderParser(AppendableCharSequence seq, int maxLength)
  {
    this.seq = seq;
    this.maxLength = maxLength;
  }
  
  public AppendableCharSequence parse(ByteBuf buffer)
  {
    int oldSize = size;
    seq.reset();
    int i = buffer.forEachByte(this);
    if (i == -1)
    {
      size = oldSize;
      return null;
    }
    buffer.readerIndex(i + 1);
    return seq;
  }
  
  public void reset()
  {
    size = 0;
  }
  
  public boolean process(byte value)
    throws Exception
  {
    char nextByte = (char)value;
    if (nextByte == '\r') {
      return true;
    }
    if (nextByte == '\n') {
      return false;
    }
    if (++size > maxLength) {
      throw newException(maxLength);
    }
    seq.append(nextByte);
    return true;
  }
  
  protected TooLongFrameException newException(int maxLength)
  {
    return new TooLongFrameException("HTTP header is larger than " + maxLength + " bytes.");
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http.HttpObjectDecoder.HeaderParser
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */