package io.netty.handler.codec.http.cookie;

import io.netty.handler.codec.http.HttpHeaderDateFormat;
import java.text.ParsePosition;
import java.util.Date;

class ClientCookieDecoder$CookieBuilder
{
  private final String header;
  private final DefaultCookie cookie;
  private String domain;
  private String path;
  private long maxAge = Long.MIN_VALUE;
  private int expiresStart;
  private int expiresEnd;
  private boolean secure;
  private boolean httpOnly;
  
  public ClientCookieDecoder$CookieBuilder(DefaultCookie cookie, String header)
  {
    this.cookie = cookie;
    this.header = header;
  }
  
  private long mergeMaxAgeAndExpires()
  {
    if (maxAge != Long.MIN_VALUE) {
      return maxAge;
    }
    String expires = computeValue(expiresStart, expiresEnd);
    if (expires != null)
    {
      Date expiresDate = HttpHeaderDateFormat.get().parse(expires, new ParsePosition(0));
      if (expiresDate != null)
      {
        long maxAgeMillis = expiresDate.getTime() - System.currentTimeMillis();
        return maxAgeMillis / 1000L + (maxAgeMillis % 1000L != 0L ? 1 : 0);
      }
    }
    return Long.MIN_VALUE;
  }
  
  public Cookie cookie()
  {
    cookie.setDomain(domain);
    cookie.setPath(path);
    cookie.setMaxAge(mergeMaxAgeAndExpires());
    cookie.setSecure(secure);
    cookie.setHttpOnly(httpOnly);
    return cookie;
  }
  
  public void appendAttribute(int keyStart, int keyEnd, int valueStart, int valueEnd)
  {
    int length = keyEnd - keyStart;
    if (length == 4) {
      parse4(keyStart, valueStart, valueEnd);
    } else if (length == 6) {
      parse6(keyStart, valueStart, valueEnd);
    } else if (length == 7) {
      parse7(keyStart, valueStart, valueEnd);
    } else if (length == 8) {
      parse8(keyStart, valueStart, valueEnd);
    }
  }
  
  private void parse4(int nameStart, int valueStart, int valueEnd)
  {
    if (header.regionMatches(true, nameStart, "Path", 0, 4)) {
      path = computeValue(valueStart, valueEnd);
    }
  }
  
  private void parse6(int nameStart, int valueStart, int valueEnd)
  {
    if (header.regionMatches(true, nameStart, "Domain", 0, 5)) {
      domain = computeValue(valueStart, valueEnd);
    } else if (header.regionMatches(true, nameStart, "Secure", 0, 5)) {
      secure = true;
    }
  }
  
  private void setMaxAge(String value)
  {
    try
    {
      maxAge = Math.max(Long.parseLong(value), 0L);
    }
    catch (NumberFormatException localNumberFormatException) {}
  }
  
  private void parse7(int nameStart, int valueStart, int valueEnd)
  {
    if (header.regionMatches(true, nameStart, "Expires", 0, 7))
    {
      expiresStart = valueStart;
      expiresEnd = valueEnd;
    }
    else if (header.regionMatches(true, nameStart, "Max-Age", 0, 7))
    {
      setMaxAge(computeValue(valueStart, valueEnd));
    }
  }
  
  private void parse8(int nameStart, int valueStart, int valueEnd)
  {
    if (header.regionMatches(true, nameStart, "HTTPOnly", 0, 8)) {
      httpOnly = true;
    }
  }
  
  private String computeValue(int valueStart, int valueEnd)
  {
    return (valueStart == -1) || (valueStart == valueEnd) ? null : header.substring(valueStart, valueEnd);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http.cookie.ClientCookieDecoder.CookieBuilder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */