package io.netty.handler.codec.http2;

import io.netty.util.internal.logging.InternalLogger;
import java.util.ArrayDeque;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Queue;
import java.util.Set;

final class DefaultHttp2Connection$ActiveStreams
{
  private final List<Http2Connection.Listener> listeners;
  private final Queue<DefaultHttp2Connection.Event> pendingEvents = new ArrayDeque(4);
  private final Set<Http2Stream> streams = new LinkedHashSet();
  private int pendingIterations;
  
  public DefaultHttp2Connection$ActiveStreams(List<Http2Connection.Listener> arg1)
  {
    this.listeners = listeners;
  }
  
  public int size()
  {
    return streams.size();
  }
  
  public void activate(final DefaultHttp2Connection.DefaultStream stream)
  {
    if (allowModifications()) {
      addToActiveStreams(stream);
    } else {
      pendingEvents.add(new DefaultHttp2Connection.Event()
      {
        public void process()
        {
          addToActiveStreams(stream);
        }
      });
    }
  }
  
  public void deactivate(final DefaultHttp2Connection.DefaultStream stream, final Iterator<?> itr)
  {
    if ((allowModifications()) || (itr != null)) {
      removeFromActiveStreams(stream, itr);
    } else {
      pendingEvents.add(new DefaultHttp2Connection.Event()
      {
        public void process()
        {
          if (stream.parent() == null) {
            return;
          }
          removeFromActiveStreams(stream, itr);
        }
      });
    }
  }
  
  public Http2Stream forEachActiveStream(Http2StreamVisitor visitor)
    throws Http2Exception
  {
    incrementPendingIterations();
    try
    {
      for (Http2Stream stream : streams) {
        if (!visitor.visit(stream)) {
          return stream;
        }
      }
      return null;
    }
    finally
    {
      decrementPendingIterations();
    }
  }
  
  void addToActiveStreams(DefaultHttp2Connection.DefaultStream stream)
  {
    if (streams.add(stream))
    {
      createdBynumActiveStreams += 1;
      for (int i = 0; i < listeners.size(); i++) {
        try
        {
          ((Http2Connection.Listener)listeners.get(i)).onStreamActive(stream);
        }
        catch (Throwable cause)
        {
          DefaultHttp2Connection.access$600().error("Caught Throwable from listener onStreamActive.", cause);
        }
      }
    }
  }
  
  void removeFromActiveStreams(DefaultHttp2Connection.DefaultStream stream, Iterator<?> itr)
  {
    if (streams.remove(stream))
    {
      createdBynumActiveStreams -= 1;
      this$0.notifyClosed(stream);
    }
    this$0.removeStream(stream, itr);
  }
  
  boolean allowModifications()
  {
    return pendingIterations == 0;
  }
  
  void incrementPendingIterations()
  {
    pendingIterations += 1;
  }
  
  void decrementPendingIterations()
  {
    pendingIterations -= 1;
    if (allowModifications()) {
      for (;;)
      {
        DefaultHttp2Connection.Event event = (DefaultHttp2Connection.Event)pendingEvents.poll();
        if (event == null) {
          break;
        }
        try
        {
          event.process();
        }
        catch (Throwable cause)
        {
          DefaultHttp2Connection.access$600().error("Caught Throwable while processing pending ActiveStreams$Event.", cause);
        }
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2Connection.ActiveStreams
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */