package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPromise;
import io.netty.channel.CoalescingBufferQueue;

final class DefaultHttp2ConnectionEncoder$FlowControlledData
  extends DefaultHttp2ConnectionEncoder.FlowControlledBase
{
  private final CoalescingBufferQueue queue;
  private int dataSize;
  
  DefaultHttp2ConnectionEncoder$FlowControlledData(DefaultHttp2ConnectionEncoder paramDefaultHttp2ConnectionEncoder, Http2Stream stream, ByteBuf buf, int padding, boolean endOfStream, ChannelPromise promise)
  {
    super(paramDefaultHttp2ConnectionEncoder, stream, padding, endOfStream, promise);
    queue = new CoalescingBufferQueue(promise.channel());
    queue.add(buf, promise);
    dataSize = queue.readableBytes();
  }
  
  public int size()
  {
    return dataSize + padding;
  }
  
  public void error(ChannelHandlerContext ctx, Throwable cause)
  {
    queue.releaseAndFailAll(cause);
    
    DefaultHttp2ConnectionEncoder.access$000(this$0).onError(ctx, cause);
  }
  
  public void write(ChannelHandlerContext ctx, int allowedBytes)
  {
    int queuedData = queue.readableBytes();
    if (!endOfStream)
    {
      if (queuedData == 0)
      {
        ChannelPromise writePromise = ctx.newPromise().addListener(this);
        queue.remove(0, writePromise).release();
        ctx.write(Unpooled.EMPTY_BUFFER, writePromise);
        return;
      }
      if (allowedBytes == 0) {
        return;
      }
    }
    int writeableData = Math.min(queuedData, allowedBytes);
    ChannelPromise writePromise = ctx.newPromise().addListener(this);
    ByteBuf toWrite = queue.remove(writeableData, writePromise);
    dataSize = queue.readableBytes();
    
    int writeablePadding = Math.min(allowedBytes - writeableData, padding);
    padding -= writeablePadding;
    
    this$0.frameWriter().writeData(ctx, stream.id(), toWrite, writeablePadding, (endOfStream) && (size() == 0), writePromise);
  }
  
  public boolean merge(ChannelHandlerContext ctx, Http2RemoteFlowController.FlowControlled next)
  {
    FlowControlledData nextData;
    if ((FlowControlledData.class != next.getClass()) || (Integer.MAX_VALUE - (nextData = (FlowControlledData)next).size() < size())) {
      return false;
    }
    FlowControlledData nextData;
    queue.copyTo(queue);
    dataSize = queue.readableBytes();
    
    padding = Math.max(padding, padding);
    endOfStream = endOfStream;
    return true;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2ConnectionEncoder.FlowControlledData
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */