package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.util.IllegalReferenceCountException;
import io.netty.util.internal.ObjectUtil;

public final class DefaultHttp2DataFrame
  extends AbstractHttp2StreamFrame
  implements Http2DataFrame
{
  private final ByteBuf content;
  private final boolean endStream;
  private final int padding;
  
  public DefaultHttp2DataFrame(ByteBuf content)
  {
    this(content, false);
  }
  
  public DefaultHttp2DataFrame(boolean endStream)
  {
    this(Unpooled.EMPTY_BUFFER, endStream);
  }
  
  public DefaultHttp2DataFrame(ByteBuf content, boolean endStream)
  {
    this(content, endStream, 0);
  }
  
  public DefaultHttp2DataFrame(ByteBuf content, boolean endStream, int padding)
  {
    this.content = ((ByteBuf)ObjectUtil.checkNotNull(content, "content"));
    this.endStream = endStream;
    Http2CodecUtil.verifyPadding(padding);
    this.padding = padding;
  }
  
  public DefaultHttp2DataFrame setStreamId(int streamId)
  {
    super.setStreamId(streamId);
    return this;
  }
  
  public String name()
  {
    return "DATA";
  }
  
  public boolean isEndStream()
  {
    return endStream;
  }
  
  public int padding()
  {
    return padding;
  }
  
  public ByteBuf content()
  {
    if (content.refCnt() <= 0) {
      throw new IllegalReferenceCountException(content.refCnt());
    }
    return content;
  }
  
  public DefaultHttp2DataFrame copy()
  {
    return replace(content().copy());
  }
  
  public DefaultHttp2DataFrame duplicate()
  {
    return replace(content().duplicate());
  }
  
  public DefaultHttp2DataFrame retainedDuplicate()
  {
    return replace(content().retainedDuplicate());
  }
  
  public DefaultHttp2DataFrame replace(ByteBuf content)
  {
    return new DefaultHttp2DataFrame(content, endStream, padding);
  }
  
  public int refCnt()
  {
    return content.refCnt();
  }
  
  public boolean release()
  {
    return content.release();
  }
  
  public boolean release(int decrement)
  {
    return content.release(decrement);
  }
  
  public DefaultHttp2DataFrame retain()
  {
    content.retain();
    return this;
  }
  
  public DefaultHttp2DataFrame retain(int increment)
  {
    content.retain(increment);
    return this;
  }
  
  public String toString()
  {
    return "DefaultHttp2DataFrame(streamId=" + streamId() + ", content=" + content + ", endStream=" + endStream + ", padding=" + padding + ")";
  }
  
  public DefaultHttp2DataFrame touch()
  {
    content.touch();
    return this;
  }
  
  public DefaultHttp2DataFrame touch(Object hint)
  {
    content.touch(hint);
    return this;
  }
  
  public boolean equals(Object o)
  {
    if (!(o instanceof DefaultHttp2DataFrame)) {
      return false;
    }
    DefaultHttp2DataFrame other = (DefaultHttp2DataFrame)o;
    return (super.equals(other)) && (content.equals(other.content())) && (endStream == endStream) && (padding == padding);
  }
  
  public int hashCode()
  {
    int hash = super.hashCode();
    hash = hash * 31 + content.hashCode();
    hash = hash * 31 + (endStream ? 0 : 1);
    hash = hash * 31 + padding;
    return hash;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2DataFrame
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */