package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.handler.codec.http2.internal.hpack.Decoder;

public class DefaultHttp2HeadersDecoder
  implements Http2HeadersDecoder, Http2HeadersDecoder.Configuration
{
  private static final float HEADERS_COUNT_WEIGHT_NEW = 0.2F;
  private static final float HEADERS_COUNT_WEIGHT_HISTORICAL = 0.8F;
  private final int maxHeaderSize;
  private final Decoder decoder;
  private final Http2HeaderTable headerTable;
  private final boolean validateHeaders;
  private float headerArraySizeAccumulator = 8.0F;
  
  public DefaultHttp2HeadersDecoder()
  {
    this(true);
  }
  
  public DefaultHttp2HeadersDecoder(boolean validateHeaders)
  {
    this(8192, 4096, validateHeaders, 32);
  }
  
  public DefaultHttp2HeadersDecoder(int maxHeaderSize, int maxHeaderTableSize, boolean validateHeaders, int initialHuffmanDecodeCapacity)
  {
    if (maxHeaderSize <= 0) {
      throw new IllegalArgumentException("maxHeaderSize must be positive: " + maxHeaderSize);
    }
    decoder = new Decoder(maxHeaderSize, maxHeaderTableSize, initialHuffmanDecodeCapacity);
    headerTable = new Http2HeaderTableDecoder(null);
    this.maxHeaderSize = maxHeaderSize;
    this.validateHeaders = validateHeaders;
  }
  
  public Http2HeaderTable headerTable()
  {
    return headerTable;
  }
  
  public int maxHeaderSize()
  {
    return maxHeaderSize;
  }
  
  public Http2HeadersDecoder.Configuration configuration()
  {
    return this;
  }
  
  public Http2Headers decodeHeaders(ByteBuf headerBlock)
    throws Http2Exception
  {
    try
    {
      Http2Headers headers = new DefaultHttp2Headers(validateHeaders, (int)headerArraySizeAccumulator);
      decoder.decode(headerBlock, headers);
      headerArraySizeAccumulator = (0.2F * headers.size() + 0.8F * headerArraySizeAccumulator);
      
      return headers;
    }
    catch (Http2Exception e)
    {
      throw e;
    }
    catch (Throwable e)
    {
      throw Http2Exception.connectionError(Http2Error.COMPRESSION_ERROR, e, e.getMessage(), new Object[0]);
    }
  }
  
  private final class Http2HeaderTableDecoder
    extends DefaultHttp2HeaderTableListSize
    implements Http2HeaderTable
  {
    private Http2HeaderTableDecoder() {}
    
    public void maxHeaderTableSize(int max)
      throws Http2Exception
    {
      if (max < 0) {
        throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, "Header Table Size must be non-negative but was %d", new Object[] { Integer.valueOf(max) });
      }
      try
      {
        decoder.setMaxHeaderTableSize(max);
      }
      catch (Throwable t)
      {
        throw Http2Exception.connectionError(Http2Error.PROTOCOL_ERROR, t.getMessage(), new Object[] { t });
      }
    }
    
    public int maxHeaderTableSize()
    {
      return decoder.getMaxHeaderTableSize();
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2HeadersDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */