package io.netty.handler.codec.http2;

import io.netty.util.internal.ObjectUtil;

public final class DefaultHttp2HeadersFrame
  extends AbstractHttp2StreamFrame
  implements Http2HeadersFrame
{
  private final Http2Headers headers;
  private final boolean endStream;
  private final int padding;
  
  public DefaultHttp2HeadersFrame(Http2Headers headers)
  {
    this(headers, false);
  }
  
  public DefaultHttp2HeadersFrame(Http2Headers headers, boolean endStream)
  {
    this(headers, endStream, 0);
  }
  
  public DefaultHttp2HeadersFrame(Http2Headers headers, boolean endStream, int padding)
  {
    this.headers = ((Http2Headers)ObjectUtil.checkNotNull(headers, "headers"));
    this.endStream = endStream;
    Http2CodecUtil.verifyPadding(padding);
    this.padding = padding;
  }
  
  public DefaultHttp2HeadersFrame setStreamId(int streamId)
  {
    super.setStreamId(streamId);
    return this;
  }
  
  public String name()
  {
    return "HEADERS";
  }
  
  public Http2Headers headers()
  {
    return headers;
  }
  
  public boolean isEndStream()
  {
    return endStream;
  }
  
  public int padding()
  {
    return padding;
  }
  
  public String toString()
  {
    return "DefaultHttp2HeadersFrame(streamId=" + streamId() + ", headers=" + headers + ", endStream=" + endStream + ", padding=" + padding + ")";
  }
  
  public boolean equals(Object o)
  {
    if (!(o instanceof DefaultHttp2HeadersFrame)) {
      return false;
    }
    DefaultHttp2HeadersFrame other = (DefaultHttp2HeadersFrame)o;
    return (super.equals(other)) && (headers.equals(headers)) && (endStream == endStream) && (padding == padding);
  }
  
  public int hashCode()
  {
    int hash = super.hashCode();
    hash = hash * 31 + headers.hashCode();
    hash = hash * 31 + (endStream ? 0 : 1);
    hash = hash * 31 + padding;
    return hash;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DefaultHttp2HeadersFrame
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */