package io.netty.handler.codec.http2;

import io.netty.channel.embedded.EmbeddedChannel;

final class DelegatingDecompressorFrameListener$Http2Decompressor
{
  private final EmbeddedChannel decompressor;
  private int compressed;
  private int decompressed;
  
  DelegatingDecompressorFrameListener$Http2Decompressor(EmbeddedChannel decompressor)
  {
    this.decompressor = decompressor;
  }
  
  EmbeddedChannel decompressor()
  {
    return decompressor;
  }
  
  void incrementCompressedBytes(int delta)
  {
    assert (delta >= 0);
    compressed += delta;
  }
  
  void incrementDecompressedBytes(int delta)
  {
    assert (delta >= 0);
    decompressed += delta;
  }
  
  int consumeBytes(int streamId, int decompressedBytes)
    throws Http2Exception
  {
    if (decompressedBytes < 0) {
      throw new IllegalArgumentException("decompressedBytes must not be negative: " + decompressedBytes);
    }
    if (decompressed - decompressedBytes < 0) {
      throw Http2Exception.streamError(streamId, Http2Error.INTERNAL_ERROR, "Attempting to return too many bytes for stream %d. decompressed: %d decompressedBytes: %d", new Object[] { Integer.valueOf(streamId), Integer.valueOf(decompressed), Integer.valueOf(decompressedBytes) });
    }
    double consumedRatio = decompressedBytes / decompressed;
    int consumedCompressed = Math.min(compressed, (int)Math.ceil(compressed * consumedRatio));
    if (compressed - consumedCompressed < 0) {
      throw Http2Exception.streamError(streamId, Http2Error.INTERNAL_ERROR, "overflow when converting decompressed bytes to compressed bytes for stream %d.decompressedBytes: %d decompressed: %d compressed: %d consumedCompressed: %d", new Object[] { Integer.valueOf(streamId), Integer.valueOf(decompressedBytes), Integer.valueOf(decompressed), Integer.valueOf(compressed), Integer.valueOf(consumedCompressed) });
    }
    decompressed -= decompressedBytes;
    compressed -= consumedCompressed;
    
    return consumedCompressed;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.DelegatingDecompressorFrameListener.Http2Decompressor
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */