package io.netty.handler.codec.http2;

import io.netty.channel.ChannelDuplexHandler;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;

public final class Http2Codec
  extends ChannelDuplexHandler
{
  private final Http2FrameCodec frameCodec;
  private final Http2MultiplexCodec multiplexCodec;
  
  public Http2Codec(boolean server, ChannelHandler streamHandler)
  {
    this(server, streamHandler, null);
  }
  
  public Http2Codec(boolean server, ChannelHandler streamHandler, EventLoopGroup streamGroup)
  {
    this(server, streamHandler, streamGroup, new DefaultHttp2FrameWriter());
  }
  
  Http2Codec(boolean server, ChannelHandler streamHandler, EventLoopGroup streamGroup, Http2FrameWriter frameWriter)
  {
    frameCodec = new Http2FrameCodec(server, frameWriter);
    multiplexCodec = new Http2MultiplexCodec(server, streamGroup, streamHandler);
  }
  
  Http2FrameCodec frameCodec()
  {
    return frameCodec;
  }
  
  public void handlerAdded(ChannelHandlerContext ctx)
    throws Exception
  {
    ctx.pipeline().addBefore(ctx.executor(), ctx.name(), null, frameCodec);
    ctx.pipeline().addBefore(ctx.executor(), ctx.name(), null, multiplexCodec);
    
    ctx.pipeline().remove(this);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.Http2Codec
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */