package io.netty.handler.codec.http2;

import io.netty.channel.Channel;
import io.netty.channel.ChannelPromise;
import io.netty.channel.DefaultChannelPromise;
import io.netty.util.concurrent.EventExecutor;

final class Http2CodecUtil$SimpleChannelPromiseAggregator
  extends DefaultChannelPromise
{
  private final ChannelPromise promise;
  private int expectedCount;
  private int doneCount;
  private Throwable lastFailure;
  private boolean doneAllocating;
  
  Http2CodecUtil$SimpleChannelPromiseAggregator(ChannelPromise promise, Channel c, EventExecutor e)
  {
    super(c, e);
    assert ((promise != null) && (!promise.isDone()));
    this.promise = promise;
  }
  
  public ChannelPromise newPromise()
  {
    assert (!doneAllocating) : "Done allocating. No more promises can be allocated.";
    expectedCount += 1;
    return this;
  }
  
  public ChannelPromise doneAllocatingPromises()
  {
    if (!doneAllocating)
    {
      doneAllocating = true;
      if ((doneCount == expectedCount) || (expectedCount == 0)) {
        return setPromise();
      }
    }
    return this;
  }
  
  public boolean tryFailure(Throwable cause)
  {
    if (allowFailure())
    {
      doneCount += 1;
      lastFailure = cause;
      if (allPromisesDone()) {
        return tryPromise();
      }
      return true;
    }
    return false;
  }
  
  public ChannelPromise setFailure(Throwable cause)
  {
    if (allowFailure())
    {
      doneCount += 1;
      lastFailure = cause;
      if (allPromisesDone()) {
        return setPromise();
      }
    }
    return this;
  }
  
  public ChannelPromise setSuccess(Void result)
  {
    if (awaitingPromises())
    {
      doneCount += 1;
      if (allPromisesDone()) {
        setPromise();
      }
    }
    return this;
  }
  
  public boolean trySuccess(Void result)
  {
    if (awaitingPromises())
    {
      doneCount += 1;
      if (allPromisesDone()) {
        return tryPromise();
      }
      return true;
    }
    return false;
  }
  
  private boolean allowFailure()
  {
    return (awaitingPromises()) || (expectedCount == 0);
  }
  
  private boolean awaitingPromises()
  {
    return doneCount < expectedCount;
  }
  
  private boolean allPromisesDone()
  {
    return (doneCount == expectedCount) && (doneAllocating);
  }
  
  private ChannelPromise setPromise()
  {
    if (lastFailure == null)
    {
      promise.setSuccess();
      return super.setSuccess(null);
    }
    promise.setFailure(lastFailure);
    return super.setFailure(lastFailure);
  }
  
  private boolean tryPromise()
  {
    if (lastFailure == null)
    {
      promise.trySuccess();
      return super.trySuccess(null);
    }
    promise.tryFailure(lastFailure);
    return super.tryFailure(lastFailure);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.Http2CodecUtil.SimpleChannelPromiseAggregator
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */