package io.netty.handler.codec.http2;

import io.netty.handler.codec.http.HttpHeaderNames;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.util.AsciiString;
import java.util.Map.Entry;

final class HttpConversionUtil$Http2ToHttpHeaderTranslator
{
  private static final CharSequenceMap<AsciiString> REQUEST_HEADER_TRANSLATIONS = new CharSequenceMap();
  private static final CharSequenceMap<AsciiString> RESPONSE_HEADER_TRANSLATIONS = new CharSequenceMap();
  private final int streamId;
  private final HttpHeaders output;
  private final CharSequenceMap<AsciiString> translations;
  
  static
  {
    RESPONSE_HEADER_TRANSLATIONS.add(Http2Headers.PseudoHeaderName.AUTHORITY.value(), HttpHeaderNames.HOST);
    
    RESPONSE_HEADER_TRANSLATIONS.add(Http2Headers.PseudoHeaderName.SCHEME.value(), HttpConversionUtil.ExtensionHeaderNames.SCHEME.text());
    
    REQUEST_HEADER_TRANSLATIONS.add(RESPONSE_HEADER_TRANSLATIONS);
    RESPONSE_HEADER_TRANSLATIONS.add(Http2Headers.PseudoHeaderName.PATH.value(), HttpConversionUtil.ExtensionHeaderNames.PATH.text());
  }
  
  HttpConversionUtil$Http2ToHttpHeaderTranslator(int streamId, HttpHeaders output, boolean request)
  {
    this.streamId = streamId;
    this.output = output;
    translations = (request ? REQUEST_HEADER_TRANSLATIONS : RESPONSE_HEADER_TRANSLATIONS);
  }
  
  public void translate(Map.Entry<CharSequence, CharSequence> entry)
    throws Http2Exception
  {
    CharSequence name = (CharSequence)entry.getKey();
    CharSequence value = (CharSequence)entry.getValue();
    AsciiString translatedName = (AsciiString)translations.get(name);
    if (translatedName != null)
    {
      output.add(translatedName, AsciiString.of(value));
    }
    else if (!Http2Headers.PseudoHeaderName.isPseudoHeader(name))
    {
      if ((name.length() == 0) || (name.charAt(0) == ':')) {
        throw Http2Exception.streamError(streamId, Http2Error.PROTOCOL_ERROR, "Invalid HTTP/2 header '%s' encountered in translation to HTTP/1.x", new Object[] { name });
      }
      if (HttpHeaderNames.COOKIE.equals(name))
      {
        String existingCookie = output.get(HttpHeaderNames.COOKIE);
        output.set(HttpHeaderNames.COOKIE, existingCookie != null ? existingCookie + "; " + value : value);
      }
      else
      {
        output.add(name, value);
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.HttpConversionUtil.Http2ToHttpHeaderTranslator
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */