package io.netty.handler.codec.http2;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPromise;
import io.netty.handler.codec.http.FullHttpMessage;
import io.netty.handler.codec.http.HttpContent;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.handler.codec.http.HttpMessage;
import io.netty.handler.codec.http.LastHttpContent;
import io.netty.util.ReferenceCountUtil;

public class HttpToHttp2ConnectionHandler
  extends Http2ConnectionHandler
{
  private final boolean validateHeaders;
  private int currentStreamId;
  
  protected HttpToHttp2ConnectionHandler(Http2ConnectionDecoder decoder, Http2ConnectionEncoder encoder, Http2Settings initialSettings, boolean validateHeaders)
  {
    super(decoder, encoder, initialSettings);
    this.validateHeaders = validateHeaders;
  }
  
  private int getStreamId(HttpHeaders httpHeaders)
    throws Exception
  {
    return httpHeaders.getInt(HttpConversionUtil.ExtensionHeaderNames.STREAM_ID.text(), connection().local().incrementAndGetNextStreamId());
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise)
  {
    if ((!(msg instanceof HttpMessage)) && (!(msg instanceof HttpContent)))
    {
      ctx.write(msg, promise);
      return;
    }
    boolean release = true;
    Http2CodecUtil.SimpleChannelPromiseAggregator promiseAggregator = new Http2CodecUtil.SimpleChannelPromiseAggregator(promise, ctx.channel(), ctx.executor());
    try
    {
      Http2ConnectionEncoder encoder = encoder();
      boolean endStream = false;
      if ((msg instanceof HttpMessage))
      {
        HttpMessage httpMsg = (HttpMessage)msg;
        
        currentStreamId = getStreamId(httpMsg.headers());
        
        Http2Headers http2Headers = HttpConversionUtil.toHttp2Headers(httpMsg, validateHeaders);
        endStream = ((msg instanceof FullHttpMessage)) && (!((FullHttpMessage)msg).content().isReadable());
        encoder.writeHeaders(ctx, currentStreamId, http2Headers, 0, endStream, promiseAggregator.newPromise());
      }
      if ((!endStream) && ((msg instanceof HttpContent)))
      {
        boolean isLastContent = false;
        Http2Headers trailers = EmptyHttp2Headers.INSTANCE;
        if ((msg instanceof LastHttpContent))
        {
          isLastContent = true;
          
          LastHttpContent lastContent = (LastHttpContent)msg;
          trailers = HttpConversionUtil.toHttp2Headers(lastContent.trailingHeaders(), validateHeaders);
        }
        ByteBuf content = ((HttpContent)msg).content();
        endStream = (isLastContent) && (trailers.isEmpty());
        release = false;
        encoder.writeData(ctx, currentStreamId, content, 0, endStream, promiseAggregator.newPromise());
        if (!trailers.isEmpty()) {
          encoder.writeHeaders(ctx, currentStreamId, trailers, 0, true, promiseAggregator.newPromise());
        }
      }
    }
    catch (Throwable t)
    {
      promiseAggregator.setFailure(t);
    }
    finally
    {
      if (release) {
        ReferenceCountUtil.release(msg);
      }
      promiseAggregator.doneAllocatingPromises();
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.HttpToHttp2ConnectionHandler
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */