package io.netty.handler.codec.http2;

import io.netty.util.internal.MathUtil;
import io.netty.util.internal.PriorityQueue;
import io.netty.util.internal.PriorityQueueNode;
import java.util.Queue;

final class WeightedFairQueueByteDistributor$State
  implements PriorityQueueNode<State>
{
  final Http2Stream stream;
  private final Queue<State> queue;
  int streamableBytes;
  int activeCountForTree;
  private int priorityQueueIndex = -1;
  long pseudoTimeToWrite;
  long pseudoTime;
  long totalQueuedWeights;
  boolean active;
  
  WeightedFairQueueByteDistributor$State(WeightedFairQueueByteDistributor paramWeightedFairQueueByteDistributor, Http2Stream stream)
  {
    this(paramWeightedFairQueueByteDistributor, stream, 0);
  }
  
  WeightedFairQueueByteDistributor$State(WeightedFairQueueByteDistributor paramWeightedFairQueueByteDistributor, Http2Stream stream, int initialSize)
  {
    this.stream = stream;
    queue = new PriorityQueue(initialSize);
  }
  
  void write(int numBytes, StreamByteDistributor.Writer writer)
    throws Http2Exception
  {
    try
    {
      writer.write(stream, numBytes);
    }
    catch (Throwable t)
    {
      throw Http2Exception.connectionError(Http2Error.INTERNAL_ERROR, t, "byte distribution write error", new Object[0]);
    }
  }
  
  void isActiveCountChangeForTree(int increment)
  {
    assert (activeCountForTree + increment >= 0);
    activeCountForTree += increment;
    if (!stream.isRoot())
    {
      State pState = WeightedFairQueueByteDistributor.access$100(this$0, stream.parent());
      if (activeCountForTree == 0) {
        pState.remove(this);
      } else if (activeCountForTree - increment == 0) {
        pState.offerAndInitializePseudoTime(this);
      }
      pState.isActiveCountChangeForTree(increment);
    }
  }
  
  void updateStreamableBytes(int newStreamableBytes, boolean isActive)
  {
    if (active != isActive)
    {
      isActiveCountChangeForTree(isActive ? 1 : -1);
      active = isActive;
    }
    streamableBytes = newStreamableBytes;
  }
  
  void updatePseudoTime(State parentState, int nsent, long totalQueuedWeights)
  {
    assert ((stream.id() != 0) && (nsent >= 0));
    
    pseudoTimeToWrite = (Math.min(pseudoTimeToWrite, pseudoTime) + nsent * totalQueuedWeights / stream.weight());
  }
  
  void offerAndInitializePseudoTime(State state)
  {
    pseudoTimeToWrite = pseudoTime;
    offer(state);
  }
  
  void offer(State state)
  {
    queue.offer(state);
    totalQueuedWeights += stream.weight();
  }
  
  State poll()
  {
    State state = (State)queue.poll();
    
    totalQueuedWeights -= stream.weight();
    return state;
  }
  
  void remove(State state)
  {
    if (queue.remove(state)) {
      totalQueuedWeights -= stream.weight();
    }
  }
  
  State peek()
  {
    return (State)queue.peek();
  }
  
  void close()
  {
    updateStreamableBytes(0, false);
  }
  
  public int compareTo(State o)
  {
    return MathUtil.compare(pseudoTimeToWrite, pseudoTimeToWrite);
  }
  
  public int priorityQueueIndex()
  {
    return priorityQueueIndex;
  }
  
  public void priorityQueueIndex(int i)
  {
    priorityQueueIndex = i;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.WeightedFairQueueByteDistributor.State
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */