package io.netty.handler.codec.http2.internal.hpack;

import io.netty.handler.codec.http2.Http2Exception;
import io.netty.util.AsciiString;
import io.netty.util.ByteProcessor;
import io.netty.util.internal.ObjectUtil;

final class HuffmanDecoder$DecoderProcessor
  implements ByteProcessor
{
  private final int initialCapacity;
  private byte[] bytes;
  private int index;
  private HuffmanDecoder.Node node;
  private int current;
  private int currentBits;
  private int symbolBits;
  
  HuffmanDecoder$DecoderProcessor(int initialCapacity)
  {
    this.initialCapacity = ObjectUtil.checkPositive(initialCapacity, "initialCapacity");
  }
  
  void reset()
  {
    node = HuffmanDecoder.access$200();
    current = 0;
    currentBits = 0;
    symbolBits = 0;
    bytes = new byte[initialCapacity];
    index = 0;
  }
  
  public boolean process(byte value)
    throws Http2Exception
  {
    current = (current << 8 | value & 0xFF);
    currentBits += 8;
    symbolBits += 8;
    do
    {
      node = HuffmanDecoder.Node.access$100(node)[(current >>> currentBits - 8 & 0xFF)];
      currentBits -= HuffmanDecoder.Node.access$300(node);
      if (HuffmanDecoder.Node.access$000(node))
      {
        if (HuffmanDecoder.Node.access$400(node) == 256) {
          throw HuffmanDecoder.access$500();
        }
        append(HuffmanDecoder.Node.access$400(node));
        node = HuffmanDecoder.access$200();
        
        symbolBits = currentBits;
      }
    } while (currentBits >= 8);
    return true;
  }
  
  AsciiString end()
    throws Http2Exception
  {
    while (currentBits > 0)
    {
      node = HuffmanDecoder.Node.access$100(node)[(current << 8 - currentBits & 0xFF)];
      if ((!HuffmanDecoder.Node.access$000(node)) || (HuffmanDecoder.Node.access$300(node) > currentBits)) {
        break;
      }
      if (HuffmanDecoder.Node.access$400(node) == 256) {
        throw HuffmanDecoder.access$500();
      }
      currentBits -= HuffmanDecoder.Node.access$300(node);
      append(HuffmanDecoder.Node.access$400(node));
      node = HuffmanDecoder.access$200();
      symbolBits = currentBits;
    }
    int mask = (1 << symbolBits) - 1;
    if ((symbolBits > 7) || ((current & mask) != mask)) {
      throw HuffmanDecoder.access$600();
    }
    return new AsciiString(bytes, 0, index, false);
  }
  
  private void append(int i)
  {
    try
    {
      bytes[index] = ((byte)i);
    }
    catch (IndexOutOfBoundsException ignore)
    {
      byte[] newBytes = new byte[bytes.length + initialCapacity];
      System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
      bytes = newBytes;
      bytes[index] = ((byte)i);
    }
    index += 1;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.http2.internal.hpack.HuffmanDecoder.DecoderProcessor
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */