package io.netty.handler.codec.rtsp;

import io.netty.handler.codec.http.DefaultFullHttpRequest;
import io.netty.handler.codec.http.DefaultFullHttpResponse;
import io.netty.handler.codec.http.DefaultHttpRequest;
import io.netty.handler.codec.http.DefaultHttpResponse;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.handler.codec.http.HttpMessage;
import io.netty.handler.codec.http.HttpObjectDecoder;
import io.netty.handler.codec.http.HttpResponseStatus;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RtspDecoder
  extends HttpObjectDecoder
{
  private static final HttpResponseStatus UNKNOWN_STATUS = new HttpResponseStatus(999, "Unknown");
  private boolean isDecodingRequest;
  private static final Pattern versionPattern = Pattern.compile("RTSP/\\d\\.\\d");
  public static final int DEFAULT_MAX_INITIAL_LINE_LENGTH = 4096;
  public static final int DEFAULT_MAX_HEADER_SIZE = 8192;
  public static final int DEFAULT_MAX_CONTENT_LENGTH = 8192;
  
  public RtspDecoder()
  {
    this(4096, 8192, 8192);
  }
  
  public RtspDecoder(int maxInitialLineLength, int maxHeaderSize, int maxContentLength)
  {
    super(maxInitialLineLength, maxHeaderSize, maxContentLength * 2, false);
  }
  
  public RtspDecoder(int maxInitialLineLength, int maxHeaderSize, int maxContentLength, boolean validateHeaders)
  {
    super(maxInitialLineLength, maxHeaderSize, maxContentLength * 2, false, validateHeaders);
  }
  
  protected HttpMessage createMessage(String[] initialLine)
    throws Exception
  {
    if (versionPattern.matcher(initialLine[0]).matches())
    {
      isDecodingRequest = false;
      return new DefaultHttpResponse(RtspVersions.valueOf(initialLine[0]), new HttpResponseStatus(Integer.parseInt(initialLine[1]), initialLine[2]), validateHeaders);
    }
    isDecodingRequest = true;
    return new DefaultHttpRequest(RtspVersions.valueOf(initialLine[2]), RtspMethods.valueOf(initialLine[0]), initialLine[1], validateHeaders);
  }
  
  protected boolean isContentAlwaysEmpty(HttpMessage msg)
  {
    return (super.isContentAlwaysEmpty(msg)) || (!msg.headers().contains(RtspHeaderNames.CONTENT_LENGTH));
  }
  
  protected HttpMessage createInvalidMessage()
  {
    if (isDecodingRequest) {
      return new DefaultFullHttpRequest(RtspVersions.RTSP_1_0, RtspMethods.OPTIONS, "/bad-request", validateHeaders);
    }
    return new DefaultFullHttpResponse(RtspVersions.RTSP_1_0, UNKNOWN_STATUS, validateHeaders);
  }
  
  protected boolean isDecodingRequest()
  {
    return isDecodingRequest;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.rtsp.RtspDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */