package io.netty.handler.codec.serialization;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufOutputStream;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import io.netty.util.Attribute;
import io.netty.util.AttributeKey;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

public class CompatibleObjectEncoder
  extends MessageToByteEncoder<Serializable>
{
  private static final AttributeKey<ObjectOutputStream> OOS = AttributeKey.valueOf(CompatibleObjectEncoder.class, "OOS");
  private final int resetInterval;
  private int writtenObjects;
  
  public CompatibleObjectEncoder()
  {
    this(16);
  }
  
  public CompatibleObjectEncoder(int resetInterval)
  {
    if (resetInterval < 0) {
      throw new IllegalArgumentException("resetInterval: " + resetInterval);
    }
    this.resetInterval = resetInterval;
  }
  
  protected ObjectOutputStream newObjectOutputStream(OutputStream out)
    throws Exception
  {
    return new ObjectOutputStream(out);
  }
  
  protected void encode(ChannelHandlerContext ctx, Serializable msg, ByteBuf out)
    throws Exception
  {
    Attribute<ObjectOutputStream> oosAttr = ctx.attr(OOS);
    ObjectOutputStream oos = (ObjectOutputStream)oosAttr.get();
    if (oos == null)
    {
      oos = newObjectOutputStream(new ByteBufOutputStream(out));
      ObjectOutputStream newOos = (ObjectOutputStream)oosAttr.setIfAbsent(oos);
      if (newOos != null) {
        oos = newOos;
      }
    }
    synchronized (oos)
    {
      if (resetInterval != 0)
      {
        writtenObjects += 1;
        if (writtenObjects % resetInterval == 0) {
          oos.reset();
        }
      }
      oos.writeObject(msg);
      oos.flush();
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.serialization.CompatibleObjectEncoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */