package io.netty.handler.codec.socks;

import io.netty.buffer.ByteBuf;
import io.netty.util.CharsetUtil;
import java.nio.charset.CharsetEncoder;

public final class SocksAuthRequest
  extends SocksRequest
{
  private static final CharsetEncoder asciiEncoder = CharsetUtil.encoder(CharsetUtil.US_ASCII);
  private static final SocksSubnegotiationVersion SUBNEGOTIATION_VERSION = SocksSubnegotiationVersion.AUTH_PASSWORD;
  private final String username;
  private final String password;
  
  public SocksAuthRequest(String username, String password)
  {
    super(SocksRequestType.AUTH);
    if (username == null) {
      throw new NullPointerException("username");
    }
    if (password == null) {
      throw new NullPointerException("username");
    }
    if ((!asciiEncoder.canEncode(username)) || (!asciiEncoder.canEncode(password))) {
      throw new IllegalArgumentException("username: " + username + " or password: **** values should be in pure ascii");
    }
    if (username.length() > 255) {
      throw new IllegalArgumentException("username: " + username + " exceeds 255 char limit");
    }
    if (password.length() > 255) {
      throw new IllegalArgumentException("password: **** exceeds 255 char limit");
    }
    this.username = username;
    this.password = password;
  }
  
  public String username()
  {
    return username;
  }
  
  public String password()
  {
    return password;
  }
  
  public void encodeAsByteBuf(ByteBuf byteBuf)
  {
    byteBuf.writeByte(SUBNEGOTIATION_VERSION.byteValue());
    byteBuf.writeByte(username.length());
    byteBuf.writeBytes(username.getBytes(CharsetUtil.US_ASCII));
    byteBuf.writeByte(password.length());
    byteBuf.writeBytes(password.getBytes(CharsetUtil.US_ASCII));
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.socks.SocksAuthRequest
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */