package io.netty.handler.codec.socks;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.ReplayingDecoder;
import java.util.List;

public class SocksAuthRequestDecoder
  extends ReplayingDecoder<State>
{
  private SocksSubnegotiationVersion version;
  private int fieldLength;
  private String username;
  private String password;
  private SocksRequest msg = SocksCommonUtils.UNKNOWN_SOCKS_REQUEST;
  
  public SocksAuthRequestDecoder()
  {
    super(State.CHECK_PROTOCOL_VERSION);
  }
  
  protected void decode(ChannelHandlerContext ctx, ByteBuf byteBuf, List<Object> out)
    throws Exception
  {
    switch ((State)state())
    {
    case CHECK_PROTOCOL_VERSION: 
      version = SocksSubnegotiationVersion.valueOf(byteBuf.readByte());
      if (version == SocksSubnegotiationVersion.AUTH_PASSWORD) {
        checkpoint(State.READ_USERNAME);
      }
      break;
    case READ_USERNAME: 
      fieldLength = byteBuf.readByte();
      username = SocksCommonUtils.readUsAscii(byteBuf, fieldLength);
      checkpoint(State.READ_PASSWORD);
    case READ_PASSWORD: 
      fieldLength = byteBuf.readByte();
      password = SocksCommonUtils.readUsAscii(byteBuf, fieldLength);
      msg = new SocksAuthRequest(username, password);
    }
    ctx.pipeline().remove(this);
    out.add(msg);
  }
  
  static enum State
  {
    CHECK_PROTOCOL_VERSION,  READ_USERNAME,  READ_PASSWORD;
    
    private State() {}
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.socks.SocksAuthRequestDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */