package io.netty.handler.codec.socks;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.ReplayingDecoder;
import java.util.List;

public class SocksAuthResponseDecoder
  extends ReplayingDecoder<State>
{
  private SocksSubnegotiationVersion version;
  private SocksAuthStatus authStatus;
  private SocksResponse msg = SocksCommonUtils.UNKNOWN_SOCKS_RESPONSE;
  
  public SocksAuthResponseDecoder()
  {
    super(State.CHECK_PROTOCOL_VERSION);
  }
  
  protected void decode(ChannelHandlerContext channelHandlerContext, ByteBuf byteBuf, List<Object> out)
    throws Exception
  {
    switch ((State)state())
    {
    case CHECK_PROTOCOL_VERSION: 
      version = SocksSubnegotiationVersion.valueOf(byteBuf.readByte());
      if (version == SocksSubnegotiationVersion.AUTH_PASSWORD) {
        checkpoint(State.READ_AUTH_RESPONSE);
      }
      break;
    case READ_AUTH_RESPONSE: 
      authStatus = SocksAuthStatus.valueOf(byteBuf.readByte());
      msg = new SocksAuthResponse(authStatus);
    }
    channelHandlerContext.pipeline().remove(this);
    out.add(msg);
  }
  
  static enum State
  {
    CHECK_PROTOCOL_VERSION,  READ_AUTH_RESPONSE;
    
    private State() {}
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.socks.SocksAuthResponseDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */