package io.netty.handler.codec.socks;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.ReplayingDecoder;
import java.util.List;

public class SocksCmdResponseDecoder
  extends ReplayingDecoder<State>
{
  private SocksProtocolVersion version;
  private int fieldLength;
  private SocksCmdStatus cmdStatus;
  private SocksAddressType addressType;
  private byte reserved;
  private String host;
  private int port;
  private SocksResponse msg = SocksCommonUtils.UNKNOWN_SOCKS_RESPONSE;
  
  public SocksCmdResponseDecoder()
  {
    super(State.CHECK_PROTOCOL_VERSION);
  }
  
  protected void decode(ChannelHandlerContext ctx, ByteBuf byteBuf, List<Object> out)
    throws Exception
  {
    switch ((State)state())
    {
    case CHECK_PROTOCOL_VERSION: 
      version = SocksProtocolVersion.valueOf(byteBuf.readByte());
      if (version == SocksProtocolVersion.SOCKS5) {
        checkpoint(State.READ_CMD_HEADER);
      }
      break;
    case READ_CMD_HEADER: 
      cmdStatus = SocksCmdStatus.valueOf(byteBuf.readByte());
      reserved = byteBuf.readByte();
      addressType = SocksAddressType.valueOf(byteBuf.readByte());
      checkpoint(State.READ_CMD_ADDRESS);
    case READ_CMD_ADDRESS: 
      switch (addressType)
      {
      case IPv4: 
        host = SocksCommonUtils.intToIp(byteBuf.readInt());
        port = byteBuf.readUnsignedShort();
        msg = new SocksCmdResponse(cmdStatus, addressType, host, port);
        break;
      case DOMAIN: 
        fieldLength = byteBuf.readByte();
        host = SocksCommonUtils.readUsAscii(byteBuf, fieldLength);
        port = byteBuf.readUnsignedShort();
        msg = new SocksCmdResponse(cmdStatus, addressType, host, port);
        break;
      case IPv6: 
        byte[] bytes = new byte[16];
        byteBuf.readBytes(bytes);
        host = SocksCommonUtils.ipv6toStr(bytes);
        port = byteBuf.readUnsignedShort();
        msg = new SocksCmdResponse(cmdStatus, addressType, host, port);
      }
      break;
    }
    ctx.pipeline().remove(this);
    out.add(msg);
  }
  
  static enum State
  {
    CHECK_PROTOCOL_VERSION,  READ_CMD_HEADER,  READ_CMD_ADDRESS;
    
    private State() {}
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.socks.SocksCmdResponseDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */