package io.netty.handler.codec.socks;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.ReplayingDecoder;
import java.util.ArrayList;
import java.util.List;

public class SocksInitRequestDecoder
  extends ReplayingDecoder<State>
{
  private final List<SocksAuthScheme> authSchemes = new ArrayList();
  private SocksProtocolVersion version;
  private byte authSchemeNum;
  private SocksRequest msg = SocksCommonUtils.UNKNOWN_SOCKS_REQUEST;
  
  public SocksInitRequestDecoder()
  {
    super(State.CHECK_PROTOCOL_VERSION);
  }
  
  protected void decode(ChannelHandlerContext ctx, ByteBuf byteBuf, List<Object> out)
    throws Exception
  {
    switch ((State)state())
    {
    case CHECK_PROTOCOL_VERSION: 
      version = SocksProtocolVersion.valueOf(byteBuf.readByte());
      if (version == SocksProtocolVersion.SOCKS5) {
        checkpoint(State.READ_AUTH_SCHEMES);
      }
      break;
    case READ_AUTH_SCHEMES: 
      authSchemes.clear();
      authSchemeNum = byteBuf.readByte();
      for (int i = 0; i < authSchemeNum; i++) {
        authSchemes.add(SocksAuthScheme.valueOf(byteBuf.readByte()));
      }
      msg = new SocksInitRequest(authSchemes);
    }
    ctx.pipeline().remove(this);
    out.add(msg);
  }
  
  static enum State
  {
    CHECK_PROTOCOL_VERSION,  READ_AUTH_SCHEMES;
    
    private State() {}
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.socks.SocksInitRequestDecoder
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */