package io.netty.handler.codec.spdy;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicInteger;

final class SpdySession$StreamState
{
  private final byte priority;
  private boolean remoteSideClosed;
  private boolean localSideClosed;
  private boolean receivedReply;
  private final AtomicInteger sendWindowSize;
  private final AtomicInteger receiveWindowSize;
  private int receiveWindowSizeLowerBound;
  private final Queue<SpdySession.PendingWrite> pendingWriteQueue = new ConcurrentLinkedQueue();
  
  SpdySession$StreamState(byte priority, boolean remoteSideClosed, boolean localSideClosed, int sendWindowSize, int receiveWindowSize)
  {
    this.priority = priority;
    this.remoteSideClosed = remoteSideClosed;
    this.localSideClosed = localSideClosed;
    this.sendWindowSize = new AtomicInteger(sendWindowSize);
    this.receiveWindowSize = new AtomicInteger(receiveWindowSize);
  }
  
  byte getPriority()
  {
    return priority;
  }
  
  boolean isRemoteSideClosed()
  {
    return remoteSideClosed;
  }
  
  void closeRemoteSide()
  {
    remoteSideClosed = true;
  }
  
  boolean isLocalSideClosed()
  {
    return localSideClosed;
  }
  
  void closeLocalSide()
  {
    localSideClosed = true;
  }
  
  boolean hasReceivedReply()
  {
    return receivedReply;
  }
  
  void receivedReply()
  {
    receivedReply = true;
  }
  
  int getSendWindowSize()
  {
    return sendWindowSize.get();
  }
  
  int updateSendWindowSize(int deltaWindowSize)
  {
    return sendWindowSize.addAndGet(deltaWindowSize);
  }
  
  int updateReceiveWindowSize(int deltaWindowSize)
  {
    return receiveWindowSize.addAndGet(deltaWindowSize);
  }
  
  int getReceiveWindowSizeLowerBound()
  {
    return receiveWindowSizeLowerBound;
  }
  
  void setReceiveWindowSizeLowerBound(int receiveWindowSizeLowerBound)
  {
    this.receiveWindowSizeLowerBound = receiveWindowSizeLowerBound;
  }
  
  boolean putPendingWrite(SpdySession.PendingWrite msg)
  {
    return pendingWriteQueue.offer(msg);
  }
  
  SpdySession.PendingWrite getPendingWrite()
  {
    return (SpdySession.PendingWrite)pendingWriteQueue.peek();
  }
  
  SpdySession.PendingWrite removePendingWrite()
  {
    return (SpdySession.PendingWrite)pendingWriteQueue.poll();
  }
  
  void clearPendingWrites(Throwable cause)
  {
    for (;;)
    {
      SpdySession.PendingWrite pendingWrite = (SpdySession.PendingWrite)pendingWriteQueue.poll();
      if (pendingWrite == null) {
        break;
      }
      pendingWrite.fail(cause);
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.codec.spdy.SpdySession.StreamState
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */