package io.netty.handler.ipfilter;

import java.net.Inet4Address;
import java.net.InetSocketAddress;

final class IpSubnetFilterRule$Ip4SubnetFilterRule
  implements IpFilterRule
{
  private final int networkAddress;
  private final int subnetMask;
  private final IpFilterRuleType ruleType;
  
  private IpSubnetFilterRule$Ip4SubnetFilterRule(Inet4Address ipAddress, int cidrPrefix, IpFilterRuleType ruleType)
  {
    if ((cidrPrefix < 0) || (cidrPrefix > 32)) {
      throw new IllegalArgumentException(String.format("IPv4 requires the subnet prefix to be in range of [0,32]. The prefix was: %d", new Object[] { Integer.valueOf(cidrPrefix) }));
    }
    subnetMask = prefixToSubnetMask(cidrPrefix);
    networkAddress = (ipToInt(ipAddress) & subnetMask);
    this.ruleType = ruleType;
  }
  
  public boolean matches(InetSocketAddress remoteAddress)
  {
    int ipAddress = ipToInt((Inet4Address)remoteAddress.getAddress());
    
    return (ipAddress & subnetMask) == networkAddress;
  }
  
  public IpFilterRuleType ruleType()
  {
    return ruleType;
  }
  
  private static int ipToInt(Inet4Address ipAddress)
  {
    byte[] octets = ipAddress.getAddress();
    assert (octets.length == 4);
    
    return (octets[0] & 0xFF) << 24 | (octets[1] & 0xFF) << 16 | (octets[2] & 0xFF) << 8 | octets[3] & 0xFF;
  }
  
  private static int prefixToSubnetMask(int cidrPrefix)
  {
    return (int)(-1L << 32 - cidrPrefix & 0xFFFFFFFFFFFFFFFF);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.ipfilter.IpSubnetFilterRule.Ip4SubnetFilterRule
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */