package io.netty.handler.ipfilter;

import java.math.BigInteger;
import java.net.Inet6Address;
import java.net.InetSocketAddress;

final class IpSubnetFilterRule$Ip6SubnetFilterRule
  implements IpFilterRule
{
  private static final BigInteger MINUS_ONE = BigInteger.valueOf(-1L);
  private final BigInteger networkAddress;
  private final BigInteger subnetMask;
  private final IpFilterRuleType ruleType;
  
  private IpSubnetFilterRule$Ip6SubnetFilterRule(Inet6Address ipAddress, int cidrPrefix, IpFilterRuleType ruleType)
  {
    if ((cidrPrefix < 0) || (cidrPrefix > 128)) {
      throw new IllegalArgumentException(String.format("IPv6 requires the subnet prefix to be in range of [0,128]. The prefix was: %d", new Object[] { Integer.valueOf(cidrPrefix) }));
    }
    subnetMask = prefixToSubnetMask(cidrPrefix);
    networkAddress = ipToInt(ipAddress).and(subnetMask);
    this.ruleType = ruleType;
  }
  
  public boolean matches(InetSocketAddress remoteAddress)
  {
    BigInteger ipAddress = ipToInt((Inet6Address)remoteAddress.getAddress());
    
    return ipAddress.and(subnetMask).equals(networkAddress);
  }
  
  public IpFilterRuleType ruleType()
  {
    return ruleType;
  }
  
  private static BigInteger ipToInt(Inet6Address ipAddress)
  {
    byte[] octets = ipAddress.getAddress();
    assert (octets.length == 16);
    
    return new BigInteger(octets);
  }
  
  private static BigInteger prefixToSubnetMask(int cidrPrefix)
  {
    return MINUS_ONE.shiftLeft(128 - cidrPrefix);
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.ipfilter.IpSubnetFilterRule.Ip6SubnetFilterRule
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */