package io.netty.handler.ssl;

import io.netty.util.internal.logging.InternalLogger;
import java.util.HashSet;
import java.util.Set;
import javax.net.ssl.SSLHandshakeException;
import javax.security.auth.x500.X500Principal;
import org.apache.tomcat.jni.CertificateRequestedCallback;
import org.apache.tomcat.jni.CertificateRequestedCallback.KeyMaterial;

final class ReferenceCountedOpenSslClientContext$OpenSslCertificateRequestedCallback
  implements CertificateRequestedCallback
{
  private final OpenSslEngineMap engineMap;
  private final OpenSslKeyMaterialManager keyManagerHolder;
  
  ReferenceCountedOpenSslClientContext$OpenSslCertificateRequestedCallback(OpenSslEngineMap engineMap, OpenSslKeyMaterialManager keyManagerHolder)
  {
    this.engineMap = engineMap;
    this.keyManagerHolder = keyManagerHolder;
  }
  
  public CertificateRequestedCallback.KeyMaterial requested(long ssl, byte[] keyTypeBytes, byte[][] asn1DerEncodedPrincipals)
  {
    ReferenceCountedOpenSslEngine engine = engineMap.get(ssl);
    try
    {
      Set<String> keyTypesSet = supportedClientKeyTypes(keyTypeBytes);
      String[] keyTypes = (String[])keyTypesSet.toArray(new String[keyTypesSet.size()]);
      X500Principal[] issuers;
      X500Principal[] issuers;
      if (asn1DerEncodedPrincipals == null)
      {
        issuers = null;
      }
      else
      {
        issuers = new X500Principal[asn1DerEncodedPrincipals.length];
        for (int i = 0; i < asn1DerEncodedPrincipals.length; i++) {
          issuers[i] = new X500Principal(asn1DerEncodedPrincipals[i]);
        }
      }
      return keyManagerHolder.keyMaterial(engine, keyTypes, issuers);
    }
    catch (Throwable cause)
    {
      ReferenceCountedOpenSslClientContext.access$000().debug("request of key failed", cause);
      SSLHandshakeException e = new SSLHandshakeException("General OpenSslEngine problem");
      e.initCause(cause);
      handshakeException = e;
    }
    return null;
  }
  
  private static Set<String> supportedClientKeyTypes(byte[] clientCertificateTypes)
  {
    Set<String> result = new HashSet(clientCertificateTypes.length);
    for (byte keyTypeCode : clientCertificateTypes)
    {
      String keyType = clientKeyType(keyTypeCode);
      if (keyType != null) {
        result.add(keyType);
      }
    }
    return result;
  }
  
  private static String clientKeyType(byte clientCertificateType)
  {
    switch (clientCertificateType)
    {
    case 1: 
      return "RSA";
    case 3: 
      return "DH_RSA";
    case 64: 
      return "EC";
    case 65: 
      return "EC_RSA";
    case 66: 
      return "EC_EC";
    }
    return null;
  }
}

/* Location:
 * Qualified Name:     io.netty.handler.ssl.ReferenceCountedOpenSslClientContext.OpenSslCertificateRequestedCallback
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */