package io.netty.resolver.dns;

import io.netty.util.NetUtil;
import io.netty.util.collection.IntObjectHashMap;
import io.netty.util.collection.IntObjectMap;
import io.netty.util.internal.ThreadLocalRandom;
import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

final class DnsQueryContextManager
{
  final Map<InetSocketAddress, IntObjectMap<DnsQueryContext>> map = new HashMap();
  
  int add(DnsQueryContext qCtx)
  {
    IntObjectMap<DnsQueryContext> contexts = getOrCreateContextMap(qCtx.nameServerAddr());
    
    int id = ThreadLocalRandom.current().nextInt(1, 65536);
    int maxTries = 131070;
    int tries = 0;
    synchronized (contexts)
    {
      do
      {
        if (!contexts.containsKey(id))
        {
          contexts.put(id, qCtx);
          return id;
        }
        id = id + 1 & 0xFFFF;
        
        tries++;
      } while (tries < 131070);
      throw new IllegalStateException("query ID space exhausted: " + qCtx.question());
    }
  }
  
  DnsQueryContext get(InetSocketAddress nameServerAddr, int id)
  {
    IntObjectMap<DnsQueryContext> contexts = getContextMap(nameServerAddr);
    DnsQueryContext qCtx;
    DnsQueryContext qCtx;
    if (contexts != null)
    {
      DnsQueryContext qCtx;
      synchronized (contexts)
      {
        qCtx = (DnsQueryContext)contexts.get(id);
      }
    }
    else
    {
      qCtx = null;
    }
    return qCtx;
  }
  
  /* Error */
  DnsQueryContext remove(InetSocketAddress nameServerAddr, int id)
  {
    // Byte code:
    //   0: aload_0
    //   1: aload_1
    //   2: invokespecial 23	io/netty/resolver/dns/DnsQueryContextManager:getContextMap	(Ljava/net/InetSocketAddress;)Lio/netty/util/collection/IntObjectMap;
    //   5: astore_3
    //   6: aload_3
    //   7: ifnonnull +5 -> 12
    //   10: aconst_null
    //   11: areturn
    //   12: aload_3
    //   13: dup
    //   14: astore 4
    //   16: monitorenter
    //   17: aload_3
    //   18: iload_2
    //   19: invokeinterface 26 2 0
    //   24: checkcast 25	io/netty/resolver/dns/DnsQueryContext
    //   27: aload 4
    //   29: monitorexit
    //   30: areturn
    //   31: astore 5
    //   33: aload 4
    //   35: monitorexit
    //   36: aload 5
    //   38: athrow
    // Line number table:
    //   Java source line #79	-> byte code offset #0
    //   Java source line #80	-> byte code offset #6
    //   Java source line #81	-> byte code offset #10
    //   Java source line #84	-> byte code offset #12
    //   Java source line #85	-> byte code offset #17
    //   Java source line #86	-> byte code offset #31
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	39	0	this	DnsQueryContextManager
    //   0	39	1	nameServerAddr	InetSocketAddress
    //   0	39	2	id	int
    //   5	13	3	contexts	IntObjectMap<DnsQueryContext>
    //   14	20	4	Ljava/lang/Object;	Object
    //   31	6	5	localObject1	Object
    // Exception table:
    //   from	to	target	type
    //   17	30	31	finally
    //   31	36	31	finally
  }
  
  /* Error */
  private IntObjectMap<DnsQueryContext> getContextMap(InetSocketAddress nameServerAddr)
  {
    // Byte code:
    //   0: aload_0
    //   1: getfield 4	io/netty/resolver/dns/DnsQueryContextManager:map	Ljava/util/Map;
    //   4: dup
    //   5: astore_2
    //   6: monitorenter
    //   7: aload_0
    //   8: getfield 4	io/netty/resolver/dns/DnsQueryContextManager:map	Ljava/util/Map;
    //   11: aload_1
    //   12: invokeinterface 27 2 0
    //   17: checkcast 28	io/netty/util/collection/IntObjectMap
    //   20: aload_2
    //   21: monitorexit
    //   22: areturn
    //   23: astore_3
    //   24: aload_2
    //   25: monitorexit
    //   26: aload_3
    //   27: athrow
    // Line number table:
    //   Java source line #90	-> byte code offset #0
    //   Java source line #91	-> byte code offset #7
    //   Java source line #92	-> byte code offset #23
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	28	0	this	DnsQueryContextManager
    //   0	28	1	nameServerAddr	InetSocketAddress
    //   5	20	2	Ljava/lang/Object;	Object
    //   23	4	3	localObject1	Object
    // Exception table:
    //   from	to	target	type
    //   7	22	23	finally
    //   23	26	23	finally
  }
  
  private IntObjectMap<DnsQueryContext> getOrCreateContextMap(InetSocketAddress nameServerAddr)
  {
    synchronized (map)
    {
      IntObjectMap<DnsQueryContext> contexts = (IntObjectMap)map.get(nameServerAddr);
      if (contexts != null) {
        return contexts;
      }
      IntObjectMap<DnsQueryContext> newContexts = new IntObjectHashMap();
      InetAddress a = nameServerAddr.getAddress();
      int port = nameServerAddr.getPort();
      map.put(nameServerAddr, newContexts);
      if ((a instanceof Inet4Address))
      {
        Inet4Address a4 = (Inet4Address)a;
        if (a4.isLoopbackAddress()) {
          map.put(new InetSocketAddress(NetUtil.LOCALHOST6, port), newContexts);
        } else {
          map.put(new InetSocketAddress(toCompatAddress(a4), port), newContexts);
        }
      }
      else if ((a instanceof Inet6Address))
      {
        Inet6Address a6 = (Inet6Address)a;
        if (a6.isLoopbackAddress()) {
          map.put(new InetSocketAddress(NetUtil.LOCALHOST4, port), newContexts);
        } else if (a6.isIPv4CompatibleAddress()) {
          map.put(new InetSocketAddress(toIPv4Address(a6), port), newContexts);
        }
      }
      return newContexts;
    }
  }
  
  private static Inet6Address toCompatAddress(Inet4Address a4)
  {
    byte[] b4 = a4.getAddress();
    byte[] b6 = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, b4[0], b4[1], b4[2], b4[3] };
    try
    {
      return (Inet6Address)InetAddress.getByAddress(b6);
    }
    catch (UnknownHostException e)
    {
      throw new Error(e);
    }
  }
  
  private static Inet4Address toIPv4Address(Inet6Address a6)
  {
    byte[] b6 = a6.getAddress();
    byte[] b4 = { b6[12], b6[13], b6[14], b6[15] };
    try
    {
      return (Inet4Address)InetAddress.getByAddress(b4);
    }
    catch (UnknownHostException e)
    {
      throw new Error(e);
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.resolver.dns.DnsQueryContextManager
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */