package io.netty.util;

import io.netty.util.internal.PlatformDependent;
import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;

public abstract class AbstractReferenceCounted
  implements ReferenceCounted
{
  private static final AtomicIntegerFieldUpdater<AbstractReferenceCounted> refCntUpdater;
  
  static
  {
    AtomicIntegerFieldUpdater<AbstractReferenceCounted> updater = PlatformDependent.newAtomicIntegerFieldUpdater(AbstractReferenceCounted.class, "refCnt");
    if (updater == null) {
      updater = AtomicIntegerFieldUpdater.newUpdater(AbstractReferenceCounted.class, "refCnt");
    }
    refCntUpdater = updater;
  }
  
  private volatile int refCnt = 1;
  
  public final int refCnt()
  {
    return refCnt;
  }
  
  protected final void setRefCnt(int refCnt)
  {
    this.refCnt = refCnt;
  }
  
  public ReferenceCounted retain()
  {
    for (;;)
    {
      int refCnt = this.refCnt;
      int nextCnt = refCnt + 1;
      if (nextCnt <= 1) {
        throw new IllegalReferenceCountException(refCnt, 1);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, nextCnt)) {
        break;
      }
    }
    return this;
  }
  
  public ReferenceCounted retain(int increment)
  {
    if (increment <= 0) {
      throw new IllegalArgumentException("increment: " + increment + " (expected: > 0)");
    }
    for (;;)
    {
      int refCnt = this.refCnt;
      int nextCnt = refCnt + increment;
      if (nextCnt <= increment) {
        throw new IllegalReferenceCountException(refCnt, increment);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, nextCnt)) {
        break;
      }
    }
    return this;
  }
  
  public ReferenceCounted touch()
  {
    return touch(null);
  }
  
  public boolean release()
  {
    for (;;)
    {
      int refCnt = this.refCnt;
      if (refCnt == 0) {
        throw new IllegalReferenceCountException(0, -1);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt - 1))
      {
        if (refCnt == 1)
        {
          deallocate();
          return true;
        }
        return false;
      }
    }
  }
  
  public boolean release(int decrement)
  {
    if (decrement <= 0) {
      throw new IllegalArgumentException("decrement: " + decrement + " (expected: > 0)");
    }
    for (;;)
    {
      int refCnt = this.refCnt;
      if (refCnt < decrement) {
        throw new IllegalReferenceCountException(refCnt, -decrement);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt - decrement))
      {
        if (refCnt == decrement)
        {
          deallocate();
          return true;
        }
        return false;
      }
    }
  }
  
  protected abstract void deallocate();
}

/* Location:
 * Qualified Name:     io.netty.util.AbstractReferenceCounted
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */