package io.netty.util;

import io.netty.util.internal.PlatformDependent;
import io.netty.util.internal.StringUtil;
import io.netty.util.internal.logging.InternalLogger;
import java.util.Queue;
import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;

final class HashedWheelTimer$HashedWheelTimeout
  implements Timeout
{
  private static final int ST_INIT = 0;
  private static final int ST_CANCELLED = 1;
  private static final int ST_EXPIRED = 2;
  private static final AtomicIntegerFieldUpdater<HashedWheelTimeout> STATE_UPDATER;
  private final HashedWheelTimer timer;
  private final TimerTask task;
  private final long deadline;
  
  static
  {
    AtomicIntegerFieldUpdater<HashedWheelTimeout> updater = PlatformDependent.newAtomicIntegerFieldUpdater(HashedWheelTimeout.class, "state");
    if (updater == null) {
      updater = AtomicIntegerFieldUpdater.newUpdater(HashedWheelTimeout.class, "state");
    }
    STATE_UPDATER = updater;
  }
  
  private volatile int state = 0;
  long remainingRounds;
  HashedWheelTimeout next;
  HashedWheelTimeout prev;
  HashedWheelTimer.HashedWheelBucket bucket;
  
  HashedWheelTimer$HashedWheelTimeout(HashedWheelTimer timer, TimerTask task, long deadline)
  {
    this.timer = timer;
    this.task = task;
    this.deadline = deadline;
  }
  
  public Timer timer()
  {
    return timer;
  }
  
  public TimerTask task()
  {
    return task;
  }
  
  public boolean cancel()
  {
    if (!compareAndSetState(0, 1)) {
      return false;
    }
    HashedWheelTimer.access$1000(timer).add(this);
    return true;
  }
  
  void remove()
  {
    HashedWheelTimer.HashedWheelBucket bucket = this.bucket;
    if (bucket != null) {
      bucket.remove(this);
    }
  }
  
  public boolean compareAndSetState(int expected, int state)
  {
    return STATE_UPDATER.compareAndSet(this, expected, state);
  }
  
  public int state()
  {
    return state;
  }
  
  public boolean isCancelled()
  {
    return state() == 1;
  }
  
  public boolean isExpired()
  {
    return state() == 2;
  }
  
  public void expire()
  {
    if (!compareAndSetState(0, 2)) {
      return;
    }
    try
    {
      task.run(this);
    }
    catch (Throwable t)
    {
      if (HashedWheelTimer.logger.isWarnEnabled()) {
        HashedWheelTimer.logger.warn("An exception was thrown by " + TimerTask.class.getSimpleName() + '.', t);
      }
    }
  }
  
  public String toString()
  {
    long currentTime = System.nanoTime();
    long remaining = deadline - currentTime + HashedWheelTimer.access$200(timer);
    
    StringBuilder buf = new StringBuilder(192).append(StringUtil.simpleClassName(this)).append('(').append("deadline: ");
    if (remaining > 0L) {
      buf.append(remaining).append(" ns later");
    } else if (remaining < 0L) {
      buf.append(-remaining).append(" ns ago");
    } else {
      buf.append("now");
    }
    if (isCancelled()) {
      buf.append(", cancelled");
    }
    return ", task: " + task() + ')';
  }
}

/* Location:
 * Qualified Name:     io.netty.util.HashedWheelTimer.HashedWheelTimeout
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */