package io.netty.util;

import io.netty.util.internal.StringUtil;
import java.lang.ref.PhantomReference;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.concurrent.atomic.AtomicBoolean;

final class ResourceLeakDetector$DefaultResourceLeak
  extends PhantomReference<Object>
  implements ResourceLeak
{
  private final String creationRecord;
  private final Deque<String> lastRecords = new ArrayDeque();
  private final AtomicBoolean freed;
  private ResourceLeakDetector<T>.DefaultResourceLeak prev;
  private ResourceLeakDetector<T>.DefaultResourceLeak next;
  private int removedRecords;
  
  ResourceLeakDetector$DefaultResourceLeak(ResourceLeakDetector arg1, Object referent)
  {
    super(referent, referent != null ? ResourceLeakDetector.access$200(???) : null);
    ResourceLeakDetector.Level level;
    if (referent != null)
    {
      level = ResourceLeakDetector.getLevel();
      if (level.ordinal() >= ResourceLeakDetector.Level.ADVANCED.ordinal()) {
        creationRecord = ResourceLeakDetector.newRecord(null, 3);
      } else {
        creationRecord = null;
      }
      synchronized (ResourceLeakDetector.access$300(???))
      {
        prev = ResourceLeakDetector.access$300(???);
        next = access$300next;
        access$300next.prev = this;
        access$300next = this;
        ResourceLeakDetector.access$408(???);
      }
      freed = new AtomicBoolean();
    }
    else
    {
      creationRecord = null;
      freed = new AtomicBoolean(true);
    }
  }
  
  public void record()
  {
    record0(null, 3);
  }
  
  public void record(Object hint)
  {
    record0(hint, 3);
  }
  
  private void record0(Object hint, int recordsToSkip)
  {
    if (creationRecord != null)
    {
      String value = ResourceLeakDetector.newRecord(hint, recordsToSkip);
      synchronized (lastRecords)
      {
        int size = lastRecords.size();
        if ((size == 0) || (!((String)lastRecords.getLast()).equals(value))) {
          lastRecords.add(value);
        }
        if (size > ResourceLeakDetector.access$500())
        {
          lastRecords.removeFirst();
          removedRecords += 1;
        }
      }
    }
  }
  
  public boolean close()
  {
    if (freed.compareAndSet(false, true))
    {
      synchronized (ResourceLeakDetector.access$300(this$0))
      {
        ResourceLeakDetector.access$410(this$0);
        prev.next = next;
        next.prev = prev;
        prev = null;
        next = null;
      }
      return true;
    }
    return false;
  }
  
  public String toString()
  {
    if (creationRecord == null) {
      return "";
    }
    int removedRecords;
    synchronized (lastRecords)
    {
      Object[] array = lastRecords.toArray();
      removedRecords = this.removedRecords;
    }
    int removedRecords;
    Object[] array;
    StringBuilder buf = new StringBuilder(16384).append(StringUtil.NEWLINE);
    if (removedRecords > 0) {
      buf.append("WARNING: ").append(removedRecords).append(" leak records were discarded because the leak record count is limited to ").append(ResourceLeakDetector.access$500()).append(". Use system property ").append("io.netty.leakDetection.maxRecords").append(" to increase the limit.").append(StringUtil.NEWLINE);
    }
    buf.append("Recent access records: ").append(array.length).append(StringUtil.NEWLINE);
    if (array.length > 0) {
      for (int i = array.length - 1; i >= 0; i--) {
        buf.append('#').append(i + 1).append(':').append(StringUtil.NEWLINE).append(array[i]);
      }
    }
    buf.append("Created at:").append(StringUtil.NEWLINE).append(creationRecord);
    
    buf.setLength(buf.length() - StringUtil.NEWLINE.length());
    return buf.toString();
  }
}

/* Location:
 * Qualified Name:     io.netty.util.ResourceLeakDetector.DefaultResourceLeak
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */