package io.netty.util;

import io.netty.util.internal.PlatformDependent;
import io.netty.util.internal.SystemPropertyUtil;
import io.netty.util.internal.logging.InternalLogger;
import java.lang.reflect.Constructor;
import java.security.AccessController;
import java.security.PrivilegedAction;

final class ResourceLeakDetectorFactory$DefaultResourceLeakDetectorFactory
  extends ResourceLeakDetectorFactory
{
  private final Constructor<?> customClassConstructor;
  
  ResourceLeakDetectorFactory$DefaultResourceLeakDetectorFactory()
  {
    String customLeakDetector;
    try
    {
      customLeakDetector = (String)AccessController.doPrivileged(new PrivilegedAction()
      {
        public String run()
        {
          return SystemPropertyUtil.get("io.netty.customResourceLeakDetector");
        }
      });
    }
    catch (Throwable cause)
    {
      String customLeakDetector;
      ResourceLeakDetectorFactory.access$000().error("Could not access System property: io.netty.customResourceLeakDetector", cause);
      customLeakDetector = null;
    }
    customClassConstructor = (customLeakDetector == null ? null : customClassConstructor(customLeakDetector));
  }
  
  private static Constructor<?> customClassConstructor(String customLeakDetector)
  {
    try
    {
      Class<?> detectorClass = Class.forName(customLeakDetector, true, PlatformDependent.getSystemClassLoader());
      if (ResourceLeakDetector.class.isAssignableFrom(detectorClass)) {
        return detectorClass.getConstructor(new Class[] { Class.class, Integer.TYPE, Long.TYPE });
      }
      ResourceLeakDetectorFactory.access$000().error("Class {} does not inherit from ResourceLeakDetector.", customLeakDetector);
    }
    catch (Throwable t)
    {
      ResourceLeakDetectorFactory.access$000().error("Could not load custom resource leak detector class provided: {}", customLeakDetector, t);
    }
    return null;
  }
  
  public <T> ResourceLeakDetector<T> newResourceLeakDetector(Class<T> resource, int samplingInterval, long maxActive)
  {
    if (customClassConstructor != null) {
      try
      {
        ResourceLeakDetector<T> leakDetector = (ResourceLeakDetector)customClassConstructor.newInstance(new Object[] { resource, Integer.valueOf(samplingInterval), Long.valueOf(maxActive) });
        
        ResourceLeakDetectorFactory.access$000().debug("Loaded custom ResourceLeakDetector: {}", customClassConstructor.getDeclaringClass().getName());
        
        return leakDetector;
      }
      catch (Throwable t)
      {
        ResourceLeakDetectorFactory.access$000().error("Could not load custom resource leak detector provided: {} with the given resource: {}", new Object[] { customClassConstructor.getDeclaringClass().getName(), resource, t });
      }
    }
    ResourceLeakDetector<T> resourceLeakDetector = new ResourceLeakDetector(resource, samplingInterval, maxActive);
    
    ResourceLeakDetectorFactory.access$000().debug("Loaded default ResourceLeakDetector: {}", resourceLeakDetector);
    return resourceLeakDetector;
  }
}

/* Location:
 * Qualified Name:     io.netty.util.ResourceLeakDetectorFactory.DefaultResourceLeakDetectorFactory
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */