package io.netty.util.concurrent;

import io.netty.util.internal.PlatformDependent;
import java.util.Queue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

final class NonStickyEventExecutorGroup$NonStickyOrderedEventExecutor
  extends AbstractEventExecutor
  implements Runnable, OrderedEventExecutor
{
  private final EventExecutor executor;
  private final Queue<Runnable> tasks = PlatformDependent.newMpscQueue();
  private static final int NONE = 0;
  private static final int SUBMITTED = 1;
  private static final int RUNNING = 2;
  private final AtomicInteger state = new AtomicInteger();
  private final int maxTaskExecutePerRun;
  
  NonStickyEventExecutorGroup$NonStickyOrderedEventExecutor(EventExecutor executor, int maxTaskExecutePerRun)
  {
    super(executor);
    this.executor = executor;
    this.maxTaskExecutePerRun = maxTaskExecutePerRun;
  }
  
  public void run()
  {
    if (!state.compareAndSet(1, 2)) {
      return;
    }
    for (;;)
    {
      int i = 0;
      try
      {
        for (; i < maxTaskExecutePerRun; i++)
        {
          Runnable task = (Runnable)tasks.poll();
          if (task == null) {
            break;
          }
          safeExecute(task);
        }
      }
      finally
      {
        if (i == maxTaskExecutePerRun)
        {
          try
          {
            state.set(1);
            executor.execute(this);
            return;
          }
          catch (Throwable ignore)
          {
            state.set(2);
          }
        }
        else
        {
          state.set(0);
          return;
        }
      }
      throw ((Throwable)localObject);
    }
  }
  
  public boolean inEventLoop(Thread thread)
  {
    return false;
  }
  
  public boolean inEventLoop()
  {
    return false;
  }
  
  public boolean isShuttingDown()
  {
    return executor.isShutdown();
  }
  
  public Future<?> shutdownGracefully(long quietPeriod, long timeout, TimeUnit unit)
  {
    return executor.shutdownGracefully(quietPeriod, timeout, unit);
  }
  
  public Future<?> terminationFuture()
  {
    return executor.terminationFuture();
  }
  
  public void shutdown()
  {
    executor.shutdown();
  }
  
  public boolean isShutdown()
  {
    return executor.isShutdown();
  }
  
  public boolean isTerminated()
  {
    return executor.isTerminated();
  }
  
  public boolean awaitTermination(long timeout, TimeUnit unit)
    throws InterruptedException
  {
    return executor.awaitTermination(timeout, unit);
  }
  
  public void execute(Runnable command)
  {
    if (!tasks.offer(command)) {
      throw new RejectedExecutionException();
    }
    if (state.compareAndSet(0, 1)) {
      try
      {
        executor.execute(this);
      }
      catch (Throwable e)
      {
        tasks.remove(command);
        PlatformDependent.throwException(e);
      }
    }
  }
}

/* Location:
 * Qualified Name:     io.netty.util.concurrent.NonStickyEventExecutorGroup.NonStickyOrderedEventExecutor
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */