package io.netty.util.internal;

import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.shaded.org.jctools.queues.MpscChunkedArrayQueue;
import io.netty.util.internal.shaded.org.jctools.queues.atomic.MpscLinkedAtomicQueue;
import io.netty.util.internal.shaded.org.jctools.util.UnsafeAccess;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Queue;

final class PlatformDependent$Mpsc
{
  private static final boolean USE_MPSC_CHUNKED_ARRAY_QUEUE;
  
  static
  {
    Object unsafe = null;
    if (PlatformDependent.hasUnsafe()) {
      unsafe = AccessController.doPrivileged(new PrivilegedAction()
      {
        public Object run()
        {
          return UnsafeAccess.UNSAFE;
        }
      });
    }
    if (unsafe == null)
    {
      PlatformDependent.access$100().debug("org.jctools-core.MpscChunkedArrayQueue: unavailable");
      USE_MPSC_CHUNKED_ARRAY_QUEUE = false;
    }
    else
    {
      PlatformDependent.access$100().debug("org.jctools-core.MpscChunkedArrayQueue: available");
      USE_MPSC_CHUNKED_ARRAY_QUEUE = true;
    }
  }
  
  static <T> Queue<T> newMpscQueue(int maxCapacity)
  {
    if (USE_MPSC_CHUNKED_ARRAY_QUEUE)
    {
      int capacity = Math.max(Math.min(maxCapacity, 1073741824), 2048);
      
      return new MpscChunkedArrayQueue(1024, capacity, true);
    }
    return new MpscLinkedAtomicQueue();
  }
}

/* Location:
 * Qualified Name:     io.netty.util.internal.PlatformDependent.Mpsc
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */