package io.netty.util.internal;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
import sun.misc.Unsafe;

final class UnsafeAtomicReferenceFieldUpdater<U, M>
  extends AtomicReferenceFieldUpdater<U, M>
{
  private final long offset;
  private final Unsafe unsafe;
  
  UnsafeAtomicReferenceFieldUpdater(Unsafe unsafe, Class<? super U> tClass, String fieldName)
    throws NoSuchFieldException
  {
    Field field = tClass.getDeclaredField(fieldName);
    if (!Modifier.isVolatile(field.getModifiers())) {
      throw new IllegalArgumentException("Must be volatile");
    }
    this.unsafe = unsafe;
    offset = unsafe.objectFieldOffset(field);
  }
  
  public boolean compareAndSet(U obj, M expect, M update)
  {
    return unsafe.compareAndSwapObject(obj, offset, expect, update);
  }
  
  public boolean weakCompareAndSet(U obj, M expect, M update)
  {
    return unsafe.compareAndSwapObject(obj, offset, expect, update);
  }
  
  public void set(U obj, M newValue)
  {
    unsafe.putObjectVolatile(obj, offset, newValue);
  }
  
  public void lazySet(U obj, M newValue)
  {
    unsafe.putOrderedObject(obj, offset, newValue);
  }
  
  public M get(U obj)
  {
    return (M)unsafe.getObjectVolatile(obj, offset);
  }
}

/* Location:
 * Qualified Name:     io.netty.util.internal.UnsafeAtomicReferenceFieldUpdater
 * Java Class Version: 6 (50.0)
 * JD-Core Version:    0.7.1
 */