package org.apache.commons.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.NoSuchElementException;

public class LineIterator
{
  private final BufferedReader bufferedReader;
  private String cachedLine;
  private boolean finished = false;
  
  public LineIterator(Reader reader)
    throws IllegalArgumentException
  {
    if (reader == null) {
      throw new IllegalArgumentException("Reader must not be null");
    }
    if ((reader instanceof BufferedReader)) {
      bufferedReader = ((BufferedReader)reader);
    } else {
      bufferedReader = new BufferedReader(reader);
    }
  }
  
  public boolean hasNext()
  {
    if (cachedLine != null) {
      return true;
    }
    if (finished) {
      return false;
    }
    try
    {
      for (;;)
      {
        String line = bufferedReader.readLine();
        if (line == null)
        {
          finished = true;
          return false;
        }
        if (isValidLine(line))
        {
          cachedLine = line;
          return true;
        }
      }
    }
    catch (IOException ioe)
    {
      close();
      throw new IllegalStateException(ioe.toString());
    }
  }
  
  protected boolean isValidLine(String line)
  {
    return true;
  }
  
  public Object next()
  {
    return nextLine();
  }
  
  public String nextLine()
  {
    if (!hasNext()) {
      throw new NoSuchElementException("No more lines");
    }
    String currentLine = cachedLine;
    cachedLine = null;
    return currentLine;
  }
  
  public void close()
  {
    finished = true;
    IOUtils.closeQuietly(bufferedReader);
    cachedLine = null;
  }
  
  public void remove()
  {
    throw new UnsupportedOperationException("Remove unsupported on LineIterator");
  }
  
  public static void closeQuietly(LineIterator iterator)
  {
    if (iterator != null) {
      iterator.close();
    }
  }
}

/* Location:
 * Qualified Name:     org.apache.commons.io.LineIterator
 * Java Class Version: 1.1 (45.3)
 * JD-Core Version:    0.7.1
 */