package org.apache.commons.io.filefilter;

import java.io.File;
import java.util.List;
import org.apache.commons.io.FilenameUtils;

public class WildcardFilter
  extends AbstractFileFilter
{
  private String[] wildcards = null;
  
  public WildcardFilter(String wildcard)
  {
    if (wildcard == null) {
      throw new IllegalArgumentException();
    }
    wildcards = new String[] { wildcard };
  }
  
  public WildcardFilter(String[] wildcards)
  {
    if (wildcards == null) {
      throw new IllegalArgumentException();
    }
    this.wildcards = wildcards;
  }
  
  public WildcardFilter(List wildcards)
  {
    if (wildcards == null) {
      throw new IllegalArgumentException();
    }
    this.wildcards = ((String[])wildcards.toArray(new String[wildcards.size()]));
  }
  
  public boolean accept(File dir, String name)
  {
    if ((dir != null) && (new File(dir, name).isDirectory())) {
      return false;
    }
    for (int i = 0; i < wildcards.length; i++) {
      if (FilenameUtils.wildcardMatch(name, wildcards[i])) {
        return true;
      }
    }
    return false;
  }
  
  public boolean accept(File file)
  {
    if (file.isDirectory()) {
      return false;
    }
    for (int i = 0; i < wildcards.length; i++) {
      if (FilenameUtils.wildcardMatch(file.getName(), wildcards[i])) {
        return true;
      }
    }
    return false;
  }
}

/* Location:
 * Qualified Name:     org.apache.commons.io.filefilter.WildcardFilter
 * Java Class Version: 1.1 (45.3)
 * JD-Core Version:    0.7.1
 */