package org.apache.commons.io.input;

import java.io.IOException;
import java.io.InputStream;

public class CountingInputStream
  extends ProxyInputStream
{
  private int count;
  
  public CountingInputStream(InputStream in)
  {
    super(in);
  }
  
  public int read(byte[] b)
    throws IOException
  {
    int found = super.read(b);
    count += (found >= 0 ? found : 0);
    return found;
  }
  
  public int read(byte[] b, int off, int len)
    throws IOException
  {
    int found = super.read(b, off, len);
    count += (found >= 0 ? found : 0);
    return found;
  }
  
  public int read()
    throws IOException
  {
    int found = super.read();
    count += (found >= 0 ? 1 : 0);
    return found;
  }
  
  public long skip(long length)
    throws IOException
  {
    long skip = super.skip(length);
    count = ((int)(count + skip));
    return skip;
  }
  
  public int getCount()
  {
    return count;
  }
  
  public synchronized int resetCount()
  {
    int tmp = count;
    count = 0;
    return tmp;
  }
}

/* Location:
 * Qualified Name:     org.apache.commons.io.input.CountingInputStream
 * Java Class Version: 1.1 (45.3)
 * JD-Core Version:    0.7.1
 */